@@ -100,11 +100,12 @@ UnknownProtocol.prototype = {
 // 2 values should be stored.
 function imAccount(aKey, aName, aPrplId)
 {
-  if (aKey.indexOf(kAccountKeyPrefix) != 0)
+  if (!aKey.startsWith(kAccountKeyPrefix))
     throw Cr.NS_ERROR_INVALID_ARG;
 
   this.id = aKey;
   this.numericId = parseInt(aKey.substr(kAccountKeyPrefix.length));
+  gAccountsService._keepAccount(this);
   this.prefBranch = Services.prefs.getBranch(kPrefAccountPrefix + aKey + ".");
 
   if (aName) {
@@ -244,13 +245,40 @@ imAccount.prototype = {
         this._sendNotification("account-connect-error");
       }
     }
-    else if (aTopic == "account-disconnected")
+    else if (aTopic == "account-disconnected") {
       this.connectionState = Ci.imIAccount.STATE_DISCONNECTED;
+      if (this._statusObserver &&
+          this.prplAccount.connectionErrorReason == Ci.prplIAccount.NO_ERROR &&
+          this.statusInfo.statusType > Ci.imIStatusInfo.STATUS_OFFLINE) {
+        // If the status changed back to online while an account was still
+        // disconnecting, it was not reconnected automatically at that point,
+        // so we must do it now. (This happens for protocols like IRC where
+        // disconnection is not immediate.)
+        this._sendNotification(aTopic, aData);
+        this.connect();
+        return;
+      }
+    }
     else
       throw Cr.NS_ERROR_UNEXPECTED;
     this._sendNotification(aTopic, aData);
   },
 
+  _debugMessages: null,
+  logDebugMessage: function(aMessage, aLevel) {
+    if (!this._debugMessages)
+      this._debugMessages = [];
+    if (this._debugMessages.length >= 50)
+      this._debugMessages.shift();
+    this._debugMessages.push({logLevel: aLevel, message: aMessage});
+  },
+  getDebugMessages: function(aCount) {
+    let messages = this._debugMessages || [];
+    if (aCount)
+      aCount.value = messages.length;
+    return messages;
+  },
+
   _observedStatusInfo: null,
   get observedStatusInfo() this._observedStatusInfo,
   _statusObserver: null,
@@ -600,7 +628,8 @@ imAccount.prototype = {
             this._cancelReconnection();
           }
           else if (statusType > Ci.imIStatusInfo.STATUS_OFFLINE &&
-                   this.disconnected)
+                   this.disconnected &&
+                   this.connectionErrorReason == Ci.prplIAccount.NO_ERROR)
             this.prplAccount.connect();
           else if (this.connected)
             this.prplAccount.observe(aSubject, aTopic, aData);
@@ -725,9 +754,7 @@ AccountsService.prototype = {
         account.trim();
         if (!account)
           throw Cr.NS_ERROR_INVALID_ARG;
-        let newAccount = new imAccount(account);
-        this._accounts.push(newAccount);
-        this._accountsById[newAccount.numericId] = newAccount;
+        new imAccount(account);
       } catch (e) {
         Cu.reportError(e);
         dump(e + " " + e.toSource() + "\n");
@@ -752,7 +779,7 @@ AccountsService.prototype = {
 
     this._accounts =
       this._accountList.split(",").map(String.trim)
-          .filter(function (k) k.indexOf(kAccountKeyPrefix) == 0)
+          .filter(function (k) k.startsWith(kAccountKeyPrefix))
           .map(function (k) parseInt(k.substr(kAccountKeyPrefix.length)))
           .map(this.getAccountByNumericId, this)
           .filter(function (a) a);
@@ -893,13 +920,17 @@ AccountsService.prototype = {
   },
 
   getAccountById: function(aAccountId) {
-    if (aAccountId.indexOf(kAccountKeyPrefix) != 0)
+    if (!aAccountId.startsWith(kAccountKeyPrefix))
       throw Cr.NS_ERROR_INVALID_ARG;
 
     let id = parseInt(aAccountId.substr(kAccountKeyPrefix.length));
     return this.getAccountByNumericId(id);
   },
 
+  _keepAccount: function(aAccount) {
+    this._accounts.push(aAccount);
+    this._accountsById[aAccount.numericId] = aAccount;
+  },
   getAccountByNumericId: function(aAccountId) this._accountsById[aAccountId],
   getAccounts: function() new nsSimpleEnumerator(this._accounts),
 
@@ -931,10 +962,6 @@ AccountsService.prototype = {
     let key = kAccountKeyPrefix + id;
     let account = new imAccount(key, aName, aPrpl);
 
-    /* Keep it in the local account lists. */
-    this._accounts.push(account);
-    this._accountsById[id] = account;
-
     /* Save the account list pref. */
     let list = this._accountList;
     this._accountList = list ? list + "," + key : key;
