@@ -4,6 +4,7 @@
 
 Components.utils.import("resource:///modules/folderUtils.jsm");
 Components.utils.import("resource:///modules/iteratorUtils.jsm");
+Components.utils.import("resource:///modules/mailServices.js");
 Components.utils.import("resource:///modules/MailUtils.js");
 Components.utils.import("resource://gre/modules/Services.jsm");
 
@@ -173,9 +174,7 @@ let gFolderTreeView = {
     aTree.view = this;
 
     // Add this listener so that we can update the tree when things change
-    let session = Cc["@mozilla.org/messenger/services/session;1"]
-                     .getService(Ci.nsIMsgMailSession);
-    session.AddFolderListener(this, Ci.nsIFolderListener.all);
+    MailServices.mailSession.AddFolderListener(this, Ci.nsIFolderListener.all);
   },
 
   /**
@@ -187,9 +186,7 @@ let gFolderTreeView = {
     const Ci = Components.interfaces;
 
     // Remove our listener
-    let session = Cc["@mozilla.org/messenger/services/session;1"]
-                     .getService(Ci.nsIMsgMailSession);
-    session.RemoveFolderListener(this);
+    MailServices.mailSession.RemoveFolderListener(this);
 
     if (aJSONFile) {
       // Write out our json file...
@@ -599,8 +596,12 @@ let gFolderTreeView = {
 
     let dt = this._currentTransfer;
     let count = dt.mozItemCount;
-    let cs = Cc["@mozilla.org/messenger/messagecopyservice;1"]
-                .getService(Ci.nsIMsgCopyService);
+    let cs = MailServices.copy;
+
+    // This is a potential rss feed.  A link image as well as link text url
+    // should be handled; try to extract a url from non moz apps as well.
+    let feedUri = targetFolder.server.type == "rss" && count == 1 ?
+                    FeedUtils.getFeedUriFromDataTransfer(dt) : null;
 
     // we only support drag of a single flavor at a time.
     let types = dt.mozTypesAt(0);
@@ -662,28 +663,22 @@ let gFolderTreeView = {
       cs.CopyMessages(sourceFolder, array, targetFolder, isMove, null,
                         msgWindow, true);
     }
+    else if (feedUri) {
+      Cc["@mozilla.org/newsblog-feed-downloader;1"]
+         .getService(Ci.nsINewsBlogFeedDownloader)
+         .subscribeToFeed(feedUri.spec, targetFolder, msgWindow);
+    }
     else if (Array.indexOf(types, "application/x-moz-file") != -1) {
       for (let i = 0; i < count; i++) {
         let extFile = dt.mozGetDataAt("application/x-moz-file", i)
                         .QueryInterface(Ci.nsILocalFile);
         if (extFile.isFile()) {
           let len = extFile.leafName.length;
-          if (len > 4 && extFile.leafName.substr(len - 4).toLowerCase() == ".eml")
+          if (len > 4 && extFile.leafName.toLowerCase().endsWith(".eml"))
             cs.CopyFileMessage(extFile, targetFolder, null, false, 1, "", null, msgWindow);
         }
       }
     }
-
-    if (targetFolder.server.type == "rss" && count == 1) {
-      // This is a potential rss feed.  A link image as well as link text url
-      // should be handled; try to extract a url from non moz apps as well.
-      let validUri = FeedUtils.getFeedUriFromDataTransfer(dt);
-
-      if (validUri)
-        Cc["@mozilla.org/newsblog-feed-downloader;1"]
-           .getService(Ci.nsINewsBlogFeedDownloader)
-           .subscribeToFeed(validUri.spec, targetFolder, msgWindow);
-    }
   },
 
   _onDragStart: function ftv_dragStart(aEvent) {
@@ -713,8 +708,8 @@ let gFolderTreeView = {
    * CSS files will cue off of these.  Note that we reach into the rowMap's
    * items so that custom data-displays can define their own properties
    */
-  getCellProperties: function ftv_getCellProperties(aRow, aCol, aProps) {
-    this._rowMap[aRow].getProperties(aProps, aCol);
+  getCellProperties: function ftv_getCellProperties(aRow, aCol) {
+    return this._rowMap[aRow].getProperties(aCol);
   },
 
   /**
@@ -745,8 +740,8 @@ let gFolderTreeView = {
    * This is duplicative for our normal ftv views, but custom data-displays may
    * want to do something special here
    */
-  getRowProperties: function ftv_getRowProperties(aIndex, aProps) {
-    this._rowMap[aIndex].getProperties(aProps);
+  getRowProperties: function ftv_getRowProperties(aRow) {
+    return this._rowMap[aRow].getProperties();
   },
 
   /**
@@ -919,8 +914,8 @@ let gFolderTreeView = {
     for each (let acct in accounts) {
       let foldersWithFlag = acct.incomingServer.rootFolder.getFoldersWithFlags(aFolderFlag);
       if (foldersWithFlag.length > 0) {
-        for each (let folderWithFlag in fixIterator(foldersWithFlag.enumerate(),
-                                                Components.interfaces.nsIMsgFolder)) {
+        for (let folderWithFlag in fixIterator(foldersWithFlag,
+                                               Components.interfaces.nsIMsgFolder)) {
           folders.push(folderWithFlag);
           // Add sub-folders of Sent and Archive to the result.
           if (deep && (aFolderFlag & (nsMsgFolderFlags.SentMail | nsMsgFolderFlags.Archive)))
@@ -1056,9 +1051,7 @@ let gFolderTreeView = {
         vfdb.Close(true);
       }
       parentFolder.NotifyItemAdded(newFolder);
-      Components.classes["@mozilla.org/messenger/account-manager;1"]
-        .getService(Components.interfaces.nsIMsgAccountManager)
-        .saveVirtualFolders();
+      MailServices.accounts.saveVirtualFolders();
     }
     catch(e) {
        throw(e);
@@ -1075,7 +1068,7 @@ let gFolderTreeView = {
   setCellText: function ftv_setCellText(aRow, aCol, aValue) {},
   setCellValue: function ftv_setCellValue(aRow, aCol, aValue) {},
   getCellValue: function ftv_getCellValue(aRow, aCol) {},
-  getColumnProperties: function ftv_getColumnProperties(aCol, aProps) {},
+  getColumnProperties: function ftv_getColumnProperties(aCol) { return ""; },
   getImageSrc: function ftv_getImageSrc(aRow, aCol) {},
   getProgressMode: function ftv_getProgressMode(aRow, aCol) {},
   cycleCell: function ftv_cycleCell(aRow, aCol) {},
@@ -1278,7 +1271,7 @@ let gFolderTreeView = {
         }
         for each (let item in faves) {
           let name = item._folder.abbreviatedName.toLowerCase();
-          item.addServerName = (uniqueNames[name] > 1) ? true : false;
+          item.addServerName = (uniqueNames[name] > 1);
         }
         sortFolderItems(faves);
         return faves;
@@ -1374,19 +1367,16 @@ let gFolderTreeView = {
        * The smart server. This will create the server if it doesn't exist.
        */
       get _smartServer() {
-        let acctMgr = Components.classes["@mozilla.org/messenger/account-manager;1"]
-                                .getService(Components.interfaces.nsIMsgAccountManager);
         let smartServer;
         try {
-          smartServer = acctMgr.FindServer("nobody", "smart mailboxes", "none");
+          smartServer = MailServices.accounts.FindServer("nobody", "smart mailboxes", "none");
         }
         catch (ex) {
-          smartServer = acctMgr.createIncomingServer("nobody",
-                                                     "smart mailboxes", "none");
+          smartServer = MailServices.accounts.createIncomingServer("nobody", "smart mailboxes", "none");
           // We don't want the "smart" server/account leaking out into the ui in
           // other places, so set it as hidden.
           smartServer.hidden = true;
-          let account = acctMgr.createAccount();
+          let account = MailServices.accounts.createAccount();
           account.incomingServer = smartServer;
         }
         delete this._smartServer;
@@ -1409,17 +1399,11 @@ let gFolderTreeView = {
         [nsMsgFolderFlags.Junk, "Junk", false, true],
         [nsMsgFolderFlags.Queue, "Outbox", true, true]
       ],
-      // hard code, this could be build from _flagNameList dynamically
-      _smartFlags: nsMsgFolderFlags.Inbox | nsMsgFolderFlags.Drafts |
-                    nsMsgFolderFlags.Trash | nsMsgFolderFlags.SentMail |
-                    nsMsgFolderFlags.Templates |
-                    nsMsgFolderFlags.Junk |
-                    nsMsgFolderFlags.Archive,
 
       /**
        * support for addons to add special folder types, this must be called
        * prior to onload.
-       * 
+       *
        * @param aFolderName  name of the folder
        * @param isDeep  include subfolders
        * @param folderOptions  object with searchStr and searchOnline options, or null
@@ -1429,11 +1413,11 @@ let gFolderTreeView = {
       },
 
       /**
-       * Returns a triple describing the smart folder if the given folder is a
-       * special folder, else returns null.
+       * Returns an array of 4 elements describing the smart folder
+       * if the given folder is a special folder, else returns null.
        */
       getSmartFolderTypeByName: function ftv_smart__getSmartFolderType(aName) {
-        for each (let [, type,,] in Iterator(this._flagNameList)) {
+        for (let type of this._flagNameList) {
           if (type[1] == aName)
             return type;
         }
@@ -1443,7 +1427,7 @@ let gFolderTreeView = {
        * check to see if a folder is a smart folder
        */
       isSmartFolder: function ftv_smart__isSmartFolder(aFolder) {
-        if (aFolder.flags & this._smartFlags)
+        if (aFolder.flags & this._allSmartFlags)
             return true;
         // Also check the folder name itself, as containers do not
         // have the smartFolderName property.  We check all folders here, since
@@ -1456,9 +1440,9 @@ let gFolderTreeView = {
       /**
        * All the flags above, bitwise ORed.
        */
-      get _allFlags() {
-        delete this._allFlags;
-        return this._allFlags = this._flagNameList.reduce(
+      get _allSmartFlags() {
+        delete this._allSmartFlags;
+        return this._allSmartFlags = this._flagNameList.reduce(
           function (res, [flag,, isDeep,]) res | flag, 0);
       },
 
@@ -1472,12 +1456,12 @@ let gFolderTreeView = {
       },
 
       /**
-       * Returns a triple describing the smart folder if the given folder is a
-       * special folder, else returns null.
+       * Returns an array of 4 elements describing the smart folder
+       * if the given folder is a special folder, else returns null.
        */
       _getSmartFolderType: function ftv_smart__getSmartFolderType(aFolder) {
         let smartFolderName = getSmartFolderName(aFolder);
-        for each (let [, type] in Iterator(this._flagNameList)) {
+        for (let type of this._flagNameList) {
           if (smartFolderName) {
             if (type[1] == smartFolderName)
               return type;
@@ -1500,8 +1484,6 @@ let gFolderTreeView = {
 
       generateMap: function ftv_smart_generateMap(ftv) {
         let map = [];
-        let acctMgr = Components.classes["@mozilla.org/messenger/account-manager;1"]
-                                .getService(Components.interfaces.nsIMsgAccountManager);
         let accounts = gFolderTreeView._sortedAccounts();
         let smartServer = this._smartServer;
         smartServer.prettyName = document.getElementById("bundle_messenger")
@@ -1510,7 +1492,7 @@ let gFolderTreeView = {
 
         let smartRoot = smartServer.rootFolder;
         let smartChildren = [];
-        for each (let [, [flag, name,,]] in Iterator(this._flagNameList)) {
+        for (let [flag, name,,] of this._flagNameList) {
           gFolderTreeView._addSmartFoldersForFlag(smartChildren, accounts,
                                                   smartRoot, flag, name);
         }
@@ -1548,8 +1530,7 @@ let gFolderTreeView = {
         let smartType = this._getSmartFolderType(aFolder);
         if (smartType) {
           // This is a special folder
-          let [, name,] = smartType;
-          let smartFolder = this._getSmartFolderNamed(name);
+          let smartFolder = this._getSmartFolderNamed(smartType[1]);
           if (smartFolder &&
               gFolderTreeView.getIndexOfFolder(smartFolder) != null)
             return smartFolder;
@@ -1576,8 +1557,7 @@ let gFolderTreeView = {
 
         let smartType = this._getSmartFolderType(folder);
         if (smartType) {
-          let [, name,] = smartType;
-          let smartFolder = this._getSmartFolderNamed(name);
+          let smartFolder = this._getSmartFolderNamed(smartType[1]);
           if (smartFolder &&
               gFolderTreeView.getIndexOfFolder(smartFolder) != null)
             return smartFolder;
@@ -1595,16 +1575,15 @@ let gFolderTreeView = {
        * - Otherwise, we need to add it as a child of its parent (as normal).
        */
       onFolderAdded: function ftv_smart_onFolderAdded(aParent, aFolder) {
-        if (aFolder.flags & this._allFlags) {
+        if (aFolder.flags & this._allSmartFlags) {
           // add as child of corresponding smart folder
           let smartServer = this._smartServer;
           let smartRoot = smartServer.rootFolder;
           // In theory, a folder can have multiple flags set, so we need to
           // check each flag separately.
-          for each (let [, [flag, name,,]] in Iterator(this._flagNameList)) {
+          for (let [flag, name,,] of this._flagNameList) {
             if (aFolder.flags & flag)
-              gFolderTreeView._addSmartSubFolder(aFolder, smartRoot, name,
-                                                 flag);
+              gFolderTreeView._addSmartSubFolder(aFolder, smartRoot, name, flag);
           }
         }
         else if (aParent.isSpecialFolder(this._allShallowFlags, false)) {
@@ -1639,9 +1618,7 @@ let gFolderTreeView = {
     const Ci = Components.interfaces;
     let folders = [];
 
-    let acctMgr = Cc["@mozilla.org/messenger/account-manager;1"]
-                     .getService(Ci.nsIMsgAccountManager);
-    for each (let server in fixIterator(acctMgr.allServers, Ci.nsIMsgIncomingServer)) {
+    for each (let server in fixIterator(MailServices.accounts.allServers, Ci.nsIMsgIncomingServer)) {
       // Skip deferred accounts
       if (server instanceof Ci.nsIPop3IncomingServer &&
           server.deferredToAccount)
@@ -1908,7 +1885,7 @@ ftvItem.prototype = {
         text += " - " + this._folder.server.prettyName;
     }
     // Yeah, we hard-code this, but so did the old code...
-    let unread = this._folder.getNumUnread(false);
+    let unread = this._folder.getNumUnread(!this.open);
     if (unread > 0)
       text += " (" + unread + ")";
     return text;
@@ -1918,25 +1895,25 @@ ftvItem.prototype = {
     return this._level;
   },
 
-  getProperties: function ftvItem_getProperties(aProps) {
+  getProperties: function ftvItem_getProperties() {
     // From folderUtils.jsm
-    setPropertyAtoms(this._folder, aProps);
-    if (this._folder.flags & nsMsgFolderFlags.Virtual) {
-        aProps.AppendElement(Components.classes["@mozilla.org/atom-service;1"]
-                             .getService(Components.interfaces.nsIAtomService)
-                             .getAtom("specialFolder-Smart"));
-        // a second possibility for customized smart folders
-        aProps.AppendElement(Components.classes["@mozilla.org/atom-service;1"]
-                             .getService(Components.interfaces.nsIAtomService)
-                             .getAtom("specialFolder-"+this._folder.name.replace(' ','')));
+    let properties = getFolderProperties(this._folder);
+    if (this._folder.getFlag(nsMsgFolderFlags.Virtual)) {
+      properties += " specialFolder-Smart";
+      // a second possibility for customized smart folders
+      properties += " specialFolder-" + this._folder.name.replace(' ','');
     }
     // if there is a smartFolder name property, add it
     let smartFolderName = getSmartFolderName(this._folder);
     if (smartFolderName) {
-      aProps.AppendElement(Components.classes["@mozilla.org/atom-service;1"]
-                         .getService(Components.interfaces.nsIAtomService)
-                         .getAtom("specialFolder-"+smartFolderName.replace(' ','')));
+      properties += " specialFolder-" + smartFolderName.replace(' ','');
     }
+
+    if (this._folder.server.type == "rss" && !this._folder.isServer &&
+        FeedUtils.getFeedUrlsInFolder(this._folder))
+      properties += " isFeedFolder-true";
+
+    return properties;
   },
 
   command: function fti_command() {
@@ -2056,11 +2033,7 @@ let gFolderTreeController = {
       gFolderDisplay.view.close();
 
       // Send a notification that we are triggering a database rebuild.
-      let notifier =
-        Components.classes["@mozilla.org/messenger/msgnotificationservice;1"]
-                  .getService(
-                    Components.interfaces.nsIMsgFolderNotificationService);
-      notifier.notifyItemEvent(folder, "FolderReindexTriggered", null);
+      MailServices.mfn.notifyItemEvent(folder, "FolderReindexTriggered", null);
 
       folder.msgDatabase.summaryValid = false;
 
@@ -2297,11 +2270,9 @@ let gFolderTreeController = {
    * @param aCommand - the command to prompt for
    */
   _checkConfirmationPrompt: function ftc_confirm(aCommand) {
-    const Cc = Components.classes;
-    const Ci = Components.interfaces;
     let showPrompt = true;
     try {
-      showPrompt = !Services.prefs.getBoolPref("mail." + aCommand + ".dontAskAgain");
+      showPrompt = !Services.prefs.getBoolPref("mailnews." + aCommand + ".dontAskAgain");
     } catch (ex) {}
 
     if (showPrompt) {
@@ -2315,7 +2286,7 @@ let gFolderTreeController = {
                                          bundle.getString(aCommand + "DontAsk"),
                                          checkbox) == 0;
       if (checkbox.value)
-        Services.prefs.setBoolPref("mail." + aCommand + ".dontAskAgain", true);
+        Services.prefs.setBoolPref("mailnews." + aCommand + ".dontAskAgain", true);
       if (!ok)
         return false;
     }
