@@ -209,20 +209,35 @@ function MailToolboxCustomizeDone(aEvent, customizePopupId)
   if (this.UpdateMailToolbar != undefined)
     UpdateMailToolbar(focus);
 
-  var toolbox = document.getElementsByAttribute("doCustomization", "true")[0];
+  let toolbox = document.querySelector('[doCustomization="true"]');
   if (toolbox) {
     toolbox.removeAttribute("doCustomization");
 
     // The GetMail button is stuck in a strange state right now, since the
     // customization wrapping preserves its children, but not its initialized
     // state. Fix that here.
+    // That is also true for the File -> "Get new messages for" menuitems in both
+    // menus (old and new App menu).
+    // TODO: try to fix folderWidgets.xml to not do this.
+    // See Bug 520457 and Bug 534448.
     // Fix Bug 565045: Only treat "Get Message Button" if it is in our toolbox
-    var popup = toolbox.getElementsByAttribute("id", "button-getMsgPopup")[0];
-    if (popup) {
-      // We can't use _teardown here, because it'll remove the Get All menuitem
-      let sep = toolbox.getElementsByAttribute("id", "button-getAllNewMsgSeparator")[0];
-      while (popup.lastChild != sep)
-        popup.removeChild(popup.lastChild);
+    for (let popup of [ toolbox.querySelector("#button-getMsgPopup"),
+                        document.getElementById("menu_getAllNewMsgPopup"),
+                        document.getElementById("appmenu_getAllNewMsgPopup") ])
+    {
+      if (!popup)
+        continue;
+
+      // .teardown() is only available here if the menu has its frame
+      // otherwise the folderWidgets.xml::folder-menupopup binding is not
+      // attached to the popup. So if it is not available, remove the items
+      // explicitly, starting the the menuseparator element.
+      if ("_teardown" in popup) {
+        popup._teardown();
+      } else {
+        while (popup.lastChild.tagName != "menuseparator")
+          popup.removeChild(popup.lastChild);
+      }
     }
   }
 }
@@ -253,9 +268,7 @@ function onViewToolbarsPopupShowing(aEvent, toolboxIds, aInsertPoint)
 
     // We'll consider either childnodes that have a toolbarname attribute,
     // or externalToolbars.
-    let childToolbars = Array.slice(toolbox
-                                    .getElementsByAttribute("toolbarname",
-                                                            "*"));
+    let childToolbars = Array.slice(toolbox.querySelectorAll("[toolbarname]"));
     let potentialToolbars = childToolbars.concat(toolbox.externalToolbars);
 
     for (let [, toolbarElement] in Iterator(potentialToolbars)) {
