@@ -36,9 +36,7 @@ function OnMailWindowUnload()
   //  the tabs which close their views) and OnUnloadMessageWindow for the
   //  standalone message window.
 
-  var mailSession = Components.classes["@mozilla.org/messenger/services/session;1"]
-                              .getService(Components.interfaces.nsIMsgMailSession);
-  mailSession.RemoveMsgWindow(msgWindow);
+  MailServices.mailSession.RemoveMsgWindow(msgWindow);
   // the tabs have the FolderDisplayWidget close their 'messenger' instances for us
 
   window.QueryInterface(Components.interfaces.nsIDOMChromeWindow)
@@ -90,7 +88,7 @@ function CreateMailWindowGlobals()
   msgWindow = Components.classes["@mozilla.org/messenger/msgwindow;1"]
                         .createInstance(Components.interfaces.nsIMsgWindow);
 
-  accountManager = Components.classes["@mozilla.org/messenger/account-manager;1"].getService(Components.interfaces.nsIMsgAccountManager);
+  accountManager = MailServices.accounts;
 
   msgWindow.notificationCallbacks = new BadCertHandler();
 }
@@ -102,9 +100,7 @@ function InitMsgWindow()
   msgWindow.domWindow = window;
   msgWindow.statusFeedback = statusFeedback;
   msgWindow.msgHeaderSink = messageHeaderSink;
-  Components.classes["@mozilla.org/messenger/services/session;1"]
-            .getService(Components.interfaces.nsIMsgMailSession)
-            .AddMsgWindow(msgWindow);
+  MailServices.mailSession.AddMsgWindow(msgWindow);
   let messagepane = document.getElementById("messagepane");
   messagepane.docShell.allowAuth = false;
   messagepane.docShell.allowDNSPrefetch = false;
@@ -164,13 +160,6 @@ nsMsgStatusFeedback.prototype =
       this.showStatusString(status);
   },
 
-  setJSDefaultStatus: function(status) {
-    if (status.length > 0) {
-      this._defaultStatusText = status;
-      this._statusText.label = status;
-    }
-  },
-
   setOverLink: function(link, context) {
     this._statusText.label = link;
   },
@@ -200,6 +189,13 @@ nsMsgStatusFeedback.prototype =
     this._statusText.label = statusText;
   },
 
+  setStatusString: function(status) {
+    if (status.length > 0) {
+      this._defaultStatusText = status;
+      this._statusText.label = status;
+    }
+  },
+
   _startMeteors: function() {
     this._meteorsSpinning = true;
     this._startTimeoutID = null;
@@ -233,7 +229,7 @@ nsMsgStatusFeedback.prototype =
   },
 
   _stopMeteors: function() {
-    this.showStatusString(defaultStatus);
+    this.showStatusString(this._defaultStatusText);
 
     // stop the throbber
     if (this._throbber)
@@ -440,10 +436,7 @@ function loadStartPage(aForce)
   if (startpage)
   {
     try {
-      let urifixup = Components.classes["@mozilla.org/docshell/urifixup;1"]
-                               .getService(Components.interfaces.nsIURIFixup);
-
-      let uri = urifixup.createFixupURI(startpage, 0);
+      let uri = Services.uriFixup.createFixupURI(startpage, 0);
       GetMessagePaneFrame().location.href = uri.spec;
     }
     catch (e) {
