@@ -262,6 +262,7 @@ StandaloneMessageDisplayWidget.prototype = {
       window.close();
       return true;
     }
+    return false;
   },
 };
 
@@ -357,6 +358,8 @@ function delayedOnLoadMessageWindow()
     onEndHeaders: function() {
       if (gMessageDisplay.isDummy)
         gMessageDisplay.onDisplayingMessage(messageHeaderSink.dummyMsgHeader);
+      if (!FeedMessageHandler.shouldShowSummary(gMessageDisplay.displayedMessage, false))
+        FeedMessageHandler.setContent(gMessageDisplay.displayedMessage, false);
       UpdateMailToolbar(".eml/message from attachment finished loading");
     },
     onEndAttachments: function () {},
@@ -570,6 +573,10 @@ function HideMenus()
   if (viewLayoutMenu)
     viewLayoutMenu.setAttribute("hidden", "true");
 
+  let paneViewSeparator = document.getElementById("appmenu_paneViewSeparator");
+  if (paneViewSeparator)
+    paneViewSeparator.setAttribute("hidden", "true");
+
   var viewFolderMenu = document.getElementById("menu_FolderViews");
   if (viewFolderMenu)
     viewFolderMenu.setAttribute("hidden", "true");
@@ -614,7 +621,7 @@ function HideMenus()
   if (viewSortMenuSeparator)
     viewSortMenuSeparator.setAttribute("hidden", "true");
 
-  viewSortMenuSeparator = document.getElementById('appmenu_viewSortMenuSeparator');
+  viewSortMenuSeparator = document.getElementById('appmenu_viewAfterThreadsSeparator');
   if (viewSortMenuSeparator)
     viewSortMenuSeparator.setAttribute("hidden", "true");
 
@@ -653,13 +660,13 @@ function HideMenus()
   var favoriteFolder = document.getElementById('menu_favoriteFolder');
   if (favoriteFolder)
   {
-    favoriteFolder.disabled = true;
+    favoriteFolder.setAttribute("disabled", "true");
     favoriteFolder.setAttribute("hidden", "true");
   }
 
   favoriteFolder = document.getElementById('appmenu_favoriteFolder');
   if (favoriteFolder) {
-    favoriteFolder.disabled = true;
+    favoriteFolder.setAttribute("disabled", "true");
     favoriteFolder.setAttribute("hidden", "true");
   }
 
@@ -675,6 +682,14 @@ function HideMenus()
   if (trashSeparator)
     trashSeparator.setAttribute("hidden", "true");
 
+  let fileMenuAfterRenameSeparator = document.getElementById('appmenu_fileMenuAfterRenameSeparator');
+  if (fileMenuAfterRenameSeparator)
+    fileMenuAfterRenameSeparator.setAttribute("hidden", "true");
+
+  let fileMenuAfterCompactSeparator = document.getElementById('appmenu_fileMenuAfterCompactSeparator');
+  if (fileMenuAfterCompactSeparator)
+    fileMenuAfterCompactSeparator.setAttribute("hidden", "true");
+
   var goStartPageSeparator = document.getElementById('goNextSeparator');
   if (goStartPageSeparator)
     goStartPageSeparator.hidden = true;
@@ -683,6 +698,14 @@ function HideMenus()
   if (goStartPageSeparator)
     goStartPageSeparator.hidden = true;
 
+  let goRecentlyClosedTabsSeparator = document.getElementById('goRecentlyClosedTabsSeparator');
+  if (goRecentlyClosedTabsSeparator)
+    goRecentlyClosedTabsSeparator.setAttribute("hidden", "true");
+
+  goRecentlyClosedTabsSeparator = document.getElementById('appmenu_goRecentlyClosedTabsSeparator');
+  if (goRecentlyClosedTabsSeparator)
+    goRecentlyClosedTabsSeparator.setAttribute("hidden", "true");
+
   var goStartPage = document.getElementById('goStartPage');
   if (goStartPage)
    goStartPage.hidden = true;
@@ -691,6 +714,10 @@ function HideMenus()
   if (goStartPage)
    goStartPage.hidden = true;
 
+  let quickFilterBar = document.getElementById('appmenu_quickFilterBar');
+  if (quickFilterBar)
+   quickFilterBar.hidden = true;
+
   var menuFileClose = document.getElementById('menu_close');
   var menuFileQuit = document.getElementById('menu_FileQuitItem');
   if (menuFileClose && menuFileQuit)
