@@ -3,17 +3,18 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-Components.utils.import("resource:///modules/folderUtils.jsm");
 Components.utils.import("resource:///modules/activity/activityModules.js");
+Components.utils.import("resource:///modules/errUtils.js");
+Components.utils.import("resource:///modules/folderUtils.jsm");
+Components.utils.import("resource:///modules/IOUtils.js");
 Components.utils.import("resource:///modules/jsTreeSelection.js");
 Components.utils.import("resource:///modules/MailConsts.js");
-Components.utils.import("resource:///modules/errUtils.js");
-Components.utils.import("resource:///modules/IOUtils.js");
+Components.utils.import("resource:///modules/mailInstrumentation.js");
 Components.utils.import("resource:///modules/mailnewsMigrator.js");
+Components.utils.import("resource:///modules/mailServices.js");
+Components.utils.import("resource:///modules/msgDBCacheManager.js");
 Components.utils.import("resource:///modules/sessionStoreManager.js");
 Components.utils.import("resource:///modules/summaryFrameManager.js");
-Components.utils.import("resource:///modules/mailInstrumentation.js");
-Components.utils.import("resource:///modules/msgDBCacheManager.js");
 Components.utils.import("resource://gre/modules/Services.jsm");
 
 /* This is where functions related to the 3 pane window are kept */
@@ -455,7 +456,7 @@ function LoadPostAccountWizard()
     if (typeof arg0 == "string")
     {
       // filter our any feed urls that came in as arguments to the new window...
-      if (/^feed:/i.test(arg0))
+      if (arg0.toLowerCase().startsWith("feed:"))
       {
         let feedHandler = Components.classes["@mozilla.org/newsblog-feed-downloader;1"]
           .getService(Components.interfaces.nsINewsBlogFeedDownloader);
@@ -604,9 +605,7 @@ function OnUnloadMessenger()
 
   webSearchTabType.shutdown();
 
-  var mailSession = Components.classes["@mozilla.org/messenger/services/session;1"]
-                              .getService(Components.interfaces.nsIMsgMailSession);
-  mailSession.RemoveFolderListener(folderListener);
+  MailServices.mailSession.RemoveFolderListener(folderListener);
 
   gPhishingDetector.shutdown();
 
@@ -860,11 +859,9 @@ function loadStartFolder(initialUri)
 
 function AddToSession()
 {
-  var mailSession = Components.classes["@mozilla.org/messenger/services/session;1"]
-                              .getService(Components.interfaces.nsIMsgMailSession);
   var nsIFolderListener = Components.interfaces.nsIFolderListener;
   var notifyFlags = nsIFolderListener.intPropertyChanged | nsIFolderListener.event;
-  mailSession.AddFolderListener(folderListener, notifyFlags);
+  MailServices.mailSession.AddFolderListener(folderListener, notifyFlags);
 }
 
 function InitPanes()
@@ -1302,25 +1299,34 @@ function ThreadPaneOnDragStart(aEvent) {
     return;
 
   gFolderDisplay.hintAboutToDeleteMessages();
-  let fileNames = [];
+  let fileNames = new Set();
   let msgUrls = {};
 
-  // dragging multiple messages to desktop does not
-  // currently work, pending core fixes for
-  // multiple-drop-on-desktop support. (bug 513464)
+  // Dragging multiple messages to desktop does not currently work.
+  // When core fixes for multiple-drop-on-desktop support
+  // (e.g. bug 513464, bug 270292) are landed, generating of
+  // "application/x-moz-file-promise" values for i > 0 can be enabled.
+  // But first ensure suggestUniqueFileName is efficient enough on 10000+ dragged
+  // messages.
   for (let i in messages) {
     messenger.messageServiceFromURI(messages[i])
              .GetUrlForUri(messages[i], msgUrls, null);
-    var subject = messenger.messageServiceFromURI(messages[i])
-                           .messageURIToMsgHdr(messages[i]).mime2DecodedSubject;
-    var uniqueFileName = suggestUniqueFileName(subject.substr(0,124), ".eml",
-                                               fileNames);
-    fileNames[i] = uniqueFileName;
     aEvent.dataTransfer.mozSetDataAt("text/x-moz-message", messages[i], i);
     aEvent.dataTransfer.mozSetDataAt("text/x-moz-url",msgUrls.value.spec, i);
+
+    if (i > 0)
+      continue;
+
+    // Generate file name in case the object is dropped onto the desktop.
+    let subject = messenger.messageServiceFromURI(messages[i])
+                           .messageURIToMsgHdr(messages[i]).mime2DecodedSubject;
+    let uniqueFileName = suggestUniqueFileName(subject.substr(0, 124), ".eml",
+                                               fileNames);
+    fileNames.add(uniqueFileName);
+
     aEvent.dataTransfer.mozSetDataAt("application/x-moz-file-promise-url",
-                                     msgUrls.value.spec + "?fileName=" + uniqueFileName,
-                                     i);
+                                     msgUrls.value.spec + "?fileName=" +
+                                     uniqueFileName, i);
     aEvent.dataTransfer.mozSetDataAt("application/x-moz-file-promise", null, i);
   }
   aEvent.dataTransfer.effectAllowed = "copyMove";
@@ -1328,29 +1334,30 @@ function ThreadPaneOnDragStart(aEvent) {
 }
 
 /**
- * example use:
- *   suggestUniqueFileName("testname",".txt",["testname", "testname1"])
- *   returns "testname2"
- * does not check file system for existing files
- * @param existingNames array of names in use
+ * Returns a new filename that is guaranteed to not be in the Set
+ * of existing names.
+ *
+ * Example use:
+ *   suggestUniqueFileName("testname", ".txt", Set("testname", "testname1"))
+ *   returns "testname2.txt"
+ * Does not check file system for existing files.
+ *
+ * @param aIdentifier     proposed filename
+ * @param aType           extension
+ * @param aExistingNames  a Set of names already in use
  */
-function suggestUniqueFileName(identifier, type, existingNames) {
+function suggestUniqueFileName(aIdentifier, aType, aExistingNames) {
   let suffix = 1;
-  let suggestion;
-  let base = identifier;
-  let exists;
-  do {
-    exists = false;
-    suggestion = GenerateValidFilename(base, type);
-    for (let i = 0; i < existingNames.length; i++) {
-      if (existingNames[i] == suggestion) {
-        base = identifier + suffix;
-        suffix++;
-        exists = true;
-        break;
-      }
-    }
-  } while (exists);
+  let base = validateFileName(aIdentifier);
+  let suggestion = base + aType;
+  while(true) {
+    if (!aExistingNames.has(suggestion))
+      break;
+
+    suggestion = base + suffix + aType;
+    suffix++;
+  }
+
   return suggestion;
 }
 
@@ -1368,7 +1375,7 @@ function ThreadPaneOnDragOver(aEvent) {
                     .QueryInterface(Components.interfaces.nsIFile);
     if (extFile.isFile()) {
       let len = extFile.leafName.length;
-      if (len > 4 && extFile.leafName.substr(len - 4).toLowerCase() == ".eml")
+      if (len > 4 && extFile.leafName.toLowerCase().endsWith(".eml"))
         ds.canDrop = true;
     }
   }
@@ -1376,16 +1383,14 @@ function ThreadPaneOnDragOver(aEvent) {
 
 function ThreadPaneOnDrop(aEvent) {
   let dt = aEvent.dataTransfer;
-  let cs = Components.classes["@mozilla.org/messenger/messagecopyservice;1"]
-                     .getService(Components.interfaces.nsIMsgCopyService);
   for (let i = 0; i < dt.mozItemCount; i++) {
     let extFile = dt.mozGetDataAt("application/x-moz-file", i)
                     .QueryInterface(Components.interfaces.nsIFile);
     if (extFile.isFile()) {
       let len = extFile.leafName.length;
-      if (len > 4 && extFile.leafName.substr(len - 4).toLowerCase() == ".eml")
-        cs.CopyFileMessage(extFile, gFolderDisplay.displayedFolder, null, false,
-                           1, "", null, msgWindow);
+      if (len > 4 && extFile.leafName.toLowerCase().endsWith(".eml"))
+        MailServices.copy.CopyFileMessage(extFile, gFolderDisplay.displayedFolder,
+                                          null, false, 1, "", null, msgWindow);
     }
   }
 }
