@@ -42,6 +42,7 @@ function nsContextMenu(aXulMenu, aIsShift) {
   // Message Related Items
   this.inMessageArea = false;
   this.inThreadPane = false;
+  this.inStandaloneWindow = false;
   this.messagepaneIsBlank = false;
   this.numSelectedMessages = 0;
   this.isNewsgroup = false;
@@ -249,9 +250,12 @@ nsContextMenu.prototype = {
         "mailContext-archive", "mailContext-replySender",
         "mailContext-editAsNew", "mailContext-replyNewsgroup",
         "mailContext-replyAll", "mailContext-replyList",
-        "mailContext-forward", "mailContext-forwardAsAttachment",
+        "mailContext-forward", "mailContext-forwardAsMenu",
+        "mailContext-multiForwardAsAttachment",
         "mailContext-copyMessageUrl", "mailContext-moveMenu",
         "mailContext-copyMenu", "mailContext-moveToFolderAgain",
+        "mailContext-ignoreThread", "mailContext-ignoreSubthread",
+        "mailContext-watchThread",
         "mailContext-tags", "mailContext-mark", "mailContext-saveAs",
         "mailContext-printpreview", "mailContext-print", "mailContext-delete",
         "downloadSelected", "mailContext-reportPhishingURL"
@@ -280,8 +284,9 @@ nsContextMenu.prototype = {
     this.setSingleSelection("mailContext-replyAll");
     this.setSingleSelection("mailContext-replyList");
     this.setSingleSelection("mailContext-forward");
+    this.setSingleSelection("mailContext-forwardAsMenu");
 
-    this.showItem("mailContext-forwardAsAttachment",
+    this.showItem("mailContext-multiForwardAsAttachment",
                   this.numSelectedMessages > 1 && this.inThreadPane &&
                   !this.hideMailItems);
 
@@ -317,6 +322,23 @@ nsContextMenu.prototype = {
 
     this.showItem("mailContext-mark", msgModifyItems);
 
+    this.showItem("mailContext-ignoreThread", !this.inStandaloneWindow &&
+                                              this.numSelectedMessages >= 1 &&
+                                              !this.hideMailItems &&
+                                              !this.onPlayableMedia);
+
+    this.showItem("mailContext-ignoreSubthread", !this.inStandaloneWindow &&
+                                                 this.numSelectedMessages >= 1 &&
+                                                 !this.hideMailItems &&
+                                                 !this.onPlayableMedia);
+
+    this.showItem("mailContext-watchThread", !this.inStandaloneWindow &&
+                                             this.numSelectedMessages > 0 &&
+                                             !this.hideMailItems &&
+                                             !this.onPlayableMedia);
+
+    this.showItem("mailContext-afterWatchThread", !this.inStandaloneWindow);
+
     this.showItem("mailContext-saveAs", this.numSelectedMessages > 0 &&
                                         !this.hideMailItems &&
                                         !gMessageDisplay.isDummy &&
@@ -516,11 +538,14 @@ nsContextMenu.prototype = {
       // mailTabType's list of modes, then we'll assume it is a message related
       // tab.
       this.inMessageArea = tabmail.selectedTab.mode.name in mailTabType.modes;
+      this.inStandaloneWindow = false;
     }
-    else
+    else {
       // Assume that if we haven't got a tabmail item, then we're in standalone
       // window
       this.inMessageArea = true;
+      this.inStandaloneWindow = true;
+    }
 
     if (!this.inMessageArea) {
       this.inThreadPane = false;
@@ -723,7 +748,7 @@ nsContextMenu.prototype = {
       return this.link.href;
     }
     var href = this.link.getAttributeNS("http://www.w3.org/1999/xlink","href");
-    if (!href || !href.match(/\S/)) {
+    if (!href || (href.trim() == "")) {
        // Without this we try to save as the current doc,
        // for example, HTML case also throws if empty.
       throw "Empty href";
@@ -762,16 +787,16 @@ nsContextMenu.prototype = {
    */
   linkText : function CM_linkText() {
     var text = gatherTextUnder(this.link);
-    if (!text || !text.match(/\S/)) {
+    if (!text || (text.trim() == "")) {
       text = this.link.getAttribute("title");
-      if (!text || !text.match(/\S/)) {
+      if (!text || (text.trim() == "")) {
         text = this.link.getAttribute("alt");
-        if (!text || !text.match(/\S/)) {
+        if (!text || (text.trim() == "")) {
           if (this.link.href) {
             text = this.link.href;
           } else {
             text = getAttributeNS("http://www.w3.org/1999/xlink", "href");
-            if (text && text.match(/\S/)) {
+            if (text && !(text.trim() == "")) {
               text = this.makeURLAbsolute(this.link.baseURI, text);
             }
           }
@@ -888,10 +913,8 @@ nsContextMenu.prototype = {
   },
 
   openInBrowser: function CM_openInBrowser() {
-    let uri = Components.classes["@mozilla.org/network/io-service;1"]
-                        .getService(Components.interfaces.nsIIOService)
-                        .newURI(this.target.ownerDocument.defaultView.
-                                top.location.href, null, null);
+    let uri = Services.io.newURI(this.target.ownerDocument.defaultView.
+                                 top.location.href, null, null);
 
     Components.classes["@mozilla.org/uriloader/external-protocol-service;1"]
               .getService(Components.interfaces.nsIExternalProtocolService)
