@@ -95,11 +95,10 @@ var gPhishingDetector = {
       this.analyzeUrl(linkNodes[index].href, gatherTextUnder(linkNodes[index]));
 
     // extract the action urls associated with any form elements in the message and analyze them.
-    var formNodes = document.getElementById('messagepane').contentDocument.getElementsByTagName("form");
+    let formNodes = document.getElementById('messagepane').contentDocument.querySelectorAll("form[action]");
     for (index = 0; index < formNodes.length; index++)
     {
-      if (formNodes[index].action)
-        this.analyzeUrl(formNodes[index].action);
+      this.analyzeUrl(formNodes[index].action);
     }
   },
 
@@ -271,9 +270,10 @@ var gPhishingDetector = {
                         [brandShortName, unobscuredHostNameValue], 2);
 
       const nsIPS = Components.interfaces.nsIPromptService;
-      var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(nsIPS);
-      return !promptService.confirmEx(window, titleMsg, dialogMsg, nsIPS.STD_YES_NO_BUTTONS + nsIPS.BUTTON_POS_1_DEFAULT, 
-                                     "", "", "", "", {}); /* the yes button is in position 0 */
+      return !Services.prompt.confirmEx(window, titleMsg, dialogMsg,
+                                        nsIPS.STD_YES_NO_BUTTONS +
+                                        nsIPS.BUTTON_POS_1_DEFAULT,
+                                        "", "", "", "", {}); /* the yes button is in position 0 */
     }
 
     return true; // allow the link to load
