@@ -6,13 +6,14 @@ Components.utils.import("resource://gre/modules/DownloadUtils.jsm");
 Components.utils.import("resource://gre/modules/PluralForm.jsm");
 Components.utils.import("resource:///modules/gloda/connotent.js");
 Components.utils.import("resource:///modules/gloda/mimemsg.js");
+Components.utils.import("resource:///modules/mailServices.js");
 Components.utils.import("resource:///modules/templateUtils.js");
 
 let gSelectionSummaryStrings = {
   NConversations: "NConversations",
   numMsgs: "numMsgs",
   countUnread: "countUnread",
-  Nmessages: "Nmessages",
+  ignoredCount: "ignoredCount",
   messagesSize: "messagesSize",
   noticeText: "noticeText",
   noSubject: "noSubject",
@@ -40,24 +41,19 @@ function loadSelectionSummaryStrings() {
 // Ah, wouldn't it be nice if there was platform code to do the following...
 
 /**
- * the equivalent of jQuery's addClass.  Avoids duplicates, nothing fancy.
+ * Adds a classes to a node. Avoids duplicates, nothing fancy.
  *
  * @param node
  *        any old DOM node
- * @param classname
- *        a string, which will be added as a CSS class
+ * @param classArray
+ *        an array of strings, which will be added as CSS classes
  */
-function _mm_addClass(node, classname) {
-  let classes = [];
-  if (node.hasAttribute('class'))
-    classes = node.getAttribute('class').split(' ');
-
-  for each (let [, klass] in Iterator(classes)) {
-    if (klass == classname) // already have it
-      return;
+function _mm_addClass(node, classArray) {
+  for (let i = 0; i < classArray.length; i++)
+  {
+    if (!node.classList.contains(classArray[i]))
+      node.classList.add(classArray[i]);
   }
-  classes.push(classname);
-  node.setAttribute('class', classes.join(' '));
 }
 
 /**
@@ -70,15 +66,8 @@ function _mm_addClass(node, classname) {
  *        a string, which will be removed from the class set.
  */
 function _mm_removeClass(node, classname) {
-  if (! node.hasAttribute('class'))
-    return;
-  let classes = node.getAttribute('class').split(' ');
-  let newclasses = [];
-  for each (klass in classes) {
-    if (klass != classname)
-      newclasses.push(klass);
-  }
-  node.setAttribute('class', newclasses.join(' '));
+  if (node.classList.contains(classname))
+    node.classList.remove(classname);
 }
 
 /**
@@ -109,6 +98,12 @@ function _mm_FormatDisplayName(aHeaderParser, aHeaderValue, aContext)
   }
 }
 
+function _mm_escapeHTML(aUnescaped) {
+  return aUnescaped.replace('&', "&amp;", 'g')
+                   .replace('<', "&lt;", 'g')
+                   .replace('>', "&gt;", 'g');
+}
+
 /**
  * the MultiMessageSummary class is responsible for populating the message pane
  * with a reasonable summary of a set of messages that span more than one
@@ -139,8 +134,7 @@ function MultiMessageSummary(aMessages, aListener) {
 
 MultiMessageSummary.prototype = {
   init: function() {
-    this._msgTagService = Components.classes["@mozilla.org/messenger/tagservice;1"].
-                          getService(Components.interfaces.nsIMsgTagService);
+    this._msgTagService = MailServices.tags;
     this._glodaQueries = [];
     this._msgNodes = {};
     if (this._listener)
@@ -186,8 +180,8 @@ MultiMessageSummary.prototype = {
    *     name without quotes
    **/
   stripQuotes: function(senderName) {
-    if ((senderName[0] == "'" && senderName[senderName.length-1] == "'") ||
-        (senderName[0] == '"' && senderName[senderName.length-1] == '"'))
+    if ((senderName.startsWith("'") && senderName.endsWith("'")) ||
+        (senderName.startsWith('"') && senderName.endsWith('"')))
       senderName = senderName.slice(1, -1);
     return senderName;
   },
@@ -201,20 +195,18 @@ MultiMessageSummary.prototype = {
    * Fill in the summary pane describing the selected messages
    **/
   _onLoad: function() {
-    let htmlpane = document.getElementById('multimessage');
-    // First, we group the messages in threads.
-    // count threads
-    let threads = {};
-    let numThreads = 0;
-    let headerParser = Components.classes["@mozilla.org/messenger/headerparser;1"].
-                         getService(Components.interfaces.nsIMsgHeaderParser);
-    let viewThreadId = function (aMsgHdr) {
+    function viewThreadId(aMsgHdr) {
       let thread = gDBView.getThreadContainingMsgHdr(aMsgHdr);
       return thread.threadKey;
-    };
+    }
+
+    let htmlpane = document.getElementById('multimessage');
+    // First, we group the messages in threads and count the threads.
+    let threads = {};
+    let numThreads = 0;
     for (let [,msgHdr] in Iterator(this._msgHdrs))
     {
-      if (! threads[viewThreadId(msgHdr)]) {
+      if (!threads[viewThreadId(msgHdr)]) {
         threads[viewThreadId(msgHdr)] = [msgHdr];
         numThreads += 1;
       } else {
@@ -224,8 +216,8 @@ MultiMessageSummary.prototype = {
 
     // set the heading based on the number of messages & threads
     let heading = htmlpane.contentDocument.getElementById('heading');
-    _mm_addClass(heading, "heading");
-    _mm_addClass(heading, "info");
+    _mm_addClass(heading, ["heading"]);
+    _mm_addClass(heading, ["info"]);
 
     let messagesTitle =
       PluralForm.get(numThreads, gSelectionSummaryStrings["NConversations"])
@@ -260,23 +252,23 @@ MultiMessageSummary.prototype = {
 
       // we'll mark the thread unread if any messages in it are unread
       for (let [, msgHdr] in Iterator(msgs)) {
-        if (! msgHdr.isRead)
-          countUnread += 1;
+        if (!msgHdr.isRead)
+          countUnread++;
         if (msgHdr.isFlagged)
-          countStarred += 1;
+          countStarred++;
       }
 
       let numMsgs = msgs.length;
-      let msg_classes = "message ";
+      let msg_classes = ["message"];
       if (numMsgs > 1)
-        msg_classes += " thread";
+        msg_classes.push("thread");
       if (countUnread)
-        msg_classes += " unread";
+        msg_classes.push("unread");
       if (countStarred)
-        msg_classes += " starred";
+        msg_classes.push("starred");
 
       let subject = msgs[0].mime2DecodedSubject || gSelectionSummaryStrings['noSubject'];
-      let author = _mm_FormatDisplayName(headerParser, msgs[0].mime2DecodedAuthor, "from");
+      let author = _mm_FormatDisplayName(MailServices.headerParser, msgs[0].mime2DecodedAuthor, "from");
 
       let countstring = "";
       if (numMsgs > 1) {
@@ -287,28 +279,30 @@ MultiMessageSummary.prototype = {
         countstring += ")";
       }
 
-      let msgContents = <div class="row">
-                          <div class="star"/>
-                          <div class="header">
-                            <div class="wrappedsubject">
-                              <div class="author">{author}</div>
-                              <div class="subject link">{subject}</div>
-                              <div class="count">{countstring}</div>
-                              <div class="tags"></div>
-                            </div>
-                            <div class="snippet"></div>
-                          </div>
-                        </div>;
+      let msgContents = '<div class="row">' +
+                        '  <div class="star"/>' +
+                        '  <div class="header">' +
+                        '    <div class="wrappedsubject">' +
+                        '      <div class="author">' +
+                                  _mm_escapeHTML(author) + '</div>' +
+                        '      <div class="subject link">' +
+                                  _mm_escapeHTML(subject) + '</div>' +
+                        '      <div class="count">' + countstring + '</div>' +
+                        '      <div class="tags"></div>' +
+                        '    </div>' +
+                        '    <div class="snippet"></div>' +
+                        '  </div>' +
+                        '</div>';
 
       let msgNode = htmlpane.contentDocument.createElement("div");
       // innerHTML is safe here because all of the data in msgContents is
       // either generated from integers or escaped to be safe.
-      msgNode.innerHTML = msgContents.toXMLString();
+      msgNode.innerHTML = msgContents;
       _mm_addClass(msgNode, msg_classes);
       messagesElt.appendChild(msgNode);
 
-      let snippetNode = msgNode.getElementsByClassName("snippet")[0];
-      let authorNode = msgNode.getElementsByClassName("author")[0];
+      let snippetNode = msgNode.querySelector(".snippet");
+      let authorNode = msgNode.querySelector(".author");
       try {
         MsgHdrToMimeMessage(msgs[0], null, function(aMsgHdr, aMimeMsg) {
           if (aMimeMsg == null) /* shouldn't happen, but sometimes does? */
@@ -328,13 +322,13 @@ MultiMessageSummary.prototype = {
       }
 
       // get the subject node.
-      let subjectNode = msgNode.getElementsByClassName("subject")[0];
+      let subjectNode = msgNode.querySelector(".subject");
       subjectNode.msgs = msgs;
       subjectNode.addEventListener("click", function() {
         gFolderDisplay.selectMessages(this.msgs);
       }, true);
 
-      let tagsNode = msgNode.getElementsByClassName("tags")[0];
+      let tagsNode = msgNode.querySelector(".tags");
       while (tagsNode.firstChild)
         tagsNode.removeChild(tagsNode.firstChild);
       this._addTagNodes(msgs, tagsNode);
@@ -370,7 +364,7 @@ MultiMessageSummary.prototype = {
         let tagNode = tagsNode.ownerDocument.createElement('span');
         // see tagColors.css
         let colorClass = "blc-" + this._msgTagService.getColorForKey(tag.key).substr(1);
-        _mm_addClass(tagNode, "tag " + tag.tag + " " + colorClass);
+        _mm_addClass(tagNode, ["tag", tag.tag, colorClass]);
         tagNode.textContent = tag.tag;
         tagsNode.appendChild(tagNode);
       }
@@ -404,7 +398,7 @@ MultiMessageSummary.prototype = {
       notice.textContent = noticeText;
       _mm_removeClass(notice, 'hidden');
     } else {
-      _mm_addClass(notice, 'hidden');
+      _mm_addClass(notice, ['hidden']);
     }
   },
 
@@ -451,7 +445,7 @@ MultiMessageSummary.prototype = {
       // for tags, there's a minor problem in that if _some_ of the items in a
       // thread got modified
       let key = messageKey + glodaMsg.folder.uri;
-      let tagsNode = headerNode.getElementsByClassName('tags')[0];
+      let tagsNode = headerNode.querySelector('.tags');
       while (tagsNode.firstChild)
         tagsNode.removeChild(tagsNode.firstChild);
       this._addTagNodes([msg.folderMessage for each ([i,msg] in Iterator(aItems))],
@@ -460,11 +454,11 @@ MultiMessageSummary.prototype = {
 
     for ([, headerNode] in Iterator(knownMessageNodes)) {
       if (headerNode.flags['unread'])
-        _mm_addClass(headerNode, "unread");
+        _mm_addClass(headerNode, ["unread"]);
       else
         _mm_removeClass(headerNode, "unread");
       if (headerNode.flags['starred'])
-        _mm_addClass(headerNode, "starred");
+        _mm_addClass(headerNode, ["starred"]);
       else
         _mm_removeClass(headerNode, "starred");
       headerNode.flags = null;
@@ -520,12 +514,6 @@ ThreadSummary.prototype = {
 
     let firstMsgHdr = this._msgHdrs[0];
     let numMessages = this._msgHdrs.length;
-    let subject = (firstMsgHdr.mime2DecodedSubject || gSelectionSummaryStrings["noSubject"])
-       + " "
-       + PluralForm.get(numMessages, gSelectionSummaryStrings["Nmessages"]).replace('#1', numMessages);
-    let heading = htmlpane.contentDocument.getElementById('heading');
-    heading.setAttribute("class", "heading");
-    heading.textContent = subject;
 
     // enable/disable the archive button as appropriate
     let archiveBtn = htmlpane.contentDocument.getElementById('hdrArchiveButton');
@@ -535,51 +523,57 @@ ThreadSummary.prototype = {
     while (messagesElt.firstChild)
       messagesElt.removeChild(messagesElt.firstChild);
 
-    let headerParser = Components.classes["@mozilla.org/messenger/headerparser;1"]
-                                    .getService(Components.interfaces.nsIMsgHeaderParser);
     let count = 0;
+    let ignoredCount = 0;
     const MAX_THREADS = 100;
     const SNIPPET_LENGTH = 300;
     let maxCountExceeded = false;
     for (let i = 0; i < numMessages; ++i) {
-      count += 1;
+      let msgHdr = this._msgHdrs[i];
+
+      if (msgHdr.isKilled) { // ignored subthread...
+        ignoredCount++;
+        continue;
+      }
+
+      count++;
       if (count > MAX_THREADS) {
         maxCountExceeded = true;
         break;
       }
-      let msgHdr = this._msgHdrs[i];
 
-      let msg_classes = "message ";
-      if (! msgHdr.isRead)
-        msg_classes += " unread";
+      let msg_classes = ["message"];
+      if (!msgHdr.isRead)
+        msg_classes.push("unread");
       if (msgHdr.isFlagged)
-        msg_classes += " starred";
+        msg_classes.push("starred");
 
-      let senderName = _mm_FormatDisplayName(headerParser, msgHdr.mime2DecodedAuthor, "from");
+      let senderName = _mm_FormatDisplayName(MailServices.headerParser, msgHdr.mime2DecodedAuthor, "from");
       let date = makeFriendlyDateAgo(new Date(msgHdr.date/1000));
 
-      let msgContents = <div class="row">
-                          <div class="star"/>
-                          <div class="header">
-                            <div class="wrappedsender">
-                              <div class="sender link">{senderName}</div>
-                              <div class="date">{date}</div>
-                              <div class="tags"></div>
-                            </div>
-                            <div class="snippet"></div>
-                          </div>
-                        </div>;
+      let msgContents = '<div class="row">' +
+                        '  <div class="star"/>' +
+                        '  <div class="header">' +
+                        '    <div class="wrappedsender">' +
+                        '      <div class="sender link">' +
+                                 _mm_escapeHTML(senderName) + '</div>' +
+                        '      <div class="date">' + date + '</div>' +
+                        '      <div class="tags"></div>' +
+                        '    </div>' +
+                        '    <div class="snippet"></div>' +
+                        '  </div>' +
+                        '</div>';
 
       let msgNode = htmlpane.contentDocument.createElement("div");
       // innerHTML is safe here because all of the data in msgContents is
       // either generated from integers or escaped to be safe.
-      msgNode.innerHTML = msgContents.toXMLString();
+      msgNode.innerHTML = msgContents;
       _mm_addClass(msgNode, msg_classes);
       messagesElt.appendChild(msgNode);
 
       let key = msgHdr.messageKey + msgHdr.folder.URI;
-      let snippetNode = msgNode.getElementsByClassName("snippet")[0];
-      let senderNode = msgNode.getElementsByClassName("sender")[0];
+      let snippetNode = msgNode.querySelector(".snippet");
+      let senderNode = msgNode.querySelector(".sender");
       try {
         MsgHdrToMimeMessage(msgHdr, null, function(aMsgHdr, aMimeMsg) {
           if (aMimeMsg == null) /* shouldn't happen, but sometimes does? */ {
@@ -597,18 +591,18 @@ ThreadSummary.prototype = {
         // that's fixed, this code should adapt. XXX
         snippetNode.textContent = "...";
       }
-      let tagsNode = msgNode.getElementsByClassName("tags")[0];
+      let tagsNode = msgNode.querySelector(".tags");
       let tags = this.getTagsForMsg(msgHdr);
       for each (let [,tag] in Iterator(tags)) {
         let tagNode = tagsNode.ownerDocument.createElement('span');
         // see tagColors.css
         let colorClass = "blc-" + this._msgTagService.getColorForKey(tag.key).substr(1);
-        _mm_addClass(tagNode, "tag " + tag.tag + " " + colorClass);
+        _mm_addClass(tagNode, ["tag", tag.tag, colorClass]);
         tagNode.textContent = tag.tag;
         tagsNode.appendChild(tagNode);
       }
 
-      let sender = msgNode.getElementsByClassName("sender")[0];
+      let sender = msgNode.querySelector(".sender");
       sender.msgHdr = msgHdr;
       sender.addEventListener("click", function(e) {
         // if the msg is the first message in a collapsed thread, we need to
@@ -626,6 +620,23 @@ ThreadSummary.prototype = {
 
       messagesElt.appendChild(msgNode);
     }
+
+    let countInfo =
+      PluralForm.get(numMessages, gSelectionSummaryStrings["numMsgs"])
+                .replace("#1", numMessages);
+    if (ignoredCount != 0) {
+      countInfo += " - " +
+        PluralForm.get(ignoredCount, gSelectionSummaryStrings["ignoredCount"])
+                  .replace("#1", ignoredCount);
+    }
+
+    let subject = (firstMsgHdr.mime2DecodedSubject ||
+                   gSelectionSummaryStrings["noSubject"]) +
+                  " (" + countInfo + ")";
+    let heading = htmlpane.contentDocument.getElementById("heading");
+    heading.setAttribute("class", "heading");
+    heading.textContent = subject;
+
     // stash somewhere so it doesn't get GC'ed
     this._glodaQueries.push(Gloda.getMessageCollectionForHeaders(this._msgHdrs, this));
     this.notifyMaxCountExceeded(htmlpane.contentDocument, numMessages, MAX_THREADS);
