@@ -3,6 +3,9 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+Components.utils.import("resource://gre/modules/Services.jsm");
+Components.utils.import("resource:///modules/mailServices.js");
+
 top.MAX_RECIPIENTS = 1; /* for the initial listitem created in the XUL */
 
 var inputElementType = "";
@@ -13,7 +16,6 @@ var gNumberOfCols = 0;
 
 var gDragService = Components.classes["@mozilla.org/widget/dragservice;1"].getService();
 gDragService = gDragService.QueryInterface(Components.interfaces.nsIDragService);
-var gMimeHeaderParser = null;
 
 var test_addresses_sequence = false;
 
@@ -42,6 +44,32 @@ function awGetNumberOfCols()
   return gNumberOfCols;
 }
 
+/**
+ * Adjust the default and minimum number of visible recipient rows for addressingWidget
+ */
+function awInitializeNumberOfRowsShown()
+{
+  let msgHeadersToolbar = document.getElementById("MsgHeadersToolbar");
+  let addressingWidget = document.getElementById("addressingWidget");
+  let awNumRowsShownDefault =
+    Services.prefs.getIntPref("mail.compose.addresswidget.numRowsShownDefault");
+
+  // Set minimum number of rows shown for address widget, per hardwired
+  // rows="1" attribute of addressingWidget, to prevent resizing the
+  // subject and format toolbar over the address widget.
+  // This lets users shrink the address widget to one row (with delicate UX)
+  // and thus maximize the space available for composition body,
+  // especially on small screens.
+  msgHeadersToolbar.minHeight = msgHeadersToolbar.boxObject.height;
+
+  // Set default number of rows shown for address widget.
+  addressingWidget.setAttribute("rows", awNumRowsShownDefault);
+  msgHeadersToolbar.height = msgHeadersToolbar.boxObject.height;
+
+  // Update addressingWidget internals.
+  awCreateOrRemoveDummyRows();
+}
+
 function awInputElementName()
 {
   if (inputElementType == "")
@@ -93,8 +121,6 @@ function Recipients2CompFields(msgCompFields)
     var ng_Sep = "";
     var follow_Sep = "";
 
-    gMimeHeaderParser = Components.classes["@mozilla.org/messenger/headerparser;1"].getService(Components.interfaces.nsIMsgHeaderParser);
-
     var recipientType;
     var inputField;
     var fieldValue;
@@ -118,7 +144,7 @@ function Recipients2CompFields(msgCompFields)
           case "addr_bcc"   :
           case "addr_reply" :
             try {
-              recipient = gMimeHeaderParser.reformatUnquotedAddresses(fieldValue);
+              recipient = MailServices.headerParser.reformatUnquotedAddresses(fieldValue);
             } catch (ex) {recipient = fieldValue;}
             break;
         }
@@ -146,8 +172,6 @@ function Recipients2CompFields(msgCompFields)
     msgCompFields.newsgroups = addrNg;
     msgCompFields.followupTo = addrFollow;
     msgCompFields.otherRandomHeaders = addrOther;
-
-    gMimeHeaderParser = null;
   }
   else
     dump("Message Compose Error: msgCompFields is null (ExtractRecipients)");
@@ -156,51 +180,49 @@ function Recipients2CompFields(msgCompFields)
 function CompFields2Recipients(msgCompFields)
 {
   if (msgCompFields) {
-    gMimeHeaderParser = Components.classes["@mozilla.org/messenger/headerparser;1"].getService(Components.interfaces.nsIMsgHeaderParser);
-
-    var listbox = document.getElementById('addressingWidget');
-    var newListBoxNode = listbox.cloneNode(false);
-    var listBoxColsClone = listbox.firstChild.cloneNode(true);
+    let listbox = document.getElementById('addressingWidget');
+    let newListBoxNode = listbox.cloneNode(false);
+    let listBoxColsClone = listbox.firstChild.cloneNode(true);
     newListBoxNode.appendChild(listBoxColsClone);
-    var templateNode = listbox.getElementsByTagName("listitem")[0];
+    let templateNode = listbox.querySelector("listitem");
     // dump("replacing child in comp fields 2 recips \n");
     listbox.parentNode.replaceChild(newListBoxNode, listbox);
 
     top.MAX_RECIPIENTS = 0;
-    var msgReplyTo = msgCompFields.replyTo;
-    var msgTo = msgCompFields.to;
-    var msgCC = msgCompFields.cc;
-    var msgBCC = msgCompFields.bcc;
-    var msgRandomHeaders = msgCompFields.otherRandomHeaders;
-    var msgNewsgroups = msgCompFields.newsgroups;
-    var msgFollowupTo = msgCompFields.followupTo;
-    var havePrimaryRecipient = false;
-    if(msgReplyTo)
+    let msgReplyTo = msgCompFields.replyTo;
+    let msgTo = msgCompFields.to;
+    let msgCC = msgCompFields.cc;
+    let msgBCC = msgCompFields.bcc;
+    let msgRandomHeaders = msgCompFields.otherRandomHeaders;
+    let msgNewsgroups = msgCompFields.newsgroups;
+    let msgFollowupTo = msgCompFields.followupTo;
+    let havePrimaryRecipient = false;
+    if (msgReplyTo)
       awSetInputAndPopupFromArray(msgCompFields.splitRecipients(msgReplyTo, false, {}),
                                   "addr_reply", newListBoxNode, templateNode);
-    if(msgTo)
+    if (msgTo)
     {
-      var rcp = msgCompFields.splitRecipients(msgTo, false, {});
+      let rcp = msgCompFields.splitRecipients(msgTo, false, {});
       if (rcp.length)
       {
         awSetInputAndPopupFromArray(rcp, "addr_to", newListBoxNode, templateNode);
         havePrimaryRecipient = true;
       }
     }
-    if(msgCC)
+    if (msgCC)
       awSetInputAndPopupFromArray(msgCompFields.splitRecipients(msgCC, false, {}),
                                   "addr_cc", newListBoxNode, templateNode);
-    if(msgBCC)
+    if (msgBCC)
       awSetInputAndPopupFromArray(msgCompFields.splitRecipients(msgBCC, false, {}),
                                   "addr_bcc", newListBoxNode, templateNode);
-    if(msgRandomHeaders)
+    if (msgRandomHeaders)
       awSetInputAndPopup(msgRandomHeaders, "addr_other", newListBoxNode, templateNode);
-    if(msgNewsgroups)
+    if (msgNewsgroups)
     {
       awSetInputAndPopup(msgNewsgroups, "addr_newsgroups", newListBoxNode, templateNode);
       havePrimaryRecipient = true;
     }
-    if(msgFollowupTo)
+    if (msgFollowupTo)
       awSetInputAndPopup(msgFollowupTo, "addr_followup", newListBoxNode, templateNode);
 
     // If it's a new message, we need to add an extra empty recipient.
@@ -211,8 +233,6 @@ function CompFields2Recipients(msgCompFields)
     // CompFields2Recipients is called whenever a user replies or edits an existing message. We want to
     // add all of the recipients for this message to the ignore list for spell check
     addRecipientsToIgnoreList((gCurrentIdentity ? gCurrentIdentity.identityName + ', ' : '') + msgTo + ', ' + msgCC + ', ' + msgBCC);
-
-    gMimeHeaderParser = null; //Release the mime parser
   }
 }
 
@@ -225,9 +245,7 @@ function awSetInputAndPopupId(inputElem, popupElem, rowNumber)
 
 function awSetInputAndPopupValue(inputElem, inputValue, popupElem, popupValue, rowNumber)
 {
-  // remove leading spaces
-  while (inputValue && inputValue[0] == " " )
-    inputValue = inputValue.substring(1, inputValue.length);
+  inputValue = inputValue.trimLeft();
 
   inputElem.setAttribute("value", inputValue);
   inputElem.value = inputValue;
@@ -273,11 +291,10 @@ function awSetInputAndPopupFromArray(inputArray, popupValue, parentNode, templat
     for (var index = 0; index < inputArray.length; index++)
     {
       recipient = null;
-      if (gMimeHeaderParser)
-        try {
-          recipient =
-            gMimeHeaderParser.unquotePhraseOrAddrWString(inputArray[index], true);
-        } catch (ex) {};
+      try {
+        recipient =
+          MailServices.headerParser.unquotePhraseOrAddrWString(inputArray[index], true);
+      } catch (ex) {};
       if (!recipient)
         recipient = inputArray[index];
       _awSetInputAndPopup(recipient, popupValue, parentNode, templateNode);
@@ -355,20 +372,20 @@ function awTestRowSequence()
     You need to define the pref mail.debug.test_addresses_sequence to true in order to activate it
   */
 
-  if (! test_addresses_sequence)
+  if (!test_addresses_sequence)
     return true;
 
   /* debug code to verify the sequence still good */
 
-  var listbox = document.getElementById('addressingWidget');
-  var listitems = listbox.getElementsByTagName('listitem');
+  let listbox = document.getElementById('addressingWidget');
+  let listitems = listbox.getElementsByTagName('listitem');
   if (listitems.length >= top.MAX_RECIPIENTS )
   {
-    for (var i = 1; i <= listitems.length; i ++)
+    for (let i = 1; i <= listitems.length; i ++)
     {
-      var item = listitems [i - 1];
-      var inputID = item.getElementsByTagName(awInputElementName())[0].getAttribute("id").split("#")[1];
-      var popupID = item.getElementsByTagName(awSelectElementName())[0].getAttribute("id").split("#")[1];
+      let item = listitems [i - 1];
+      let inputID = item.querySelector(awInputElementName()).id.split("#")[1];
+      let popupID = item.querySelector(awSelectElementName()).id.split("#")[1];
       if (inputID != i || popupID != i)
       {
         dump("#ERROR: sequence broken at row " + i + ", inputID=" + inputID + ", popupID=" + popupID + "\n");
@@ -836,7 +853,7 @@ function awRecipientKeyPress(event, element)
   case KeyEvent.DOM_VK_RETURN:
   case KeyEvent.DOM_VK_TAB:
     // if the user text contains a comma or a line return, ignore
-    if (element.value.search(',') != -1)
+    if (element.value.contains(','))
     {
       var addresses = element.value;
       element.value = ""; // clear out the current line so we don't try to autocomplete it..
@@ -919,21 +936,19 @@ function awFitDummyRows()
 
 function awCreateOrRemoveDummyRows()
 {
-  var listbox = document.getElementById("addressingWidget");
-  var listboxHeight = listbox.boxObject.height;
+  let listbox = document.getElementById("addressingWidget");
+  let listboxHeight = listbox.boxObject.height;
 
   // remove rows to remove scrollbar
-  var kids = listbox.childNodes;
-  for (var i = kids.length-1; gAWContentHeight > listboxHeight && i >= 0; --i) {
-    if (kids[i].hasAttribute("_isDummyRow")) {
-      gAWContentHeight -= gAWRowHeight;
-      listbox.removeChild(kids[i]);
-    }
+  let kids = listbox.querySelectorAll('[_isDummyRow]');
+  for (let i = kids.length - 1; gAWContentHeight > listboxHeight && i >= 0; --i) {
+    gAWContentHeight -= gAWRowHeight;
+    listbox.removeChild(kids[i]);
   }
 
   // add rows to fill space
   if (gAWRowHeight) {
-    while (gAWContentHeight+gAWRowHeight < listboxHeight) {
+    while (gAWContentHeight + gAWRowHeight < listboxHeight) {
       awCreateDummyItem(listbox);
       gAWContentHeight += gAWRowHeight;
     }
@@ -986,13 +1001,7 @@ function awCreateDummyCell(aParent)
 function awGetNextDummyRow()
 {
   // gets the next row from the top down
-  var listbox = document.getElementById("addressingWidget");
-  var kids = listbox.childNodes;
-  for (var i = 0; i < kids.length; ++i) {
-    if (kids[i].hasAttribute("_isDummyRow"))
-      return kids[i];
-  }
-  return null;
+  return document.querySelector('#addressingWidget > [_isDummyRow]');
 }
 
 function awSizerListen()
@@ -1018,7 +1027,7 @@ function awDocumentKeyPress(event)
 {
   try {
     var id = event.target.id;
-    if (id.substr(0, 11) == 'addressCol1')
+    if (id.startsWith('addressCol1'))
       awMenulistKeyPress(event, event.target);
   } catch (e) { }
 }
@@ -1040,11 +1049,10 @@ function parseAndAddAddresses(addressText, recipientType)
   // strip any leading >> characters inserted by the autocomplete widget
   var strippedAddresses = addressText.replace(/.* >> /, "");
 
-  var hdrParser = Components.classes["@mozilla.org/messenger/headerparser;1"].getService(Components.interfaces.nsIMsgHeaderParser);
   var addresses = {};
   var names = {};
   var fullNames = {};
-  var numAddresses = hdrParser.parseHeadersWithArray(strippedAddresses, addresses, names, fullNames);
+  let numAddresses = MailServices.headerParser.parseHeadersWithArray(strippedAddresses, addresses, names, fullNames);
 
   if (numAddresses > 0)
   {
@@ -1102,7 +1110,7 @@ AutomatedAutoCompleteHandler.prototype =
     {
       /* XXX This is used to work, until switching to the new toolkit broke it
          We should fix it see bug 456550.
-      if (this.namesToComplete[this.indexIntoNames].search('@') == -1) // don't autocomplete if address has an @ sign in it
+      if (!this.namesToComplete[this.indexIntoNames].contains('@')) // don't autocomplete if address has an @ sign in it
       {
         // make sure total session count is updated before we kick off ANY actual searches
         if (gAutocompleteSession)
