@@ -10,10 +10,14 @@ var gGeneralPane = {
   init: function ()
   {
     this.mPane = document.getElementById("paneGeneral");
-    
+
+    this.updateStartPage();
+
     this.updatePlaySound();
+
+    this.updateCustomizeAlert();
   },
-    
+
   /**
    * Restores the default start page as the user's start page
    */
@@ -86,7 +90,7 @@ var gGeneralPane = {
     soundLocation = document.getElementById('soundType').value == 1 ?
                     document.getElementById('soundUrlLocation').value : "_moz_mailbeep"
 
-    if (soundLocation.indexOf("file://") == -1)
+    if (!soundLocation.contains("file://"))
       sound.playSystemSound(soundLocation);
     else
       sound.play(Services.io.newURI(soundLocation, null, null));
@@ -134,5 +138,17 @@ var gGeneralPane = {
     soundTypeEl.disabled = soundsDisabled;
     document.getElementById('browseForSound').disabled = soundsDisabled || soundTypeEl.value != 1;
     document.getElementById('playSound').disabled = soundsDisabled || soundTypeEl.value != 1; 
+  },
+
+  updateStartPage: function()
+  {
+    document.getElementById("mailnewsStartPageUrl").disabled =
+      !document.getElementById("mailnewsStartPageEnabled").checked;
+  },
+
+  updateCustomizeAlert: function()
+  {
+    document.getElementById("customizeMailAlert").disabled =
+      !document.getElementById("newMailNotificationAlert").checked;
   }
 };
