@@ -44,12 +44,11 @@ var gPermissionManager = {
     getImageSrc: function(aRow, aColumn) {},
     getProgressMode: function(aRow, aColumn) {},
     getCellValue: function(aRow, aColumn) {},
-    cycleHeader: function(column) {},
-    getRowProperties: function(row,prop){},
-    getColumnProperties: function(column,prop){},
-    getCellProperties: function(row,column,prop){
-      if (column.element.getAttribute("id") == "siteCol")
-        prop.AppendElement(this._ltrAtom);
+    cycleHeader: function(aColumn) {},
+    getRowProperties: function(aRow) { return ""; },
+    getColumnProperties: function(aColumn) { return ""; },
+    getCellProperties: function(aRow, aColumn) {
+      return (aColumn.element.getAttribute("id") == "siteCol") ? "ltr" : "";
     }
   },
 
@@ -99,8 +98,8 @@ var gPermissionManager = {
     }
 
     if (!exists) {
-      host = (host.charAt(0) == ".") ? host.substring(1,host.length) : host;
-      var uri = Services.io.newURI("http://" + host, null, null);
+      host = host.startsWith(".") ? host.substr(1) : host;
+      let uri = Services.io.newURI("http://" + host, null, null);
       Services.perms.add(uri, this._type, aCapability);
     }
     textbox.value = "";
@@ -170,10 +169,6 @@ var gPermissionManager = {
     this._loadPermissions();
 
     urlField.focus();
-
-    this._ltrAtom = Components.classes["@mozilla.org/atom-service;1"]
-                              .getService(Components.interfaces.nsIAtomService)
-                              .getAtom("ltr");
   },
 
   uninit: function ()
@@ -306,7 +301,7 @@ var gPermissionManager = {
       var host = aPermission.host;
       var capabilityString = this._getCapabilityString(aPermission.capability);
       var p = new Permission(host,
-                             (host.charAt(0) == ".") ? host.substring(1,host.length) : host,
+                             (host.startsWith(".")) ? host.substring(1, host.length) : host,
                              aPermission.type,
                              capabilityString,
                              aPermission.capability);
