@@ -59,9 +59,7 @@ function forward_selected_messages_and_go_to_drafts_folder(f) {
   // actually quite the window
   wait_for_window_close();
 
-  let acctMgr = Cc["@mozilla.org/messenger/account-manager;1"]
-                  .getService(Ci.nsIMsgAccountManager);
-  let draftsFolder = acctMgr.localFoldersServer.rootFolder.getChildNamed("Drafts");
+  let draftsFolder = MailServices.accounts.localFoldersServer.rootFolder.getChildNamed("Drafts");
   be_in_folder(draftsFolder);
 }
 
