@@ -23,6 +23,7 @@ Cu.import("resource://mozmill/modules/jum.js", jumlib);
 
 Cu.import("resource:///modules/IOUtils.js");
 Cu.import("resource:///modules/sessionStoreManager.js");
+Cu.import("resource://gre/modules/Services.jsm");
 
 // the windowHelper module
 var windowHelper;
@@ -54,22 +55,21 @@ function waitForFileRefresh() {
                 "file should exist");
 }
 
-function open3PaneWindow(windowWatcher) {
+function open3PaneWindow() {
   windowHelper.plan_for_new_window("mail:3pane");
-  windowWatcher.openWindow(null,
-                           "chrome://messenger/content/messenger.xul", "",
-                           "all,chrome,dialog=no,status,toolbar",
-                           null);
+  Services.ww.openWindow(null,
+                         "chrome://messenger/content/messenger.xul", "",
+                         "all,chrome,dialog=no,status,toolbar",
+                         null);
   return windowHelper.wait_for_new_window("mail:3pane");
 }
 
-function openAddressBook(windowWatcher) {
+function openAddressBook() {
   windowHelper.plan_for_new_window("mail:addressbook");
-  windowWatcher.openWindow(
-                      null,
-                      "chrome://messenger/content/addressbook/addressbook.xul", "",
-                      "all,chrome,dialog=no,status,toolbar",
-                      null);
+  Services.ww.openWindow(null,
+                         "chrome://messenger/content/addressbook/addressbook.xul", "",
+                         "all,chrome,dialog=no,status,toolbar",
+                         null);
   return windowHelper.wait_for_new_window("mail:addressbook");
 }
 
@@ -144,9 +144,7 @@ function test_single_3pane_periodic_session_persistence() {
 
   // get the state object. this assumes there is one and only one
   // 3pane window.
-  let windowMediator = Cc["@mozilla.org/appshell/window-mediator;1"].
-                       getService(Ci.nsIWindowMediator);
-  let mail3PaneWindow = windowMediator.getMostRecentWindow("mail:3pane");
+  let mail3PaneWindow = Services.wm.getMostRecentWindow("mail:3pane");
   let state = mail3PaneWindow.getWindowStateForSessionPersistence();
 
   sessionStoreManager.startPeriodicSave();
@@ -169,20 +167,16 @@ function test_restore_single_3pane_persistence() {
 
   // get the state object. this assumes there is one and only one
   // 3pane window.
-  let windowWatcher = Cc["@mozilla.org/embedcomp/window-watcher;1"].
-                      getService(Ci.nsIWindowWatcher);
-  let windowMediator = Cc["@mozilla.org/appshell/window-mediator;1"].
-                       getService(Ci.nsIWindowMediator);
-  let mail3PaneWindow = windowMediator.getMostRecentWindow("mail:3pane");
+  let mail3PaneWindow = Services.wm.getMostRecentWindow("mail:3pane");
 
   // make sure we have a different window open, so that we don't start shutting
   // down just because the last window was closed
-  let abwc = openAddressBook(windowWatcher);
+  let abwc = openAddressBook();
 
   // close the 3pane window
   mail3PaneWindow.close();
 
-  mc = open3PaneWindow(windowWatcher);
+  mc = open3PaneWindow();
   be_in_folder(folderA);
   assert_message_pane_hidden();
   // restore message pane.
@@ -207,11 +201,7 @@ function test_message_pane_height_persistence() {
 
   // Get the state object. This assumes there is one and only one
   // 3pane window.
-  let windowWatcher = Cc["@mozilla.org/embedcomp/window-watcher;1"].
-                      getService(Ci.nsIWindowWatcher);
-  let windowMediator = Cc["@mozilla.org/appshell/window-mediator;1"].
-                       getService(Ci.nsIWindowMediator);
-  let mail3PaneWindow = windowMediator.getMostRecentWindow("mail:3pane");
+  let mail3PaneWindow = Services.wm.getMostRecentWindow("mail:3pane");
 
   let oldHeight = mc.e("messagepaneboxwrapper").boxObject.height;
   let minHeight = Math.floor(mc.e("messagepaneboxwrapper").getAttribute("minheight"));
@@ -227,23 +217,27 @@ function test_message_pane_height_persistence() {
 
   // Check that the moving of the threadpane-splitter resulted in the correct height.
   let actualHeight = mc.e("messagepaneboxwrapper").boxObject.height;
-  assert_equals(newHeight, actualHeight, "The message pane height should be " +
-    newHeight + ", but is actually " + actualHeight);
+
+  assert_equals(newHeight, actualHeight,
+    "The message pane height should be " + newHeight + ", but is actually " +
+    actualHeight + ". The oldHeight was: " + oldHeight);
 
   // Make sure we have a different window open, so that we don't start shutting
   // down just because the last window was closed.
-  let abwc = openAddressBook(windowWatcher);
+  let abwc = openAddressBook();
 
   // The 3pane window is closed.
   mail3PaneWindow.close();
 
-  mc = open3PaneWindow(windowWatcher);
+  mc = open3PaneWindow();
   be_in_folder(folderA);
   assert_message_pane_visible();
 
   let actualHeight = mc.e("messagepaneboxwrapper").boxObject.height;
-  assert_equals(newHeight, actualHeight, "The message pane height should be " +
-    newHeight + ", but is actually " + actualHeight);
+
+  assert_equals(newHeight, actualHeight,
+    "The message pane height should be " + newHeight + ", but is actually " +
+    actualHeight + ". The oldHeight was: " + oldHeight);
 
   // The old height is restored.
   _move_splitter(mc.e("threadpane-splitter"), 0, -diffHeight);
@@ -251,13 +245,14 @@ function test_message_pane_height_persistence() {
   // The 3pane window is closed.
   mail3PaneWindow.close();
 
-  mc = open3PaneWindow(windowWatcher);
+  mc = open3PaneWindow();
   be_in_folder(folderA);
   assert_message_pane_visible();
 
   let actualHeight = mc.e("messagepaneboxwrapper").boxObject.height;
-  assert_equals(oldHeight, actualHeight, "The message pane height should be " +
-    oldHeight + ", but is actually " + actualHeight);
+  assert_equals(oldHeight, actualHeight,
+    "The message pane height should be " + oldHeight + ", but is actually " +
+    actualHeight);
 
   // We don't need the address book window any more.
   plan_for_window_close(abwc);
@@ -277,11 +272,7 @@ function test_message_pane_width_persistence() {
 
   // Get the state object. This assumes there is one and only one
   // 3pane window.
-  let windowWatcher = Cc["@mozilla.org/embedcomp/window-watcher;1"].
-                      getService(Ci.nsIWindowWatcher);
-  let windowMediator = Cc["@mozilla.org/appshell/window-mediator;1"].
-                       getService(Ci.nsIWindowMediator);
-  let mail3PaneWindow = windowMediator.getMostRecentWindow("mail:3pane");
+  let mail3PaneWindow = Services.wm.getMostRecentWindow("mail:3pane");
 
   let oldWidth = mc.e("messagepaneboxwrapper").boxObject.width;
   let minWidth = Math.floor(mc.e("messagepaneboxwrapper").getAttribute("minwidth"));
@@ -300,22 +291,19 @@ function test_message_pane_width_persistence() {
   // But this test case is not for testing moving around a splitter but for
   // persistency. Therefore it is enough if the actual width is equal to the
   // the requested width plus/minus one pixel.
-  assert_true((Math.abs(newWidth - actualWidth) <= 1),
+  assert_equals_fuzzy(newWidth, actualWidth, 1,
     "The message pane width should be " + newWidth + ", but is actually " +
     actualWidth + ". The oldWidth was: " + oldWidth);
   newWidth = actualWidth;
 
-  assert_equals(newWidth, actualWidth, "The message pane width should be " +
-    newWidth + ", but is actually " + actualWidth);
-
   // Make sure we have a different window open, so that we don't start shutting
   // down just because the last window was closed
-  let abwc = openAddressBook(windowWatcher);
+  let abwc = openAddressBook();
 
   // The 3pane window is closed.
   mail3PaneWindow.close();
 
-  mc = open3PaneWindow(windowWatcher);
+  mc = open3PaneWindow();
   be_in_folder(folderA);
   assert_message_pane_visible();
   assert_pane_layout(kVerticalMailLayout);
@@ -328,11 +316,11 @@ function test_message_pane_width_persistence() {
   _move_splitter(mc.e("threadpane-splitter"), -diffWidth, 0);
   let actualWidth = mc.e("messagepaneboxwrapper").boxObject.width;
 
-  // FIXME: For whatever reasons the new width is off by one pixel on Mac OSX
+  // FIXME: For whatever reasons the new width is off by two pixels on Mac OSX
   // But this test case is not for testing moving around a splitter but for
   // persistency. Therefore it is enough if the actual width is equal to the
-  // the requested width plus/minus one pixel.
-  assert_true((Math.abs(oldWidth - actualWidth) <= 1),
+  // the requested width plus/minus two pixels.
+  assert_equals_fuzzy(oldWidth, actualWidth, 2,
     "The message pane width should be " + oldWidth + ", but is actually " +
     actualWidth);
   oldWidth = actualWidth;
@@ -340,7 +328,7 @@ function test_message_pane_width_persistence() {
   // The 3pane window is closed.
   mail3PaneWindow.close();
 
-  mc = open3PaneWindow(windowWatcher);
+  mc = open3PaneWindow();
   be_in_folder(folderA);
   assert_message_pane_visible();
   assert_pane_layout(kVerticalMailLayout);
@@ -361,16 +349,12 @@ function test_message_pane_width_persistence() {
 
 function test_multiple_3pane_periodic_session_persistence() {
   // open a few more 3pane windows
-  let windowWatcher = Cc["@mozilla.org/embedcomp/window-watcher;1"].
-                      getService(Ci.nsIWindowWatcher);
   for (var i = 0; i < 3; ++i)
-    open3PaneWindow(windowWatcher);
+    open3PaneWindow();
 
   // then get the state objects for each window
   let state = [];
-  let windowMediator = Cc["@mozilla.org/appshell/window-mediator;1"].
-                       getService(Ci.nsIWindowMediator);
-  let enumerator = windowMediator.getEnumerator("mail:3pane");
+  let enumerator = Services.wm.getEnumerator("mail:3pane");
   while (enumerator.hasMoreElements())
     state.push(enumerator.getNext().getWindowStateForSessionPersistence());
 
@@ -391,7 +375,7 @@ function test_multiple_3pane_periodic_session_persistence() {
             "saved state and loaded state should be equal");
 
   // close all but one 3pane window
-  enumerator = windowMediator.getEnumerator("mail:3pane");
+  enumerator = Services.wm.getEnumerator("mail:3pane");
   while (enumerator.hasMoreElements()) {
     let window = enumerator.getNext();
     if (enumerator.hasMoreElements())
@@ -425,21 +409,17 @@ function test_bad_session_file_simple() {
 function test_clean_shutdown_session_persistence_simple() {
 
   // open a few more 3pane windows
-  let windowWatcher = Cc["@mozilla.org/embedcomp/window-watcher;1"].
-                      getService(Ci.nsIWindowWatcher);
   for (var i = 0; i < 3; ++i) {
-    open3PaneWindow(windowWatcher);
+    open3PaneWindow();
   }
 
   // make sure we have a different window open, so that we don't start shutting
   // down just because the last window was closed
-  let abwc = openAddressBook(windowWatcher);
+  let abwc = openAddressBook();
 
   // close all the 3pane windows
   let lastWindowState = null;
-  let windowMediator = Cc["@mozilla.org/appshell/window-mediator;1"].
-                       getService(Ci.nsIWindowMediator);
-  enumerator = windowMediator.getEnumerator("mail:3pane");
+  enumerator = Services.wm.getEnumerator("mail:3pane");
   while (enumerator.hasMoreElements()) {
     let window = enumerator.getNext();
     if (!enumerator.hasMoreElements())
@@ -461,7 +441,7 @@ function test_clean_shutdown_session_persistence_simple() {
                 "saved state and loaded state should be equal");
 
 
-  open3PaneWindow(windowWatcher);
+  open3PaneWindow();
 
   // We don't need the address book window any more.
   plan_for_window_close(abwc);
@@ -477,7 +457,7 @@ function test_clean_shutdown_session_persistence_simple() {
 function _move_splitter(aSplitter, aDiffX, aDiffY) {
 
   // catch the splitter in the middle
-  EventUtils.synthesizeMouse(aSplitter, 0, 0, {type:"mousedown"}, mc.window);
+  EventUtils.synthesizeMouse(aSplitter, 1, 0, {type:"mousedown"}, mc.window);
   EventUtils.synthesizeMouse(aSplitter, aDiffX, aDiffY, {type:"mousemove"},
                              mc.window);
   // release the splitter in the middle
@@ -485,6 +465,19 @@ function _move_splitter(aSplitter, aDiffX, aDiffY) {
 
 }
 
+/**
+ * Helper function that checks the fuzzy equivalence of two numeric
+ * values against some given tolerance.
+ *
+ * @param aLeft one value to check equivalence with
+ * @param aRight the other value to check equivalence with
+ * @param aTolerance how fuzzy can our equivalence be?
+ * @param aMessage the message to give off if we're outside of tolerance.
+ */
+function assert_equals_fuzzy(aLeft, aRight, aTolerance, aMessage) {
+  assert_true(Math.abs(aLeft - aRight) <= aTolerance, aMessage);
+}
+
 // XXX todo
 // - crash test: not sure if this test should be here. restoring a crashed
 //               session depends on periodically saved session data (there is
