@@ -47,6 +47,42 @@ var currentPageId;
 var pendingAccount;
 var pendingPageId;
 
+/**
+ * This array contains filesystem folders that are deemed inappropriate
+ * for use as the local directory pref for message storage.
+ * It is global to allow extensions to add to/remove from it if needed.
+ * Extentions adding new server types should first consider setting
+ * nsIMsgProtocolInfo(of the server type).defaultLocalPath properly
+ * so that the test will allow that directory automatically.
+ * See the checkLocalDirectoryIsSafe function for description of the members.
+ */
+var gDangerousLocalStorageDirs = [
+  // profile folder
+  { dirsvc: "ProfD",    OS: null },
+  // GRE install folder
+  { dirsvc: "GreD",     OS: null },
+  // Application install folder
+  { dirsvc: "CurProcD", OS: null },
+  // system temporary folder
+  { dirsvc: "TmpD",     OS: null },
+  // Windows system folder
+  { dirsvc: "SysD",     OS: "WINNT" },
+  // Windows folder
+  { dirsvc: "WinD",     OS: "WINNT" },
+  // Program Files folder
+  { dirsvc: "ProgF",    OS: "WINNT" },
+  // trash folder
+  { dirsvc: "Trsh",     OS: "Darwin" },
+  // Mac OS system folder
+  { dir:    "/System",  OS: "Darwin" },
+  // devices folder
+  { dir:    "/dev",     OS: "Darwin,Linux" },
+  // process info folder
+  { dir:    "/proc",    OS: "Linux" },
+  // system state folder
+  { dir:    "/sys",     OS: "Linux" }
+];
+
 // This sets an attribute in a xul element so that we can later
 // know what value to substitute in a prefstring.  Different
 // preference types set different attributes.  We get the value
@@ -73,15 +109,15 @@ function updateElementWithKeys(account, element, type) {
 }
 
 function hideShowControls(serverType) {
-  var controls = document.getElementsByAttribute("hidefor", "*");
-  for (var controlNo = 0; controlNo < controls.length; controlNo++) {
-    var control = controls[controlNo];
-    var hideFor = control.getAttribute("hidefor");
+  let controls = document.querySelectorAll("[hidefor]");
+  for (let controlNo = 0; controlNo < controls.length; controlNo++) {
+    let control = controls[controlNo];
+    let hideFor = control.getAttribute("hidefor");
 
     // Hide unsupported server types using hideFor="servertype1,servertype2".
-    var hide = false;
-    var hideForTokens = hideFor.split(",");
-    for (var tokenNo = 0; tokenNo < hideForTokens.length; tokenNo++) {
+    let hide = false;
+    let hideForTokens = hideFor.split(",");
+    for (let tokenNo = 0; tokenNo < hideForTokens.length; tokenNo++) {
       if (hideForTokens[tokenNo] == serverType) {
         hide = true;
         break;
@@ -149,8 +185,7 @@ function selectServer(server, selectPageId)
     let pageId = accountNode.getAttribute("PageTag");
     if (pageId != selectPageId) {
       // ... or one of its children.
-      let pages = accountNode.getElementsByAttribute("PageTag", selectPageId);
-      pageToSelect = pages[0];
+      pageToSelect = accountNode.querySelector('[PageTag="' + selectPageId + '"]');
     }
   }
 
@@ -260,6 +295,108 @@ function checkDirectoryIsValid(aLocalPath) {
   return true;
 }
 
+/**
+ * Even if the local path is usable, there are some special folders we do not
+ * want to allow for message storage as they cause problems (see e.g. bug 750781).
+ *
+ * aLocalPath  The nsIFile of a directory to check.
+ */
+function checkDirectoryIsAllowed(aLocalPath) {
+  /**
+   * Check if the local path (aLocalPath) is 'safe' i.e. NOT a parent
+   * or subdirectory of the given special system/app directory (aDirToCheck).
+   *
+   * @param aDirToCheck  An object describing the special directory.
+   *        The object has the following members:
+   *        dirsvc      : A path keyword to retrieve from the Directory service.
+   *        dir         : An absolute filesystem path.
+   *                      Only one of 'dirsvc' or 'dir' can be specified.
+   *        OS          : A string of comma separated values defining on which
+   *                      Operating systems the folder is unusable:
+   *                       null   = all
+   *                       WINNT  = Windows
+   *                       Darwin = OS X
+   *                       Linux  = Linux
+   *        safeSubdirs : An array of directory names that are allowed to be used
+   *                      under the tested directory.
+   * @param aLocalPath  An nsIFile of the directory to check, intended for message storage.
+   */
+  function checkLocalDirectoryIsSafe(aDirToCheck, aLocalPath) {
+    if (aDirToCheck.OS) {
+      if (aDirToCheck.OS.split(",").indexOf(Services.appinfo.OS) == -1)
+        return true;
+    }
+
+    let testDir = null;
+    if ("dirsvc" in aDirToCheck) {
+      try {
+        testDir = Services.dirsvc.get(aDirToCheck.dirsvc, Components.interfaces.nsIFile);
+      } catch (e) {
+        Components.utils.reportError("The special folder " + aDirToCheck.dirsvc +
+          " cannot be retrieved on this platform: " + e);
+      }
+
+      if (!testDir)
+        return true;
+    }
+    else if ("dir" in aDirToCheck) {
+      testDir = Components.classes["@mozilla.org/file/local;1"]
+                          .createInstance(Components.interfaces.nsIFile);
+      testDir.initWithPath(aDirToCheck.dir);
+      if (!testDir.exists())
+        return true;
+    } else {
+      Components.utils.reportError("No directory to check?");
+      return true;
+    }
+
+    testDir.normalize();
+
+    if (testDir.equals(aLocalPath) || aLocalPath.contains(testDir, true))
+      return false;
+
+    if (testDir.contains(aLocalPath, true)) {
+      if (!("safeSubdirs" in aDirToCheck))
+        return false;
+
+      // While the tested directory may not be safe,
+      // a subdirectory of some safe subdirectories may be fine.
+      let isInSubdir = false;
+      for (let subDir of aDirToCheck.safeSubdirs) {
+        let checkDir = testDir.clone();
+        checkDir.append(subDir);
+        if (checkDir.contains(aLocalPath, true)) {
+          isInSubdir = true;
+          break;
+        }
+      }
+      return isInSubdir;
+    }
+
+    return true;
+  } // end of checkDirectoryIsNotSpecial
+
+  // If the server type has a nsIMsgProtocolInfo.defaultLocalPath set,
+  // allow that directory.
+  if (currentAccount.incomingServer) {
+    try {
+      let defaultPath = currentAccount.incomingServer.protocolInfo.defaultLocalPath;
+      if (defaultPath) {
+        defaultPath.normalize();
+        if (defaultPath.contains(aLocalPath, true))
+          return true;
+      }
+    } catch (e) { /* No problem if this fails. */ }
+  }
+
+  for (let tryDir of gDangerousLocalStorageDirs) {
+    if (!checkLocalDirectoryIsSafe(tryDir, aLocalPath))
+      return false;
+  }
+
+  return true;
+}
+
 /**
  * Check if the specified directory does meet all the requirements
  * for safe mail storage.
@@ -269,6 +406,8 @@ function checkDirectoryIsValid(aLocalPath) {
 function checkDirectoryIsUsable(aLocalPath) {
   const kAlertTitle = document.getElementById("bundle_prefs")
                               .getString("prefPanel-server");
+  const originalPath = aLocalPath;
+
   let invalidPath = false;
   try{
     aLocalPath.normalize();
@@ -276,16 +415,25 @@ function checkDirectoryIsUsable(aLocalPath) {
 
   if (invalidPath || !checkDirectoryIsValid(aLocalPath)) {
     let alertString = document.getElementById("bundle_prefs")
-                              .getString("localDirectoryInvalid");
+                              .getFormattedString("localDirectoryInvalid",
+                                                  [originalPath.path]);
     Services.prompt.alert(window, kAlertTitle, alertString);
     return false;
   }
 
+  if (!checkDirectoryIsAllowed(aLocalPath)) {
+    let alertNotAllowed = document.getElementById("bundle_prefs")
+                                  .getFormattedString("localDirectoryNotAllowed",
+                                                      [originalPath.path]);
+    Services.prompt.alert(window, kAlertTitle, alertNotAllowed);
+    return false;
+  }
+
   // Check that no other account has this same or dependent local directory.
   let allServers = MailServices.accounts.allServers;
 
-  for each (let server in fixIterator(allServers,
-                                      Components.interfaces.nsIMsgIncomingServer))
+  for (let server in fixIterator(allServers,
+                                 Components.interfaces.nsIMsgIncomingServer))
   {
     if (server.key == currentAccount.incomingServer.key)
       continue;
@@ -339,6 +487,8 @@ function checkUserServerChanges(showAlert) {
   if (!pageElements)
     return true;
 
+  let currentServer = currentAccount ? currentAccount.incomingServer : null;
+
   // Get the new username, hostname and type from the page
   var newUser, newHost, newType, oldUser, oldHost;
   var uIndx, hIndx;
@@ -369,8 +519,8 @@ function checkUserServerChanges(showAlert) {
   }
 
   var checkUser = true;
-  // There is no username defined for news so reset it.
-  if (newType == "nntp") {
+  // There is no username needed for e.g. news so reset it.
+  if (currentServer && !currentServer.protocolInfo.requiresUsername) {
     oldUser = newUser = "";
     checkUser = false;
   }
@@ -388,7 +538,7 @@ function checkUserServerChanges(showAlert) {
     else {
       let sameServer = MailServices.accounts
                                    .findRealServer(newUser, newHost, newType, 0);
-      if (sameServer && (sameServer != currentAccount.incomingServer)) {
+      if (sameServer && (sameServer != currentServer)) {
         alertText = prefBundle.getString("modifiedAccountExists");
       } else {
         // New hostname passed all checks. We may have cleaned it up so set
@@ -414,8 +564,8 @@ function checkUserServerChanges(showAlert) {
     // to edit rules.
     if (showAlert) {
       let filterList;
-      if (currentAccount && checkUser) {
-        filterList = currentAccount.incomingServer.getEditableFilterList(null);
+      if (currentServer && checkUser) {
+        filterList = currentServer.getEditableFilterList(null);
       }
       let changeText = "";
       if ((oldHost != newHost) &&
@@ -552,15 +702,9 @@ function onRemoveAccount(event) {
     return;
 
   let server = currentAccount.incomingServer;
-  let type = server.type;
   let prettyName = server.prettyName;
 
-  let canDelete = false;
-  if (Components.classes["@mozilla.org/messenger/protocol/info;1?type=" + type]
-                .getService(Components.interfaces.nsIMsgProtocolInfo).canDelete)
-    canDelete = true;
-  else
-    canDelete = server.canDelete;
+  let canDelete = server.protocolInfo.canDelete || server.canDelete;
 
   if (!canDelete)
     return;
@@ -773,17 +917,12 @@ function updateItems(tree, account, addAccountItem, setDefaultItem, removeItem)
     // Otherwise we have either selected a SMTP server, or there is some
     // problem. Either way, we don't want the user to act on it.
     let server = account.incomingServer;
-    let type = server.type;
 
     if (account != MailServices.accounts.defaultAccount &&
         server.canBeDefaultServer && account.identities.length > 0)
       canSetDefault = true;
 
-    if (Components.classes["@mozilla.org/messenger/protocol/info;1?type=" + type]
-                  .getService(Components.interfaces.nsIMsgProtocolInfo).canDelete)
-      canDelete = true;
-    else
-      canDelete = server.canDelete;
+    canDelete = server.protocolInfo.canDelete || server.canDelete;
   }
 
   setEnabled(addAccountItem, true);
@@ -918,7 +1057,7 @@ function onPanelLoaded(pageId) {
   pendingPageId = null;
 }
 
-function loadPage(pageId)
+function pageURL(pageId)
 {
   let chromePackageName;
   try {
@@ -931,8 +1070,14 @@ function loadPage(pageId)
   catch (ex) {
     chromePackageName = "messenger";
   }
+  return "chrome://" + chromePackageName + "/content/" + pageId;
+}
+
+function loadPage(pageId)
+{
   const LOAD_FLAGS_NONE = Components.interfaces.nsIWebNavigation.LOAD_FLAGS_NONE;
-  document.getElementById("contentFrame").webNavigation.loadURI("chrome://" + chromePackageName + "/content/" + pageId, LOAD_FLAGS_NONE, null, null, null);
+  document.getElementById("contentFrame").webNavigation.loadURI(pageURL(pageId),
+    LOAD_FLAGS_NONE, null, null, null);
 }
 
 // save the values of the widgets to the given server
@@ -1139,9 +1284,6 @@ function getFormElementValue(formElement) {
 
 // sets the value of a widget
 function setFormElementValue(formElement, value) {
-
-  //formElement.value = formElement.defaultValue;
-  //  formElement.checked = formElement.defaultChecked;
   var type = formElement.localName;
   if (type == "checkbox") {
     if (value == undefined) {
@@ -1156,20 +1298,11 @@ function setFormElementValue(formElement, value) {
         formElement.checked = value;
     }
   }
-
   else if (type == "radiogroup" || type =="menulist") {
-
-    var selectedItem;
-    if (value == undefined) {
-      if (type == "radiogroup")
-        selectedItem = formElement.firstChild;
-      else
-        selectedItem = formElement.firstChild.firstChild;
-    }
+    if (value == undefined)
+      formElement.selectedIndex = 0;
     else
-      selectedItem = formElement.getElementsByAttribute("value", value)[0];
-
-    formElement.selectedItem = selectedItem;
+      formElement.value = value;
   }
   // handle nsILocalFile
   else if (type == "textbox" &&
@@ -1196,6 +1329,13 @@ function setFormElementValue(formElement, value) {
       formElement.value = value;
     }
   }
+  else if (type == "label") {
+    if (value == null || value == undefined) {
+      formElement.value = "";
+    } else {
+      formElement.value = value;
+    }
+  }
 
   // let the form figure out what to do with it
   else {
