@@ -445,6 +445,9 @@ nsMsgComposeService::GetOrigWindowSelection(MSG_ComposeType type, nsIMsgWindow *
     // two words selected in order to quote just the selected text
     if (requireMultipleWords)
     {
+      if (selPlain.IsEmpty())
+        return NS_ERROR_ABORT;
+
       nsCOMPtr<nsILineBreaker> lineBreaker = do_GetService(NS_LBRK_CONTRACTID, &rv);
 
       if (NS_SUCCEEDED(rv))
@@ -824,7 +827,11 @@ nsMsgComposeService::CacheWindow(nsIDOMWindow *aWindow, bool aComposeHTML, nsIMs
   nsCOMPtr<nsIDocShellTreeItem> treeItem(do_QueryInterface(window->GetDocShell(), &rv));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  nsCOMPtr<nsIXULWindow> xulWindow(do_GetInterface(treeItem, &rv));
+  nsCOMPtr <nsIDocShellTreeOwner> treeOwner;
+  rv = treeItem->GetTreeOwner(getter_AddRefs(treeOwner));
+  NS_ENSURE_SUCCESS(rv,rv);
+
+  nsCOMPtr<nsIXULWindow> xulWindow(do_GetInterface(treeOwner, &rv));
   NS_ENSURE_SUCCESS(rv, rv);
 
   int32_t i;
