@@ -331,14 +331,14 @@ function test_threading() {
  *  message header".
  */
 function test_attachment_flag() {
-  // create a synthetic message with attachment
+  // create a synthetic message with an attachment that won't normally be listed
+  // in the attachment pane (Content-Disposition: inline, no filename, and
+  // displayable inline)
   let smsg = msgGen.makeMessage({
     name: 'test message with part 1.2 attachment',
-    bodyPart: new SyntheticPartMultiMixed([
-      new SyntheticPartLeaf({body: 'I like cheese!'}),
-      let (m = msgGen.makeMessage({ body: { body: 'I like wine!' }}))
-      (m._contentType = "roquefort", m)
-    ]),
+    attachments: [{ body: 'attachment',
+                    filename: '',
+                    format: '' }],
   });
   // save it off for test_attributes_fundamental_from_disk
   let msgSet = new SyntheticMessageSet([smsg]);
@@ -722,8 +722,8 @@ function test_streamed_bodies_are_size_capped() {
 
   yield wait_for_gloda_indexer(msgSet, {augment: true});
   let gmsg = msgSet.glodaMessages[0];
-  do_check_eq(gmsg.indexedBodyText.indexOf("aabb"), 0);
-  do_check_eq(gmsg.indexedBodyText.indexOf("xxyy"), -1);
+  do_check_true(gmsg.indexedBodyText.startsWith("aabb"));
+  do_check_false(gmsg.indexedBodyText.contains("xxyy"));
 
   if (gmsg.indexedBodyText.length > (20 * 1024 + 58 + 10))
     do_throw("indexed body text is too big! (" + gmsg.indexedBodyText.length +
@@ -1034,9 +1034,7 @@ function test_folder_deletion_nested() {
   let trash = make_empty_folder(null, [Ci.nsMsgFolderFlags.Trash]);
   yield move_folder(folder1, trash);
 
-  let descendentFolders = Cc["@mozilla.org/supports-array;1"]
-                          .createInstance(Ci.nsISupportsArray);
-  get_nsIMsgFolder(trash).ListDescendents(descendentFolders);
+  let descendentFolders = get_nsIMsgFolder(trash).descendants;
   let folders = [folder for (folder in fixIterator(descendentFolders, Ci.nsIMsgFolder))];
   do_check_eq(folders.length, 2);
   let [newFolder1, newFolder2] = folders;
