@@ -15,6 +15,9 @@ var nsNewsBlogFeedDownloader =
     if (!gExternalScriptsLoaded)
       loadScripts();
 
+    if (Services.io.offline)
+      return;
+
     // We don't yet support the ability to check for new articles while we are
     // in the middle of subscribing to a feed. For now, abort the check for
     // new feeds.
@@ -25,23 +28,25 @@ var nsNewsBlogFeedDownloader =
       return;
     }
 
-    let allFolders = Cc["@mozilla.org/supports-array;1"].
-                     createInstance(Ci.nsISupportsArray);
-    if (!aFolder.isServer)
-      // Add the base folder; it does not get added by ListDescendents.  Do not
+    let allFolders = Cc["@mozilla.org/array;1"].
+                     createInstance(Ci.nsIMutableArray);
+    if (!aFolder.isServer) {
+      // Add the base folder; it does not get returned by ListDescendants. Do not
       // add the account folder as it doesn't have the feedUrl property or even
       // a msgDatabase necessarily.
-      allFolders.AppendElement(aFolder);
+      allFolders.appendElement(aFolder, false);
+    }
+
+    aFolder.ListDescendants(allFolders);
 
-    aFolder.ListDescendents(allFolders);
-    let numFolders = allFolders.Count();
     let trashFolder =
         aFolder.rootFolder.getFolderWithFlags(Ci.nsMsgFolderFlags.Trash);
 
     function feeder() {
       let folder;
+      let numFolders = allFolders.length;
       for (let i = 0; i < numFolders; i++) {
-        folder = allFolders.GetElementAt(i).QueryInterface(Ci.nsIMsgFolder);
+        folder = allFolders.queryElementAt(i, Ci.nsIMsgFolder);
         FeedUtils.log.debug("downloadFeed: START x/# foldername:uri - " +
                             (i+1) + "/" + numFolders + " " +
                             folder.name + ":" + folder.URI);
