@@ -25,6 +25,7 @@
 #include "nsIAutoSyncManager.h"
 #include "nsAlgorithm.h"
 #include "nsArrayUtils.h"
+#include <algorithm>
 
 static NS_DEFINE_CID(kRDFServiceCID, NS_RDFSERVICE_CID);
 
@@ -107,8 +108,7 @@ nsImapOfflineSync::OnStopRunningUrl(nsIURI* url, nsresult exitCode)
   // go to the next folder.
   else if (!m_singleFolderToUpdate)
   {
-    rv = AdvanceToNextFolder();
-    if (NS_SUCCEEDED(rv))
+    if (AdvanceToNextFolder())
       rv = ProcessNextOperation();
     else if (m_listener)
       m_listener->OnStopRunningUrl(url, rv);
@@ -117,11 +117,13 @@ nsImapOfflineSync::OnStopRunningUrl(nsIURI* url, nsresult exitCode)
   return rv;
 }
 
-// leaves m_currentServer at the next imap or local mail "server" that
-// might have offline events to playback. If no more servers,
-// m_currentServer will be left at nullptr.
-// Also, sets up m_serverEnumerator to enumerate over the server
-nsresult nsImapOfflineSync::AdvanceToNextServer()
+/**
+ * Leaves m_currentServer at the next imap or local mail "server" that
+ * might have offline events to playback. If no more servers,
+ * m_currentServer will be left at nullptr and the function returns false.
+ * Also, sets up m_serverEnumerator to enumerate over the server.
+ */
+bool nsImapOfflineSync::AdvanceToNextServer()
 {
   nsresult rv = NS_OK;
 
@@ -132,10 +134,11 @@ nsresult nsImapOfflineSync::AdvanceToNextServer()
     nsCOMPtr<nsIMsgAccountManager> accountManager = 
              do_GetService(NS_MSGACCOUNTMANAGER_CONTRACTID, &rv);
     NS_ASSERTION(accountManager && NS_SUCCEEDED(rv), "couldn't get account mgr");
-    if (!accountManager || NS_FAILED(rv)) return rv;
+    if (!accountManager || NS_FAILED(rv))
+      return false;
 
     rv = accountManager->GetAllServers(getter_AddRefs(m_allServers));
-    NS_ENSURE_SUCCESS(rv, rv);
+    NS_ENSURE_SUCCESS(rv, false);
   }
   uint32_t serverIndex = 0;
   if (m_currentServer)
@@ -148,7 +151,7 @@ nsresult nsImapOfflineSync::AdvanceToNextServer()
     ++serverIndex;
   }
   m_currentServer = nullptr;
-  uint32_t numServers; 
+  uint32_t numServers;
   m_allServers->GetLength(&numServers);
   nsCOMPtr <nsIMsgFolder> rootFolder;
 
@@ -157,35 +160,41 @@ nsresult nsImapOfflineSync::AdvanceToNextServer()
     nsCOMPtr<nsIMsgIncomingServer> server(do_QueryElementAt(m_allServers, serverIndex));
     serverIndex++;
 
-    nsCOMPtr <nsINntpIncomingServer> newsServer = do_QueryInterface(server);
+    nsCOMPtr<nsINntpIncomingServer> newsServer = do_QueryInterface(server);
     if (newsServer) // news servers aren't involved in offline imap
       continue;
+
     if (server)
     {
       m_currentServer = server;
       server->GetRootFolder(getter_AddRefs(rootFolder));
       if (rootFolder)
       {
-        m_allFolders = do_CreateInstance(NS_SUPPORTSARRAY_CONTRACTID, &rv);
-        NS_ENSURE_TRUE(m_allFolders, rv);
-        rv = rootFolder->ListDescendents(m_allFolders);
+        rv = rootFolder->GetDescendants(getter_AddRefs(m_allFolders));
         if (NS_SUCCEEDED(rv))
-          m_allFolders->Enumerate(getter_AddRefs(m_serverEnumerator));
-        if (NS_SUCCEEDED(rv) && m_serverEnumerator)
         {
-          rv = m_serverEnumerator->First();
-          if (NS_SUCCEEDED(rv))
-            break;
+          rv = m_allFolders->Enumerate(getter_AddRefs(m_serverEnumerator));
+          if (NS_SUCCEEDED(rv) && m_serverEnumerator)
+          {
+            bool hasMore = false;
+            rv = m_serverEnumerator->HasMoreElements(&hasMore);
+            if (NS_SUCCEEDED(rv) && hasMore)
+              return true;
+          }
         }
       }
     }
   }
-  return rv;
+  return false;
 }
 
-nsresult nsImapOfflineSync::AdvanceToNextFolder()
+/**
+ * Sets m_currentFolder to the next folder to process.
+ *
+ * @return  True if next folder to process was found, otherwise false.
+ */
+bool nsImapOfflineSync::AdvanceToNextFolder()
 {
-  nsresult rv;
   // we always start by changing flags
   mCurrentPlaybackOpType = nsIMsgOfflineImapOperation::kFlagsChanged;
 
@@ -195,35 +204,31 @@ nsresult nsImapOfflineSync::AdvanceToNextFolder()
     m_currentFolder = nullptr;
   }
 
-  if (!m_currentServer)
-     rv = AdvanceToNextServer();
-  else
-    rv = m_serverEnumerator->Next();
-  if (NS_FAILED(rv))
-    rv = AdvanceToNextServer();
+  bool hasMore = false;
+  if (m_currentServer)
+    m_serverEnumerator->HasMoreElements(&hasMore);
+  if (!hasMore)
+    hasMore = AdvanceToNextServer();
 
-  if (NS_SUCCEEDED(rv) && m_serverEnumerator)
+  if (hasMore)
   {
-    nsCOMPtr <nsISupports> supports;
-    rv = m_serverEnumerator->CurrentItem(getter_AddRefs(supports));
-    m_currentFolder = do_QueryInterface(supports);
+    nsCOMPtr<nsISupports> supports;
+    nsresult rv = m_serverEnumerator->GetNext(getter_AddRefs(supports));
+    if (NS_SUCCEEDED(rv))
+      m_currentFolder = do_QueryInterface(supports);
   }
   ClearDB();
-  return rv;
+  return m_currentFolder;
 }
 
 void nsImapOfflineSync::AdvanceToFirstIMAPFolder()
 {
-  nsresult rv;
   m_currentServer = nullptr;
-  nsCOMPtr <nsIMsgImapMailFolder> imapFolder;
-  do
+  nsCOMPtr<nsIMsgImapMailFolder> imapFolder;
+  while (!imapFolder && AdvanceToNextFolder())
   {
-    rv = AdvanceToNextFolder();
-    if (m_currentFolder)
-      imapFolder = do_QueryInterface(m_currentFolder);
+    imapFolder = do_QueryInterface(m_currentFolder);
   }
-  while (NS_SUCCEEDED(rv) && m_currentFolder && !imapFolder);
 }
 
 void nsImapOfflineSync::ProcessFlagOperation(nsIMsgOfflineImapOperation *op)
@@ -423,7 +428,7 @@ nsImapOfflineSync::ProcessAppendMsgOperation(nsIMsgOfflineImapOperation *current
                 rv = NS_OK;
                 while (bytesLeft > 0 && NS_SUCCEEDED(rv))
                 {
-                  int32_t bytesToRead = NS_MIN(inputBufferSize, bytesLeft);
+                  int32_t bytesToRead = std::min(inputBufferSize, bytesLeft);
                   rv = offlineStoreInputStream->Read(inputBuffer, bytesToRead, &bytesRead);
                   if (NS_SUCCEEDED(rv) && bytesRead > 0)
                   {
@@ -730,15 +735,15 @@ int32_t nsImapOfflineSync::GetCurrentUIDValidity()
   return mCurrentUIDValidity; 
 }
 
-// Playing back offline operations is one giant state machine that runs through ProcessNextOperation.
-// The first state is creating online any folders created offline (we do this first, so we can play back
-// any operations in them in the next pass)
-
+/**
+ * Playing back offline operations is one giant state machine that runs through
+ * ProcessNextOperation.
+ * The first state is creating online any folders created offline (we do this
+ * first, so we can play back any operations in them in the next pass)
+ */
 nsresult nsImapOfflineSync::ProcessNextOperation()
 {
   nsresult rv = NS_OK;
-  // find a folder that needs to process operations
-  nsIMsgFolder *deletedAllOfflineEventsInFolder = nullptr;
 
   // if we haven't created offline folders, and we're updating all folders,
   // first, find offline folders to create.
@@ -841,8 +846,6 @@ nsresult nsImapOfflineSync::ProcessNextOperation()
         if (NS_FAILED(m_currentDB->ListAllOfflineOpIds(&m_CurrentKeys)) || m_CurrentKeys.IsEmpty())
         {
           ClearDB();
-          if (deletedGhostMsgs)
-            deletedAllOfflineEventsInFolder = m_currentFolder;
         }
         else if (folderFlags & nsMsgFolderFlags::ImapBox)
         {
@@ -1107,8 +1110,7 @@ nsresult nsImapOfflineDownloader::ProcessNextOperation()
   }
   if (!m_mailboxupdatesFinished)
   {
-    AdvanceToNextServer();
-    if (m_currentServer)
+    if (AdvanceToNextServer())
     {
       nsCOMPtr <nsIMsgFolder> rootMsgFolder;
       m_currentServer->GetRootFolder(getter_AddRefs(rootMsgFolder));
@@ -1163,9 +1165,8 @@ nsresult nsImapOfflineDownloader::ProcessNextOperation()
       m_mailboxupdatesFinished = true;
     }
   }
-  AdvanceToNextFolder();
 
-  while (m_currentFolder)
+  while (AdvanceToNextFolder())
   {
     uint32_t folderFlags;
 
@@ -1183,7 +1184,6 @@ nsresult nsImapOfflineDownloader::ProcessNextOperation()
         return rv;
       // if this fails and the user didn't cancel/stop, fall through to code that advances to next folder
     }
-    AdvanceToNextFolder();
   }
   if (m_listener)
     m_listener->OnStopRunningUrl(nullptr, NS_OK);
