@@ -274,7 +274,6 @@ public:
   void XAOL_Option(const char *option);
   void MailboxData();
   void GetMyRightsForFolder(const char *mailboxName);
-  void AutoSubscribeToMailboxIfNecessary(const char *mailboxName);
   void Bodystructure(const nsCString &messageId, bool idIsUid);
   void PipelinedFetchMessageParts(const char *uid, nsIMAPMessagePartIDArray *parts);
 
@@ -452,8 +451,9 @@ private:
   nsImapFlagAndUidState  *m_flagState;
   nsMsgBiffState        m_currentBiffState;
   // manage the IMAP server command tags
-  char m_currentServerCommandTag[10];   // enough for a billion
-  int  m_currentServerCommandTagNumber;
+  // 11 = enough memory for the decimal representation of MAX_UINT + trailing nul
+  char m_currentServerCommandTag[11];
+  uint32_t m_currentServerCommandTagNumber;
   void IncrementCommandTagNumber();
   const char *GetServerCommandTag();
 
@@ -603,12 +603,9 @@ private:
   nsRefPtr <nsMsgImapLineDownloadCache> m_downloadLineCache;
   nsRefPtr <nsMsgImapHdrXferInfo> m_hdrDownloadCache;
   nsCOMPtr <nsIImapHeaderInfo> m_curHdrInfo;
-  // mapping between mailboxes and the corresponding folder flags
-  nsDataHashtable<nsCStringHashKey, int32_t> m_standardListMailboxes;
   // mapping between special xlist mailboxes and the corresponding folder flags
   nsDataHashtable<nsCStringHashKey, int32_t> m_specialXListMailboxes;
 
-
   nsIImapHostSessionList * m_hostSessionList;
 
   bool m_fromHeaderSeen;
@@ -655,7 +652,6 @@ private:
       kListingForInfoAndDiscovery,
       kDiscoveringNamespacesOnly,
       kXListing,
-      kListingForFolderFlags,
       kListingForCreate
   };
   EMailboxHierarchyNameState  m_hierarchyNameState;
