@@ -9,15 +9,14 @@
 Services.prefs.setIntPref("mailnews.tcptimeout", 2);
 
 load("../../../resources/logHelper.js");
-load("../../../resources/mailTestUtils.js");
 load("../../../resources/asyncTestUtils.js");
 load("../../../resources/alertTestUtils.js");
 load("../../../resources/messageGenerator.js");
 
 // IMAP pump
-load("../../../resources/IMAPpump.js");
 
 // Globals
+Components.utils.import("resource:///modules/mailServices.js");
 
 setupIMAPPump();
 
@@ -25,7 +24,7 @@ var gGotAlert = false;
 var gGotMsgAdded = false;
 
 function alert(aDialogTitle, aText) {
-  do_check_eq(aText.indexOf("Connection to server Mail for  timed out."), 0);
+  do_check_true(aText.startsWith("Connection to server Mail for  timed out."));
   gGotAlert = true;
   async_driver();
 }
@@ -53,10 +52,10 @@ var tests = [
 let gTargetFolder;
 function createTargetFolder()
 {
-  gIMAPDaemon.copySleep = 5000;
-  gIMAPIncomingServer.rootFolder.createSubfolder("targetFolder", null);
+  IMAPPump.daemon.copySleep = 5000;
+  IMAPPump.incomingServer.rootFolder.createSubfolder("targetFolder", null);
   yield false; 
-  gTargetFolder = gIMAPIncomingServer.rootFolder.getChildNamed("targetFolder");
+  gTargetFolder = IMAPPump.incomingServer.rootFolder.getChildNamed("targetFolder");
   do_check_true(gTargetFolder instanceof Ci.nsIMsgImapMailFolder);
   gTargetFolder.updateFolderWithListener(null, asyncUrlListener);
   yield false;
@@ -73,13 +72,13 @@ function loadImapMessage()
     Services.io.newURI("data:text/plain;base64," +
                        btoa(messages[0].toMessageString()),
                        null, null);
-  let imapInbox =  gIMAPDaemon.getMailbox("INBOX")
+  let imapInbox = IMAPPump.daemon.getMailbox("INBOX");
   gMessage = new imapMessage(msgURI.spec, imapInbox.uidnext++, []);
-  gIMAPMailbox.addMessage(gMessage);
-  gIMAPInbox.updateFolder(null);
+  IMAPPump.mailbox.addMessage(gMessage);
+  IMAPPump.inbox.updateFolder(null);
   yield false;
-  do_check_eq(1, gIMAPInbox.getTotalMessages(false));
-  let msgHdr = firstMsgHdr(gIMAPInbox);
+  do_check_eq(1, IMAPPump.inbox.getTotalMessages(false));
+  let msgHdr = mailTestUtils.firstMsgHdr(IMAPPump.inbox);
   do_check_true(msgHdr instanceof Ci.nsIMsgDBHdr);
 
   yield true;
@@ -88,18 +87,16 @@ function loadImapMessage()
 // move the message to a diffent folder
 function moveMessageToTargetFolder()
 {
-  let msgHdr = firstMsgHdr(gIMAPInbox);
+  let msgHdr = mailTestUtils.firstMsgHdr(IMAPPump.inbox);
 
   // Now move this message to the target folder.
   var messages = Cc["@mozilla.org/array;1"]
                    .createInstance(Ci.nsIMutableArray);
   messages.appendElement(msgHdr, false);
-  let copyService = Cc["@mozilla.org/messenger/messagecopyservice;1"]
-                      .getService(Ci.nsIMsgCopyService);
   // This should cause the move to be done as an offline imap operation
   // that's played back immediately.
-  copyService.CopyMessages(gIMAPInbox, messages, gTargetFolder, true,
-                           CopyListener, gDummyMsgWindow, true);
+  MailServices.copy.CopyMessages(IMAPPump.inbox, messages, gTargetFolder, true,
+                                 CopyListener, gDummyMsgWindow, true);
   yield false;
 }
 
@@ -123,7 +120,7 @@ function endTest()
 {
   do_check_true(gGotAlert);
   // Make sure neither source nor target folder have offline events.
-  do_check_false(gIMAPInbox.getFlag(Ci.nsMsgFolderFlags.OfflineEvents));
+  do_check_false(IMAPPump.inbox.getFlag(Ci.nsMsgFolderFlags.OfflineEvents));
   do_check_false(gTargetFolder.getFlag(Ci.nsMsgFolderFlags.OfflineEvents));
 
   // fake server does the copy, but then times out, so make sure the target
@@ -158,11 +155,9 @@ function run_test()
   Services.prefs.setBoolPref("mail.server.default.autosync_offline_stores", false);
   // Add folder listeners that will capture async events
   const nsIMFNService = Ci.nsIMsgFolderNotificationService;
-  let MFNService = Cc["@mozilla.org/messenger/msgnotificationservice;1"]
-                      .getService(nsIMFNService);
   let flags =
         nsIMFNService.folderAdded |
         nsIMFNService.msgAdded;
-  MFNService.addListener(mfnListener, flags);
+  MailServices.mfn.addListener(mfnListener, flags);
   async_run_tests(tests);
 }
