@@ -6,9 +6,7 @@
 
 load("../../../resources/logHelper.js");
 load("../../../resources/asyncTestUtils.js");
-load("../../../resources/mailTestUtils.js");
 load("../../../resources/messageGenerator.js");
-load("../../../resources/IMAPpump.js");
 
 Services.prefs.setCharPref("mail.serverDefaultStoreContractID",
                            "@mozilla.org/msgstore/berkeleystore;1");
@@ -25,7 +23,7 @@ function run_test() {
   setupIMAPPump();
 
   // Figure out the name of the IMAP inbox
-  let inboxFile = gIMAPIncomingServer.rootMsgFolder.filePath;
+  let inboxFile = IMAPPump.incomingServer.rootMsgFolder.filePath;
   inboxFile.append("INBOX");
   if (!inboxFile.exists())
     inboxFile.create(Ci.nsIFile.NORMAL_FILE_TYPE, parseInt("0644", 8));
@@ -35,18 +33,18 @@ function run_test() {
   // sparse. If it isn't, then bail out now, because in all probability it is
   // FAT32, which doesn't support file sizes greater than 4 GB.
   if ("@mozilla.org/windows-registry-key;1" in Cc &&
-      get_file_system(inboxFile) != "NTFS")
+      mailTestUtils.get_file_system(inboxFile) != "NTFS")
   {
     dump("On Windows, this test only works on NTFS volumes.\n");
     teardown();
     return;
   }
 
-  let isFileSparse = mark_file_region_sparse(inboxFile, 0, 0x10000000f);
+  let isFileSparse = mailTestUtils.mark_file_region_sparse(inboxFile, 0, 0x10000000f);
   let freeDiskSpace = inboxFile.diskSpaceAvailable;
-  do_print("Free disk space = " + toMiBString(freeDiskSpace));
+  do_print("Free disk space = " + mailTestUtils.toMiBString(freeDiskSpace));
   if (!isFileSparse && freeDiskSpace < neededFreeSpace) {
-    do_print("This test needs " + toMiBString(neededFreeSpace) +
+    do_print("This test needs " + mailTestUtils.toMiBString(neededFreeSpace) +
              " free space to run. Aborting.");
     todo_check_true(false);
 
@@ -67,21 +65,21 @@ function setup() {
   let dataUri = Services.io.newURI("data:text/plain;base64," +
                                    btoa(messages[0].toMessageString()),
                                    null, null);
-  let imapMsg = new imapMessage(dataUri.spec, gIMAPMailbox.uidnext++, []);
-  gIMAPMailbox.addMessage(imapMsg);
+  let imapMsg = new imapMessage(dataUri.spec, IMAPPump.mailbox.uidnext++, []);
+  IMAPPump.mailbox.addMessage(imapMsg);
 
   dataUri = Services.io.newURI("data:text/plain;base64," +
                                btoa(messages[1].toMessageString()),
                                null, null);
-  imapMsg = new imapMessage(dataUri.spec, gIMAPMailbox.uidnext++, []);
-  gIMAPMailbox.addMessage(imapMsg);
+  imapMsg = new imapMessage(dataUri.spec, IMAPPump.mailbox.uidnext++, []);
+  IMAPPump.mailbox.addMessage(imapMsg);
 
   // Extend local IMAP inbox to over 4 GiB.
   let outputStream = Cc["@mozilla.org/network/file-output-stream;1"]
                        .createInstance(Ci.nsIFileOutputStream)
                        .QueryInterface(Ci.nsISeekableStream);
   // Open in write-only mode, no truncate.
-  outputStream.init(gIMAPInbox.filePath, 0x02, -1, 0);
+  outputStream.init(IMAPPump.inbox.filePath, 0x02, -1, 0);
   // seek to 15 bytes past 4GB.
   outputStream.seek(0, 0x10000000f);
   // Write an empty "from" line.
@@ -89,28 +87,28 @@ function setup() {
   outputStream.close();
 
   // Save initial file size.
-  gOfflineStoreSize = gIMAPInbox.filePath.fileSize;
+  gOfflineStoreSize = IMAPPump.inbox.filePath.fileSize;
   do_print("Offline store size (before 1st downloadAllForOffline()) = " +
            gOfflineStoreSize);
 
   // Download for offline use, to append created messages to local IMAP inbox.
-  gIMAPInbox.downloadAllForOffline(asyncUrlListener, null);
+  IMAPPump.inbox.downloadAllForOffline(asyncUrlListener, null);
   yield false;
 }
 
 function check_result() {
   // Call downloadAllForOffline() a second time.
-  gIMAPInbox.downloadAllForOffline(asyncUrlListener, null);
+  IMAPPump.inbox.downloadAllForOffline(asyncUrlListener, null);
   yield false;
 
   // Make sure offline store grew (i.e., we were not writing over data).
-  let offlineStoreSize = gIMAPInbox.filePath.fileSize;
+  let offlineStoreSize = IMAPPump.inbox.filePath.fileSize;
   do_print("Offline store size (after 2nd downloadAllForOffline()) = " +
            offlineStoreSize + ". (Msg hdr offsets should be close to it.)");
   do_check_true(offlineStoreSize > gOfflineStoreSize);
 
   // Verify that the message headers have the offline flag set.
-  let msgEnumerator = gIMAPInbox.msgDatabase.EnumerateMessages();
+  let msgEnumerator = IMAPPump.inbox.msgDatabase.EnumerateMessages();
   let offset = {};
   let size = {};
   while (msgEnumerator.hasMoreElements()) {
@@ -120,7 +118,7 @@ function check_result() {
           (header.flags & Ci.nsMsgMessageFlags.Offline)))
       do_throw("Message not downloaded for offline use");
 
-    gIMAPInbox.getOfflineFileStream(header.messageKey, offset, size).close();
+    IMAPPump.inbox.getOfflineFileStream(header.messageKey, offset, size).close();
     do_print("Msg hdr offset = " + offset.value);
   }
 };
@@ -128,8 +126,8 @@ function check_result() {
 function teardown() {
   // Free up disk space - if you want to look at the file after running
   // this test, comment out this line.
-  if (gIMAPInbox)
-    gIMAPInbox.filePath.remove(false);
+  if (IMAPPump.inbox)
+    IMAPPump.inbox.filePath.remove(false);
 
   teardownIMAPPump();
 }
