@@ -28,6 +28,7 @@
 #include "EudoraDebugLog.h"
 #include "prmem.h"
 #include "plstr.h"
+#include <algorithm>
 
 static NS_DEFINE_IID(kISupportsIID,      NS_ISUPPORTS_IID);
 
@@ -202,7 +203,7 @@ nsresult nsEudoraWin32::ScanMailDir(nsIFile *pFolder, nsISupportsArray *pArray,
       inputStream->Close();
       return rv;
     }
-    uint32_t bytesLeft = NS_MIN(PR_UINT32_MAX - 1, bytesLeft64);
+    uint32_t bytesLeft = std::min(uint64_t(PR_UINT32_MAX - 1), bytesLeft64);
     pContents = (char *) PR_Malloc(bytesLeft + 1);
     if (!pContents)
       return NS_ERROR_OUT_OF_MEMORY;
@@ -835,7 +836,7 @@ bool nsEudoraWin32::BuildPOPAccount(nsIMsgAccountManager *accMgr, const char *pS
         IMPORT_LOG0("Created a new account and set the incoming server to the POP3 server.\n");
 
         nsCOMPtr<nsIPop3IncomingServer> pop3Server = do_QueryInterface(in, &rv);
-        NS_ENSURE_SUCCESS(rv,rv);
+        NS_ENSURE_SUCCESS(rv,false);
         UINT valInt = ::GetPrivateProfileInt(pSection, "LeaveMailOnServer", 0, pIni);
         pop3Server->SetLeaveMessagesOnServer(valInt ? true : false);
 
@@ -968,7 +969,7 @@ void nsEudoraWin32::SetSmtpServer(nsIMsgAccountManager *pMgr, nsIMsgAccount *pAc
     }
     nsCOMPtr<nsISmtpServer>    smtpServer;
 
-    rv = smtpService->CreateSmtpServer(getter_AddRefs(smtpServer));
+    rv = smtpService->CreateServer(getter_AddRefs(smtpServer));
     if (NS_SUCCEEDED(rv) && smtpServer)
     {
       smtpServer->SetHostname(nsDependentCString(pServer));
@@ -1079,7 +1080,7 @@ bool nsEudoraWin32::FindMimeIniFile(nsIFile *pFile)
   bool hasMore;
   nsCOMPtr<nsISimpleEnumerator> directoryEnumerator;
   nsresult rv = pFile->GetDirectoryEntries(getter_AddRefs(directoryEnumerator));
-  NS_ENSURE_SUCCESS(rv, rv);
+  NS_ENSURE_SUCCESS(rv, false);
 
   directoryEnumerator->HasMoreElements(&hasMore);
   bool              isFile;
