@@ -51,7 +51,6 @@
 #include "nsIMsgCopyService.h"
 #include "nsICryptoHash.h"
 #include "nsIStringBundle.h"
-#include "nsLocalStrings.h"
 #include "nsIMsgFilterPlugin.h"
 #include "nsIMutableArray.h"
 #include "nsArrayUtils.h"
@@ -118,8 +117,7 @@ NS_IMETHODIMP nsMsgMailboxParser::OnStartRequest(nsIRequest *request, nsISupport
           path->GetFileSize(&fileSize);
             // the size of the mailbox file is our total base line for measuring progress
             m_graph_progress_total = (uint32_t) fileSize;
-            UpdateStatusText(LOCAL_STATUS_SELECTING_MAILBOX);
-
+            UpdateStatusText("buildingSummary");
             nsCOMPtr<nsIMsgDBService> msgDBService = do_GetService(NS_MSGDB_SERVICE_CONTRACTID, &rv);
             if (msgDBService)
             {
@@ -176,7 +174,7 @@ NS_IMETHODIMP nsMsgMailboxParser::OnStopRequest(nsIRequest *request, nsISupports
     // be sure to clear any status text and progress info..
     m_graph_progress_received = 0;
     UpdateProgressPercent();
-    UpdateStatusText(LOCAL_STATUS_DOCUMENT_DONE);
+    UpdateStatusText("localStatusDocumentDone");
 
     return NS_OK;
 }
@@ -279,7 +277,7 @@ nsresult nsMsgMailboxParser::Init()
   return AcquireFolderLock();
 }
 
-void nsMsgMailboxParser::UpdateStatusText (uint32_t stringID)
+void nsMsgMailboxParser::UpdateStatusText (const char* stringName)
 {
   if (m_statusFeedback)
   {
@@ -293,14 +291,9 @@ void nsMsgMailboxParser::UpdateStatusText (uint32_t stringID)
     if (NS_FAILED(rv))
       return;
     nsString finalString;
-    if (stringID == LOCAL_STATUS_SELECTING_MAILBOX)
-    {
-      const PRUnichar * stringArray[] = { m_folderName.get() };
-      rv = bundle->FormatStringFromID(stringID, stringArray, 1,
-                                      getter_Copies(finalString));
-    }
-    else
-      bundle->GetStringFromID(stringID, getter_Copies(finalString));
+    const PRUnichar * stringArray[] = { m_folderName.get() };
+    rv = bundle->FormatStringFromName(NS_ConvertASCIItoUTF16(stringName).get(),
+                                      stringArray, 1, getter_Copies(finalString));
     m_statusFeedback->ShowStatusString(finalString);
   }
 }
@@ -500,7 +493,7 @@ nsMsgMailboxParser::ReleaseFolderLock()
   bool haveSemaphore;
   nsCOMPtr <nsISupports> supports = do_QueryInterface(static_cast<nsIMsgParseMailMsgState*>(this));
   result = folder->TestSemaphore(supports, &haveSemaphore);
-  if(NS_SUCCEEDED(result) && haveSemaphore)
+  if (NS_SUCCEEDED(result) && haveSemaphore)
     (void) folder->ReleaseSemaphore(supports);
 }
 
@@ -886,7 +879,7 @@ void nsParseMailMessageState::GetAggregateHeader (nsVoidArray &list, struct mess
 
   if (length > 0)
   {
-    char *value = (char*) PR_MALLOC (length + 1); //+1 for null term
+    char *value = (char*) PR_CALLOC (length + 1); //+1 for null term
     if (value)
     {
       // Catenate all the To lines together, separated by commas
@@ -895,7 +888,7 @@ void nsParseMailMessageState::GetAggregateHeader (nsVoidArray &list, struct mess
       for (i = 0; i < size; i++)
       {
         header = (struct message_header*) list.ElementAt(i);
-        PL_strcat (value, header->value);
+        PL_strncat (value, header->value, header->length);
         if (i + 1 < size)
           PL_strcat (value, ",");
       }
@@ -935,10 +928,11 @@ nsresult nsParseMailMessageState::StartNewEnvelope(const char *line, uint32_t li
 nsresult nsParseMailMessageState::ParseHeaders ()
 {
   char *buf = m_headers.GetBuffer();
-  char *buf_end = buf + m_headers.GetBufferPos();
+  uint32_t buf_length = m_headers.GetBufferPos();
+  char *buf_end = buf + buf_length;
   while (buf < buf_end)
   {
-    char *colon = PL_strchr (buf, ':');
+    char *colon = PL_strnchr(buf, ':', buf_length);
     char *end;
     char *value = 0;
     struct message_header *header = 0;
@@ -1051,8 +1045,8 @@ nsresult nsParseMailMessageState::ParseHeaders ()
 #endif
 
       ToLowerCase(headerStr);
-      int32_t customHeaderIndex = m_customDBHeaders.IndexOf(headerStr);
-      if (customHeaderIndex != -1)
+      uint32_t customHeaderIndex = m_customDBHeaders.IndexOf(headerStr);
+      if (customHeaderIndex != m_customDBHeaders.NoIndex)
         header = & m_customDBHeaderValues[customHeaderIndex];
     }
 
@@ -1144,13 +1138,14 @@ SEARCH_NEWLINE:
             PRTime resultTime;
             if (PR_ParseTimeString (receivedDate.get(), false, &resultTime) == PR_SUCCESS)
               m_receivedTime = resultTime;
+            else
+              NS_WARNING("PR_ParseTimeString failed in ParseHeaders().");
           }
         }
         // Someone might want the received header saved.
         if (m_customDBHeaders.Length())
         {
-          uint32_t customHeaderIndex = m_customDBHeaders.IndexOf(NS_LITERAL_CSTRING("received"));
-          if (customHeaderIndex != -1)
+          if (m_customDBHeaders.Contains(NS_LITERAL_CSTRING("received")))
           {
             if (!m_receivedValue.IsEmpty())
               m_receivedValue.Append(' ');
@@ -1183,7 +1178,9 @@ nsresult nsParseMailMessageState::ParseEnvelope (const char *line, uint32_t line
     s++;
   m_envelope_date.value = s;
   m_envelope_date.length = (uint16_t) (line_size - (s - m_envelope.GetBuffer()));
-  while (IS_SPACE (m_envelope_date.value [m_envelope_date.length - 1]))
+
+  while (m_envelope_date.length > 0 &&
+         IS_SPACE (m_envelope_date.value [m_envelope_date.length - 1]))
     m_envelope_date.length--;
 
   /* #### short-circuit const */
@@ -1307,7 +1304,7 @@ nsresult nsParseMailMessageState::FinalizeHeaders()
   recipient  = (to.length         ? &to :
   cc.length         ? &cc :
   m_newsgroups.length ? &m_newsgroups :
-  sender);
+  0);
   ccList     = (cc.length ? &cc : 0);
   bccList    = (m_bccList.length    ? &m_bccList    : 0);
   subject    = (m_subject.length    ? &m_subject    : 0);
@@ -1362,9 +1359,12 @@ nsresult nsParseMailMessageState::FinalizeHeaders()
     /* Take off <> around message ID. */
     if (id)
     {
-      if (id->value[0] == '<')
-        id->value++, id->length--;
-      if (id->value[id->length - 1] == '>')
+      if (id->length > 0 && id->value[0] == '<')
+        id->length--, id->value++;
+
+      NS_WARN_IF_FALSE(id->length > 0, "id->length failure in FinalizeHeaders().");
+
+      if (id->length > 0 && id->value[id->length - 1] == '>')
         /* generate a new null-terminated string without the final > */
         rawMsgId.Assign(id->value, id->length - 1);
       else
@@ -1558,6 +1558,9 @@ nsresult nsParseMailMessageState::FinalizeHeaders()
             case 'U': case 'u':
               m_newMsgHdr->SetFlags(msgFlags & ~nsMsgMessageFlags::Read);
               break;
+            default:            // Should check for corrupt file.
+              NS_ERROR("Corrupt file. Should not happen.");
+              break;
             }
           }
         }
@@ -1591,6 +1594,8 @@ nsresult nsParseMailMessageState::FinalizeHeaders()
             m_newMsgHdr->SetDate(resultTime);
             PRTime2Seconds(resultTime, &rcvTimeSecs);
           }
+          else
+            NS_WARNING("PR_ParseTimeString of date failed in FinalizeHeader().");
         }
         else
         {  // PR_Now()
@@ -1613,6 +1618,8 @@ nsresult nsParseMailMessageState::FinalizeHeaders()
           PRStatus timeStatus = PR_ParseTimeString (deliveryDate->value, false, &resultTime);
           if (PR_SUCCESS == timeStatus)
             PRTime2Seconds(resultTime, &rcvTimeSecs);
+          else // TODO/FIXME: We need to figure out what to do in this case!
+            NS_WARNING("PR_ParseTimeString of delivery date failed in FinalizeHeader().");
         }
         m_newMsgHdr->SetUint32Property("dateReceived", rcvTimeSecs);
 
@@ -1931,7 +1938,7 @@ nsresult nsParseNewMailState::GetTrashFolder(nsIMsgFolder **pTrashFolder)
   if (!pTrashFolder)
     return NS_ERROR_NULL_POINTER;
 
-  if(m_downloadFolder)
+  if (m_downloadFolder)
   {
     nsCOMPtr <nsIMsgIncomingServer> incomingServer;
     m_downloadFolder->GetServer(getter_AddRefs(incomingServer));
@@ -2580,7 +2587,7 @@ nsresult nsParseNewMailState::MoveIncorporatedMessage(nsIMsgDBHdr *mailHdr,
 
   if (movedMsgIsNew)
     destIFolder->SetHasNewMessages(true);
-  if (m_filterTargetFolders.IndexOf(destIFolder) == -1)
+  if (!m_filterTargetFolders.Contains(destIFolder))
     m_filterTargetFolders.AppendObject(destIFolder);
 
   destIFolder->ReleaseSemaphore (myISupports);
