@@ -20,7 +20,7 @@ function run_test()
 function test_keywords(aMessageHeaderKeys, aStatus) {
   let headerKeys = aMessageHeaderKeys;
   do_check_neq(headerKeys, null);
-  let copiedMessage = gLocalInboxFolder.GetMessageHeader(headerKeys[0]);
+  let copiedMessage = localAccountUtils.inboxFolder.GetMessageHeader(headerKeys[0]);
   do_check_eq(copiedMessage.getStringProperty("keywords"), tag1);
   do_test_finished();
 }
