@@ -336,7 +336,7 @@ LONG CMapiImp::InitContext(unsigned long session, MsgMapiListContext **listConte
 {
   nsMAPIConfiguration * pMapiConfig = nsMAPIConfiguration::GetMAPIConfiguration() ;
   if (!pMapiConfig)
-    return NS_ERROR_FAILURE ;  // get the singelton obj
+    return MAPI_E_FAILURE ;  // get the singelton obj
   *listContext = (MsgMapiListContext *) pMapiConfig->GetMapiListContext(session);
   // This is the first message
   if (!*listContext)
@@ -382,7 +382,7 @@ STDMETHODIMP CMapiImp::FindNext(unsigned long aSession, unsigned long ulUIParam,
   if (!pMapiConfig) 
   {
     NS_ASSERTION(false, "failed to get config in findnext");
-    return NS_ERROR_FAILURE ;  // get the singelton obj
+    return MAPI_E_FAILURE ;  // get the singelton obj
   }
   MsgMapiListContext *listContext;
   LONG ret = InitContext(aSession, &listContext);
