@@ -1,39 +1,7 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsCOMPtr.h"
 #include "nsIMimeEmitter.h"
@@ -64,7 +32,7 @@ static int MimeMessage_initialize (MimeObject *);
 static void MimeMessage_finalize (MimeObject *);
 static int MimeMessage_add_child (MimeObject *, MimeObject *);
 static int MimeMessage_parse_begin (MimeObject *);
-static int MimeMessage_parse_line (const char *, PRInt32, MimeObject *);
+static int MimeMessage_parse_line (const char *, int32_t, MimeObject *);
 static int MimeMessage_parse_eof (MimeObject *, bool);
 static int MimeMessage_close_headers (MimeObject *obj);
 static int MimeMessage_write_headers_html (MimeObject *);
@@ -77,7 +45,7 @@ extern void MimeHeaders_do_unix_display_hook_hack(MimeHeaders *);
 #endif /* XP_UNIX */
 
 #if defined(DEBUG) && defined(XP_UNIX)
-static int MimeMessage_debug_print (MimeObject *, PRFileDesc *, PRInt32 depth);
+static int MimeMessage_debug_print (MimeObject *, PRFileDesc *, int32_t depth);
 #endif
 
 extern MimeObjectClass mimeMultipartClass;
@@ -109,6 +77,7 @@ MimeMessage_initialize (MimeObject *object)
   MimeMessage *msg = (MimeMessage *)object;
   msg->grabSubject = false;
   msg->bodyLength = 0;
+  msg->sizeSoFar = 0;
 
   return ((MimeObjectClass*)&MIME_SUPERCLASS)->initialize(object);
 }
@@ -143,10 +112,10 @@ MimeMessage_parse_begin (MimeObject *obj)
 
 
 static int
-MimeMessage_parse_line (const char *aLine, PRInt32 aLength, MimeObject *obj)
+MimeMessage_parse_line (const char *aLine, int32_t aLength, MimeObject *obj)
 {
   const char * line = aLine;
-  PRInt32 length = aLength;
+  int32_t length = aLength;
 
   MimeMessage *msg = (MimeMessage *) obj;
   int status = 0;
@@ -154,6 +123,8 @@ MimeMessage_parse_line (const char *aLine, PRInt32 aLength, MimeObject *obj)
   NS_ASSERTION(line && *line, "empty line in mime msg parse_line");
   if (!line || !*line) return -1;
 
+  msg->sizeSoFar += length;
+
   if (msg->grabSubject)
   {
     if ( (!PL_strncasecmp(line, "Subject: ", 9)) && (obj->parent) )
@@ -187,7 +158,7 @@ MimeMessage_parse_line (const char *aLine, PRInt32 aLength, MimeObject *obj)
     PR_ASSERT(kid);
     if (!kid) return -1;
 
-          msg->bodyLength += length;
+    msg->bodyLength += length;
 
     /* Don't allow MimeMessage objects to not end in a newline, since it
      would be inappropriate for any following part to appear on the same
@@ -301,7 +272,7 @@ MimeMessage_close_headers (MimeObject *obj)
     nsCString contentType;
     contentType.Adopt(MimeHeaders_get(msg->hdrs, HEADER_CONTENT_TYPE, false, false));
     if (!contentType.IsEmpty())
-      charset.Adopt(MimeHeaders_get_parameter(contentType.get(), "charset", nsnull, nsnull));
+      charset.Adopt(MimeHeaders_get_parameter(contentType.get(), "charset", nullptr, nullptr));
 
     // If we've got a charset, use nsMsgI18NConvertToUnicode to magically decode
     // the munged subject.
@@ -314,10 +285,10 @@ MimeMessage_close_headers (MimeObject *obj)
       if (NS_SUCCEEDED(rv))
         obj->headers->munged_subject = ToNewUTF8String(dest);
       else
-        obj->headers->munged_subject = nsnull;
+        obj->headers->munged_subject = nullptr;
     } else {
       PR_Free(obj->headers->munged_subject);
-      obj->headers->munged_subject = nsnull;
+      obj->headers->munged_subject = nullptr;
     }
   }
 
@@ -544,7 +515,7 @@ MimeMessage_close_headers (MimeObject *obj)
                  false);
   if (xmoz)
   {
-    PRUint32 flags = 0;
+    uint32_t flags = 0;
     char dummy = 0;
     if (sscanf(xmoz, " %x %c", &flags, &dummy) == 1 &&
       flags & nsMsgMessageFlags::Partial)
@@ -674,7 +645,7 @@ MimeMessage_add_child (MimeObject *parent, MimeObject *child)
 char *
 DetermineMailCharset(MimeMessage *msg)
 {
-  char          *retCharset = nsnull;
+  char          *retCharset = nullptr;
 
   if ( (msg) && (msg->hdrs) )
   {
@@ -846,17 +817,16 @@ MimeMessage_write_headers_html (MimeObject *obj)
 
 static char *
 MimeMessage_partial_message_html(const char *data, void *closure,
-                 MimeHeaders *headers)
+                                 MimeHeaders *headers)
 {
   MimeMessage *msg = (MimeMessage *)closure;
-  nsCAutoString orig_url(data);
+  nsAutoCString orig_url(data);
   char *uidl = MimeHeaders_get(headers, HEADER_X_UIDL, false, false);
   char *msgId = MimeHeaders_get(headers, HEADER_MESSAGE_ID, false,
                   false);
   char *msgIdPtr = PL_strchr(msgId, '<');
-  int msgBase;
 
-  PRInt32 pos = orig_url.Find("mailbox-message");
+  int32_t pos = orig_url.Find("mailbox-message");
   if (pos != -1)
     orig_url.Cut(pos + 7, 8);
 
@@ -872,17 +842,30 @@ MimeMessage_partial_message_html(const char *data, void *closure,
   if (gtPtr)
     *gtPtr = 0;
 
-  msgBase = (msg->bodyLength > MSG_LINEBREAK_LEN) ? MIME_MSG_PARTIAL_FMT_1 : MIME_MSG_PARTIAL_FMT2_1;
+  bool msgBaseTruncated = (msg->bodyLength > MSG_LINEBREAK_LEN);
 
   nsCString partialMsgHtml;
   nsCString item;
 
-  item.Adopt(MimeGetStringByID(msgBase));
-  partialMsgHtml += item;
+  partialMsgHtml.AppendLiteral("<div style=\"margin: 1em auto; border: 1px solid black; width: 80%\">");
+  partialMsgHtml.AppendLiteral("<div style=\"margin: 5px; padding: 10px; border: 1px solid gray; font-weight: bold; text-align: center;\">");
 
-  item.Adopt(MimeGetStringByID(msgBase+1));
+  partialMsgHtml.AppendLiteral("<span style=\"font-size: 120%;\">");
+  if (msgBaseTruncated)
+    item.Adopt(MimeGetStringByName(NS_LITERAL_STRING("MIME_MSG_PARTIAL_TRUNCATED").get()));
+  else
+    item.Adopt(MimeGetStringByName(NS_LITERAL_STRING("MIME_MSG_PARTIAL_NOT_DOWNLOADED").get()));
   partialMsgHtml += item;
+  partialMsgHtml.AppendLiteral("</span><hr>");
 
+  if (msgBaseTruncated)
+    item.Adopt(MimeGetStringByName(NS_LITERAL_STRING("MIME_MSG_PARTIAL_TRUNCATED_EXPLANATION").get()));
+  else
+    item.Adopt(MimeGetStringByName(NS_LITERAL_STRING("MIME_MSG_PARTIAL_NOT_DOWNLOADED_EXPLANATION").get()));
+  partialMsgHtml += item;
+  partialMsgHtml.AppendLiteral("<br><br>");
+
+  partialMsgHtml.AppendLiteral("<a href=\"");
   partialMsgHtml.Append(orig_url);
 
   if (msgIdPtr) {
@@ -903,15 +886,19 @@ MimeMessage_partial_message_html(const char *data, void *closure,
     partialMsgHtml.Append(item);
   }
 
-  item.Adopt(MimeGetStringByID(msgBase+2));
+  partialMsgHtml.AppendLiteral("\">");
+  item.Adopt(MimeGetStringByName(NS_LITERAL_STRING("MIME_MSG_PARTIAL_CLICK_FOR_REST").get()));
   partialMsgHtml += item;
+  partialMsgHtml.AppendLiteral("</a>");
+
+  partialMsgHtml.AppendLiteral("</div></div>");
 
   return ToNewCString(partialMsgHtml);
 }
 
 #if defined(DEBUG) && defined(XP_UNIX)
 static int
-MimeMessage_debug_print (MimeObject *obj, PRFileDesc *stream, PRInt32 depth)
+MimeMessage_debug_print (MimeObject *obj, PRFileDesc *stream, int32_t depth)
 {
   MimeMessage *msg = (MimeMessage *) obj;
   char *addr = mime_part_address(obj);
@@ -923,7 +910,7 @@ MimeMessage_debug_print (MimeObject *obj, PRFileDesc *stream, PRInt32 depth)
       obj->clazz->class_name,
       addr ? addr : "???",
       (msg->container.nchildren == 0 ? " (no body)" : ""),
-      (PRUint32) msg);
+      (uint32_t) msg);
 */
   PR_FREEIF(addr);
 
