@@ -93,6 +93,8 @@ nsNntpIncomingServer::nsNntpIncomingServer()
 {
   mNewsrcHasChanged = false;
 
+  mGetOnlyNew = true;
+
   mHostInfoLoaded = false;
   mHostInfoHasChanged = false;
   mVersion = INVALID_VERSION;
@@ -1071,7 +1073,7 @@ NS_IMETHODIMP
 nsNntpIncomingServer::SetAsSubscribed(const nsACString &path)
 {
     mTempSubscribed.AppendElement(path);
-    if (mGetOnlyNew && (mGroupsOnServer.IndexOf(path) == mGroupsOnServer.NoIndex))
+    if (mGetOnlyNew && (!mGroupsOnServer.Contains(path)))
       return NS_OK;
 
     nsresult rv = EnsureInner();
@@ -1566,7 +1568,7 @@ nsNntpIncomingServer::GroupNotFound(nsIMsgWindow *aMsgWindow,
   NS_ENSURE_SUCCESS(rv,rv);
 
   nsCString hostname;
-  rv = GetHostName(hostname);
+  rv = GetRealHostName(hostname);
   NS_ENSURE_SUCCESS(rv,rv);
 
   NS_ConvertUTF8toUTF16 hostStr(hostname);
@@ -1707,19 +1709,18 @@ nsNntpIncomingServer::SetSelection(nsITreeSelection * aSelection)
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::GetRowProperties(int32_t index, nsISupportsArray *properties)
+nsNntpIncomingServer::GetRowProperties(int32_t index, nsAString& properties)
 {
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::GetCellProperties(int32_t row, nsITreeColumn* col, nsISupportsArray *properties)
+nsNntpIncomingServer::GetCellProperties(int32_t row, nsITreeColumn* col, nsAString& properties)
 {
     if (!IsValidRow(row))
       return NS_ERROR_UNEXPECTED;
 
     NS_ENSURE_ARG_POINTER(col);
-    NS_ENSURE_ARG_POINTER(properties);
 
     const PRUnichar* colID;
     col->GetIdConst(&colID);
@@ -1729,20 +1730,20 @@ nsNntpIncomingServer::GetCellProperties(int32_t row, nsITreeColumn* col, nsISupp
         // in the "subscribedCol"
         if (mSearchResultSortDescending)
           row = mSubscribeSearchResult.Length() - 1 - row;
-        if (mTempSubscribed.IndexOf(mSubscribeSearchResult.ElementAt(row)) != mTempSubscribed.NoIndex) {
-          properties->AppendElement(mSubscribedAtom);
+        if (mTempSubscribed.Contains(mSubscribeSearchResult.ElementAt(row))) {
+          properties.AssignLiteral("subscribed");
         }
     }
     else if (colID[0] == 'n') {
       // add the "nntp" property to the "nameCol"
       // so we get the news folder icon in the search view
-      properties->AppendElement(mNntpAtom);
+      properties.AssignLiteral("nntp");
     }
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::GetColumnProperties(nsITreeColumn* col, nsISupportsArray *properties)
+nsNntpIncomingServer::GetColumnProperties(nsITreeColumn* col, nsAString& properties)
 {
     return NS_OK;
 }
