@@ -17,6 +17,11 @@
  * browser/base/content/test/browser_sanitize-timespans.js.
  */
 
+Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
+
+XPCOMUtils.defineLazyModuleGetter(this, "FormHistory",
+                                  "resource://gre/modules/FormHistory.jsm");
+
 let tempScope = {};
 Cc["@mozilla.org/moz/jssubscript-loader;1"].getService(Ci.mozIJSSubScriptLoader)
                                            .loadSubScript("chrome://browser/content/sanitize.js", tempScope);
@@ -24,11 +29,11 @@ let Sanitizer = tempScope.Sanitizer;
 
 const dm = Cc["@mozilla.org/download-manager;1"].
            getService(Ci.nsIDownloadManager);
-const formhist = Cc["@mozilla.org/satchel/form-history;1"].
-                 getService(Ci.nsIFormHistory2);
 
 const kUsecPerMin = 60 * 1000000;
 
+let formEntries;
+
 // Add tests here.  Each is a function that's called by doNextTest().
 var gAllTests = [
 
@@ -54,30 +59,37 @@ var gAllTests = [
   function () {
     // Add history (within the past hour)
     let uris = [];
+    let places = [];
+    let pURI;
     for (let i = 0; i < 30; i++) {
-      uris.push(addHistoryWithMinutesAgo(i));
+      pURI = makeURI("http://" + i + "-minutes-ago.com/");
+      places.push({uri: pURI, visitDate: visitTimeForMinutesAgo(i)});
+      uris.push(pURI);
     }
 
-    let wh = new WindowHelper();
-    wh.onload = function () {
-      this.selectDuration(Sanitizer.TIMESPAN_HOUR);
-      this.checkPrefCheckbox("history", false);
-      this.checkDetails(false);
+    addVisits(places, function() {
+      let wh = new WindowHelper();
+      wh.onload = function () {
+        this.selectDuration(Sanitizer.TIMESPAN_HOUR);
+        this.checkPrefCheckbox("history", false);
+        this.checkDetails(false);
 
-      // Show details
-      this.toggleDetails();
-      this.checkDetails(true);
+        // Show details
+        this.toggleDetails();
+        this.checkDetails(true);
 
-      // Hide details
-      this.toggleDetails();
-      this.checkDetails(false);
-      this.cancelDialog();
-
-      ensureHistoryClearedState(uris, false);
-      blankSlate();
-      ensureHistoryClearedState(uris, true);
-    };
-    wh.open();
+        // Hide details
+        this.toggleDetails();
+        this.checkDetails(false);
+        this.cancelDialog();
+      };
+      wh.onunload = function () {
+        yield promiseHistoryClearedState(uris, false);
+        yield blankSlate();
+        yield promiseHistoryClearedState(uris, true);
+      };
+      wh.open();
+    });
   },
 
   /**
@@ -85,56 +97,85 @@ var gAllTests = [
    * visits and downloads when checked; the dialog respects simple timespan.
    */
   function () {
-    // Add history and downloads (within the past hour).
+    // Add history (within the past hour).
     let uris = [];
+    let places = [];
+    let pURI;
     for (let i = 0; i < 30; i++) {
-      uris.push(addHistoryWithMinutesAgo(i));
+      pURI = makeURI("http://" + i + "-minutes-ago.com/");
+      places.push({uri: pURI, visitDate: visitTimeForMinutesAgo(i)});
+      uris.push(pURI);
     }
-    let downloadIDs = [];
-    for (let i = 0; i < 5; i++) {
-      downloadIDs.push(addDownloadWithMinutesAgo(i));
-    }
-    // Add history and downloads (over an hour ago).
+    // Add history (over an hour ago).
     let olderURIs = [];
     for (let i = 0; i < 5; i++) {
-      olderURIs.push(addHistoryWithMinutesAgo(61 + i));
+      pURI = makeURI("http://" + (61 + i) + "-minutes-ago.com/");
+      places.push({uri: pURI, visitDate: visitTimeForMinutesAgo(61 + i)});
+      olderURIs.push(pURI);
     }
-    let olderDownloadIDs = [];
-    for (let i = 0; i < 5; i++) {
-      olderDownloadIDs.push(addDownloadWithMinutesAgo(61 + i));
-    }
-    let totalHistoryVisits = uris.length + olderURIs.length;
 
-    let wh = new WindowHelper();
-    wh.onload = function () {
-      this.selectDuration(Sanitizer.TIMESPAN_HOUR);
-      this.checkPrefCheckbox("history", true);
-      this.acceptDialog();
+    addVisits(places, function() {
+      // Add downloads (within the past hour).
+      let downloadIDs = [];
+      for (let i = 0; i < 5; i++) {
+        downloadIDs.push(addDownloadWithMinutesAgo(i));
+      }
+      // Add downloads (over an hour ago).
+      let olderDownloadIDs = [];
+      for (let i = 0; i < 5; i++) {
+        olderDownloadIDs.push(addDownloadWithMinutesAgo(61 + i));
+      }
+      let totalHistoryVisits = uris.length + olderURIs.length;
 
-      intPrefIs("sanitize.timeSpan", Sanitizer.TIMESPAN_HOUR,
-                "timeSpan pref should be hour after accepting dialog with " +
-                "hour selected");
-      boolPrefIs("cpd.history", true,
-                 "history pref should be true after accepting dialog with " +
-                 "history checkbox checked");
-      boolPrefIs("cpd.downloads", true,
-                 "downloads pref should be true after accepting dialog with " +
-                 "history checkbox checked");
+      let wh = new WindowHelper();
+      wh.onload = function () {
+        this.selectDuration(Sanitizer.TIMESPAN_HOUR);
+        this.checkPrefCheckbox("history", true);
+        this.acceptDialog();
 
-      // History visits and downloads within one hour should be cleared.
-      ensureHistoryClearedState(uris, true);
-      ensureDownloadsClearedState(downloadIDs, true);
+        intPrefIs("sanitize.timeSpan", Sanitizer.TIMESPAN_HOUR,
+                  "timeSpan pref should be hour after accepting dialog with " +
+                  "hour selected");
+        boolPrefIs("cpd.history", true,
+                   "history pref should be true after accepting dialog with " +
+                   "history checkbox checked");
+        boolPrefIs("cpd.downloads", true,
+                   "downloads pref should be true after accepting dialog with " +
+                   "history checkbox checked");
+      };
+      wh.onunload = function () {
+        // History visits and downloads within one hour should be cleared.
+        yield promiseHistoryClearedState(uris, true);
+        ensureDownloadsClearedState(downloadIDs, true);
 
-      // Visits and downloads > 1 hour should still exist.
-      ensureHistoryClearedState(olderURIs, false);
-      ensureDownloadsClearedState(olderDownloadIDs, false);
+        // Visits and downloads > 1 hour should still exist.
+        yield promiseHistoryClearedState(olderURIs, false);
+        ensureDownloadsClearedState(olderDownloadIDs, false);
 
-      // OK, done, cleanup after ourselves.
-      blankSlate();
-      ensureHistoryClearedState(olderURIs, true);
-      ensureDownloadsClearedState(olderDownloadIDs, true);
-    };
-    wh.open();
+        // OK, done, cleanup after ourselves.
+        yield blankSlate();
+        yield promiseHistoryClearedState(olderURIs, true);
+        ensureDownloadsClearedState(olderDownloadIDs, true);
+      };
+      wh.open();
+    });
+  },
+
+  /**
+   * Add form history entries for the next test.
+   */
+  function () {
+    formEntries = [];
+
+    let iter = function() {
+      for (let i = 0; i < 5; i++) {
+        formEntries.push(addFormEntryWithMinutesAgo(iter, i));
+        yield;
+      }
+      doNextTest();
+    }();
+
+    iter.next();
   },
 
   /**
@@ -144,51 +185,59 @@ var gAllTests = [
   function () {
     // Add history, downloads, form entries (within the past hour).
     let uris = [];
+    let places = [];
+    let pURI;
     for (let i = 0; i < 5; i++) {
-      uris.push(addHistoryWithMinutesAgo(i));
-    }
-    let downloadIDs = [];
-    for (let i = 0; i < 5; i++) {
-      downloadIDs.push(addDownloadWithMinutesAgo(i));
-    }
-    let formEntries = [];
-    for (let i = 0; i < 5; i++) {
-      formEntries.push(addFormEntryWithMinutesAgo(i));
+      pURI = makeURI("http://" + i + "-minutes-ago.com/");
+      places.push({uri: pURI, visitDate: visitTimeForMinutesAgo(i)});
+      uris.push(pURI);
     }
 
-    let wh = new WindowHelper();
-    wh.onload = function () {
-      is(this.isWarningPanelVisible(), false,
-         "Warning panel should be hidden after previously accepting dialog " +
-         "with a predefined timespan");
-      this.selectDuration(Sanitizer.TIMESPAN_HOUR);
+    addVisits(places, function() {
+      let downloadIDs = [];
+      for (let i = 0; i < 5; i++) {
+        downloadIDs.push(addDownloadWithMinutesAgo(i));
+      }
 
-      // Remove only form entries, leave history (including downloads).
-      this.checkPrefCheckbox("history", false);
-      this.checkPrefCheckbox("formdata", true);
-      this.acceptDialog();
+      let wh = new WindowHelper();
+      wh.onload = function () {
+        is(this.isWarningPanelVisible(), false,
+           "Warning panel should be hidden after previously accepting dialog " +
+           "with a predefined timespan");
+        this.selectDuration(Sanitizer.TIMESPAN_HOUR);
 
-      intPrefIs("sanitize.timeSpan", Sanitizer.TIMESPAN_HOUR,
-                "timeSpan pref should be hour after accepting dialog with " +
-                "hour selected");
-      boolPrefIs("cpd.history", false,
-                 "history pref should be false after accepting dialog with " +
-                 "history checkbox unchecked");
-      boolPrefIs("cpd.downloads", false,
-                 "downloads pref should be false after accepting dialog with " +
-                 "history checkbox unchecked");
+        // Remove only form entries, leave history (including downloads).
+        this.checkPrefCheckbox("history", false);
+        this.checkPrefCheckbox("formdata", true);
+        this.acceptDialog();
 
-      // Of the three only form entries should be cleared.
-      ensureHistoryClearedState(uris, false);
-      ensureDownloadsClearedState(downloadIDs, false);
-      ensureFormEntriesClearedState(formEntries, true);
+        intPrefIs("sanitize.timeSpan", Sanitizer.TIMESPAN_HOUR,
+                  "timeSpan pref should be hour after accepting dialog with " +
+                  "hour selected");
+        boolPrefIs("cpd.history", false,
+                   "history pref should be false after accepting dialog with " +
+                   "history checkbox unchecked");
+        boolPrefIs("cpd.downloads", false,
+                   "downloads pref should be false after accepting dialog with " +
+                   "history checkbox unchecked");
+      };
+      wh.onunload = function () {
+        // Of the three only form entries should be cleared.
+        yield promiseHistoryClearedState(uris, false);
+        ensureDownloadsClearedState(downloadIDs, false);
 
-      // OK, done, cleanup after ourselves.
-      blankSlate();
-      ensureHistoryClearedState(uris, true);
-      ensureDownloadsClearedState(downloadIDs, true);
-    };
-    wh.open();
+        formEntries.forEach(function (entry) {
+          let exists = yield formNameExists(entry);
+          is(exists, false, "form entry " + entry + " should no longer exist");
+        });
+
+        // OK, done, cleanup after ourselves.
+        yield blankSlate();
+        yield promiseHistoryClearedState(uris, true);
+        ensureDownloadsClearedState(downloadIDs, true);
+      };
+      wh.open();
+    });
   },
 
   /**
@@ -197,36 +246,44 @@ var gAllTests = [
   function () {
     // Add history.
     let uris = [];
-    uris.push(addHistoryWithMinutesAgo(10));  // within past hour
-    uris.push(addHistoryWithMinutesAgo(70));  // within past two hours
-    uris.push(addHistoryWithMinutesAgo(130)); // within past four hours
-    uris.push(addHistoryWithMinutesAgo(250)); // outside past four hours
+    let places = [];
+    let pURI;
+    // within past hour, within past two hours, within past four hours and 
+    // outside past four hours
+    [10, 70, 130, 250].forEach(function(aValue) {
+      pURI = makeURI("http://" + aValue + "-minutes-ago.com/");
+      places.push({uri: pURI, visitDate: visitTimeForMinutesAgo(aValue)});
+      uris.push(pURI);
+    });
+    addVisits(places, function() {
+      let wh = new WindowHelper();
+      wh.onload = function () {
+        is(this.isWarningPanelVisible(), false,
+           "Warning panel should be hidden after previously accepting dialog " +
+           "with a predefined timespan");
+        this.selectDuration(Sanitizer.TIMESPAN_EVERYTHING);
+        this.checkPrefCheckbox("history", true);
+        this.checkDetails(true);
 
-    let wh = new WindowHelper();
-    wh.onload = function () {
-      is(this.isWarningPanelVisible(), false,
-         "Warning panel should be hidden after previously accepting dialog " +
-         "with a predefined timespan");
-      this.selectDuration(Sanitizer.TIMESPAN_EVERYTHING);
-      this.checkPrefCheckbox("history", true);
-      this.checkDetails(true);
+        // Hide details
+        this.toggleDetails();
+        this.checkDetails(false);
 
-      // Hide details
-      this.toggleDetails();
-      this.checkDetails(false);
+        // Show details
+        this.toggleDetails();
+        this.checkDetails(true);
 
-      // Show details
-      this.toggleDetails();
-      this.checkDetails(true);
+        this.acceptDialog();
 
-      this.acceptDialog();
-
-      intPrefIs("sanitize.timeSpan", Sanitizer.TIMESPAN_EVERYTHING,
-                "timeSpan pref should be everything after accepting dialog " +
-                "with everything selected");
-      ensureHistoryClearedState(uris, true);
-    };
-    wh.open();
+        intPrefIs("sanitize.timeSpan", Sanitizer.TIMESPAN_EVERYTHING,
+                  "timeSpan pref should be everything after accepting dialog " +
+                  "with everything selected");
+      };
+      wh.onunload = function () {
+        yield promiseHistoryClearedState(uris, true);
+      };
+      wh.open();
+    });
   },
 
   /**
@@ -236,26 +293,47 @@ var gAllTests = [
   function () {
     // Add history.
     let uris = [];
-    uris.push(addHistoryWithMinutesAgo(10));  // within past hour
-    uris.push(addHistoryWithMinutesAgo(70));  // within past two hours
-    uris.push(addHistoryWithMinutesAgo(130)); // within past four hours
-    uris.push(addHistoryWithMinutesAgo(250)); // outside past four hours
+    let places = [];
+    let pURI;
+    // within past hour, within past two hours, within past four hours and 
+    // outside past four hours
+    [10, 70, 130, 250].forEach(function(aValue) {
+      pURI = makeURI("http://" + aValue + "-minutes-ago.com/");
+      places.push({uri: pURI, visitDate: visitTimeForMinutesAgo(aValue)});
+      uris.push(pURI);
+    });
+    addVisits(places, function() {
+      let wh = new WindowHelper();
+      wh.onload = function () {
+        is(this.isWarningPanelVisible(), true,
+           "Warning panel should be visible after previously accepting dialog " +
+           "with clearing everything");
+        this.selectDuration(Sanitizer.TIMESPAN_EVERYTHING);
+        this.checkPrefCheckbox("history", true);
+        this.acceptDialog();
 
-    let wh = new WindowHelper();
-    wh.onload = function () {
-      is(this.isWarningPanelVisible(), true,
-         "Warning panel should be visible after previously accepting dialog " +
-         "with clearing everything");
-      this.selectDuration(Sanitizer.TIMESPAN_EVERYTHING);
-      this.checkPrefCheckbox("history", true);
-      this.acceptDialog();
+        intPrefIs("sanitize.timeSpan", Sanitizer.TIMESPAN_EVERYTHING,
+                  "timeSpan pref should be everything after accepting dialog " +
+                  "with everything selected");
+      };
+      wh.onunload = function () {
+        yield promiseHistoryClearedState(uris, true);
+      };
+      wh.open();
+    });
+  },
 
-      intPrefIs("sanitize.timeSpan", Sanitizer.TIMESPAN_EVERYTHING,
-                "timeSpan pref should be everything after accepting dialog " +
-                "with everything selected");
-      ensureHistoryClearedState(uris, true);
-    };
-    wh.open();
+  /**
+   * Add form history entry for the next test.
+   */
+  function () {
+    let iter = function() {
+      formEntries = [ addFormEntryWithMinutesAgo(iter, 10) ];
+      yield;
+      doNextTest();
+    }();
+
+    iter.next();
   },
 
   /**
@@ -265,29 +343,34 @@ var gAllTests = [
    */
   function () {
     // Add history.
-    let uris = [ addHistoryWithMinutesAgo(10) ];
-    let formEntries = [ addFormEntryWithMinutesAgo(10) ];
+    let pURI = makeURI("http://" + 10 + "-minutes-ago.com/");
+    addVisits({uri: pURI, visitDate: visitTimeForMinutesAgo(10)}, function() {
+      let uris = [ pURI ];
 
-    let wh = new WindowHelper();
-    wh.onload = function() {
-      // Check that the relevant checkboxes are enabled
-      var cb = this.win.document.querySelectorAll(
-                 "#itemList > [preference='privacy.cpd.formdata']");
-      ok(cb.length == 1 && !cb[0].disabled, "There is formdata, checkbox to " +
-         "clear formdata should be enabled.");
+      let wh = new WindowHelper();
+      wh.onload = function() {
+        // Check that the relevant checkboxes are enabled
+        var cb = this.win.document.querySelectorAll(
+                   "#itemList > [preference='privacy.cpd.formdata']");
+        ok(cb.length == 1 && !cb[0].disabled, "There is formdata, checkbox to " +
+           "clear formdata should be enabled.");
 
-      var cb = this.win.document.querySelectorAll(
-                 "#itemList > [preference='privacy.cpd.history']");
-      ok(cb.length == 1 && !cb[0].disabled, "There is history, checkbox to " +
-         "clear history should be enabled.");
+        var cb = this.win.document.querySelectorAll(
+                   "#itemList > [preference='privacy.cpd.history']");
+        ok(cb.length == 1 && !cb[0].disabled, "There is history, checkbox to " +
+           "clear history should be enabled.");
 
-      this.checkAllCheckboxes();
-      this.acceptDialog();
+        this.checkAllCheckboxes();
+        this.acceptDialog();
+      };
+      wh.onunload = function () {
+        yield promiseHistoryClearedState(uris, true);
 
-      ensureHistoryClearedState(uris, true);
-      ensureFormEntriesClearedState(formEntries, true);
-    };
-    wh.open();
+        let exists = yield formNameExists(formEntries[0]);
+        is(exists, false, "form entry " + formEntries[0] + " should no longer exist");
+      };
+      wh.open();
+    });
   },
   function () {
     let wh = new WindowHelper();
@@ -318,9 +401,21 @@ var gAllTests = [
     }
     wh.open();
   },
-  function () {
-    let formEntries = [ addFormEntryWithMinutesAgo(10) ];
 
+  /**
+   * Add form history entry for the next test.
+   */
+  function () {
+    let iter = function() {
+      formEntries = [ addFormEntryWithMinutesAgo(iter, 10) ];
+      yield;
+      doNextTest();
+    }();
+
+    iter.next();
+  },
+
+  function () {
     let wh = new WindowHelper();
     wh.onload = function() {
       boolPrefIs("cpd.formdata", true,
@@ -334,7 +429,10 @@ var gAllTests = [
          "the pref.");
 
       this.acceptDialog();
-      ensureFormEntriesClearedState(formEntries, true);
+    };
+    wh.onunload = function () {
+      let exists = yield formNameExists(formEntries[0]);
+      is(exists, false, "form entry " + formEntries[0] + " should no longer exist");
     };
     wh.open();
   },
@@ -690,7 +788,9 @@ WindowHelper.prototype = {
    * Opens the clear recent history dialog.  Before calling this, set
    * this.onload to a function to execute onload.  It should close the dialog
    * when done so that the tests may continue.  Set this.onunload to a function
-   * to execute onunload.  this.onunload is optional.
+   * to execute onunload.  this.onunload is optional. If it returns true, the
+   * caller is expected to call waitForAsyncUpdates at some point; if false is
+   * returned, waitForAsyncUpdates is called automatically.
    */
   open: function () {
     let wh = this;
@@ -744,9 +844,13 @@ WindowHelper.prototype = {
           // Some exceptions that reach here don't reach the test harness, but
           // ok()/is() do...
           try {
-            if (wh.onunload)
-              wh.onunload();
-            waitForAsyncUpdates(doNextTest);
+            if (wh.onunload) {
+              Task.spawn(wh.onunload).then(function() {
+                waitForAsyncUpdates(doNextTest);
+              });
+            } else {
+              waitForAsyncUpdates(doNextTest);
+            }
           }
           catch (exc) {
             win.close();
@@ -838,39 +942,41 @@ function addDownloadWithMinutesAgo(aMinutesAgo) {
  * @param aMinutesAgo
  *        The entry will be added this many minutes ago
  */
-function addFormEntryWithMinutesAgo(aMinutesAgo) {
+function addFormEntryWithMinutesAgo(then, aMinutesAgo) {
   let name = aMinutesAgo + "-minutes-ago";
-  formhist.addEntry(name, "dummy");
 
   // Artifically age the entry to the proper vintage.
-  let db = formhist.DBConnection;
   let timestamp = now_uSec - (aMinutesAgo * kUsecPerMin);
-  db.executeSimpleSQL("UPDATE moz_formhistory SET firstUsed = " +
-                      timestamp +  " WHERE fieldname = '" + name + "'");
 
-  is(formhist.nameExists(name), true,
-     "Sanity check: form entry " + name + " should exist after creating it");
+  FormHistory.update({ op: "add", fieldname: name, value: "dummy", firstUsed: timestamp },
+                     { handleError: function (error) {
+                         do_throw("Error occurred updating form history: " + error);
+                       },
+                       handleCompletion: function (reason) { then.next(); }
+                     });
   return name;
 }
 
 /**
- * Adds a history visit to history.
- *
- * @param aMinutesAgo
- *        The visit will be visited this many minutes ago
+ * Checks if a form entry exists.
  */
-function addHistoryWithMinutesAgo(aMinutesAgo) {
-  let pURI = makeURI("http://" + aMinutesAgo + "-minutes-ago.com/");
-  PlacesUtils.history.addVisit(pURI,
-                               now_uSec - aMinutesAgo * kUsecPerMin,
-                               null,
-                               Ci.nsINavHistoryService.TRANSITION_LINK,
-                               false,
-                               0);
-  is(PlacesUtils.bhistory.isVisited(pURI), true,
-     "Sanity check: history visit " + pURI.spec +
-     " should exist after creating it");
-  return pURI;
+function formNameExists(name)
+{
+  let deferred = Promise.defer();
+
+  let count = 0;
+  FormHistory.count({ fieldname: name },
+                    { handleResult: function (result) count = result,
+                      handleError: function (error) {
+                        do_throw("Error occurred searching form history: " + error);
+                        deferred.reject(error);
+                      },
+                      handleCompletion: function (reason) {
+                          if (!reason) deferred.resolve(count);
+                      }
+                    });
+
+  return deferred.promise;
 }
 
 /**
@@ -879,46 +985,16 @@ function addHistoryWithMinutesAgo(aMinutesAgo) {
 function blankSlate() {
   PlacesUtils.bhistory.removeAllPages();
   dm.cleanUp();
-  formhist.removeAllEntries();
-}
 
-/**
- * Waits for all pending async statements on the default connection, before
- * proceeding with aCallback.
- *
- * @param aCallback
- *        Function to be called when done.
- * @param aScope
- *        Scope for the callback.
- * @param aArguments
- *        Arguments array for the callback.
- *
- * @note The result is achieved by asynchronously executing a query requiring
- *       a write lock.  Since all statements on the same connection are
- *       serialized, the end of this write operation means that all writes are
- *       complete.  Note that WAL makes so that writers don't block readers, but
- *       this is a problem only across different connections.
- */
-function waitForAsyncUpdates(aCallback, aScope, aArguments)
-{
-  let scope = aScope || this;
-  let args = aArguments || [];
-  let db = PlacesUtils.history.QueryInterface(Ci.nsPIPlacesDatabase)
-                              .DBConnection;
-  let begin = db.createAsyncStatement("BEGIN EXCLUSIVE");
-  begin.executeAsync();
-  begin.finalize();
-
-  let commit = db.createAsyncStatement("COMMIT");
-  commit.executeAsync({
-    handleResult: function() {},
-    handleError: function() {},
-    handleCompletion: function(aReason)
-    {
-      aCallback.apply(scope, args);
-    }
-  });
-  commit.finalize();
+  let deferred = Promise.defer();
+  FormHistory.update({ op: "remove" },
+                     { handleError: function (error) {
+                         do_throw("Error occurred updating form history: " + error);
+                         deferred.reject(error);
+                       },
+                       handleCompletion: function (reason) { if (!reason) deferred.resolve(); }
+                     });
+  return deferred.promise;
 }
 
 /**
@@ -988,38 +1064,6 @@ function ensureDownloadsClearedState(aDownloadIDs, aShouldBeCleared) {
   });
 }
 
-/**
- * Ensures that the specified form entries are either cleared or not.
- *
- * @param aFormEntries
- *        Array of form entry names
- * @param aShouldBeCleared
- *        True if each form entry should be cleared, false otherwise
- */
-function ensureFormEntriesClearedState(aFormEntries, aShouldBeCleared) {
-  let niceStr = aShouldBeCleared ? "no longer" : "still";
-  aFormEntries.forEach(function (entry) {
-    is(formhist.nameExists(entry), !aShouldBeCleared,
-       "form entry " + entry + " should " + niceStr + " exist");
-  });
-}
-
-/**
- * Ensures that the specified URIs are either cleared or not.
- *
- * @param aURIs
- *        Array of page URIs
- * @param aShouldBeCleared
- *        True if each visit to the URI should be cleared, false otherwise
- */
-function ensureHistoryClearedState(aURIs, aShouldBeCleared) {
-  let niceStr = aShouldBeCleared ? "no longer" : "still";
-  aURIs.forEach(function (aURI) {
-    is(PlacesUtils.bhistory.isVisited(aURI), !aShouldBeCleared,
-       "history visit " + aURI.spec + " should " + niceStr + " exist");
-  });
-}
-
 /**
  * Ensures that the given pref is the expected value.
  *
@@ -1034,12 +1078,22 @@ function intPrefIs(aPrefName, aExpectedVal, aMsg) {
   is(gPrefService.getIntPref("privacy." + aPrefName), aExpectedVal, aMsg);
 }
 
+/**
+ * Creates a visit time.
+ *
+ * @param aMinutesAgo
+ *        The visit will be visited this many minutes ago
+ */
+function visitTimeForMinutesAgo(aMinutesAgo) {
+  return now_uSec - aMinutesAgo * kUsecPerMin;
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 
 function test() {
   requestLongerTimeout(2);
-  blankSlate();
   waitForExplicitFinish();
+  blankSlate();
   // Kick off all the tests in the gAllTests array.
   waitForAsyncUpdates(doNextTest);
 }
