@@ -25,9 +25,9 @@ fi
 
 MOZ_CHROME_FILE_FORMAT=omni
 MOZ_SAFE_BROWSING=1
-MOZ_SERVICES_AITC=1
 MOZ_SERVICES_COMMON=1
 MOZ_SERVICES_CRYPTO=1
+MOZ_SERVICES_HEALTHREPORT=1
 MOZ_SERVICES_METRICS=1
 MOZ_SERVICES_SYNC=1
 MOZ_APP_VERSION=$FIREFOX_VERSION
@@ -54,4 +54,6 @@ MOZ_EXTENSION_MANAGER=1
 MOZ_APP_STATIC_INI=1
 MOZ_WEBAPP_RUNTIME=1
 MOZ_MEDIA_NAVIGATOR=1
-MOZ_PER_WINDOW_PRIVATE_BROWSING=1
+if test "$OS_TARGET" = "WINNT" -o "$OS_TARGET" = "Darwin"; then
+  MOZ_FOLD_LIBS=1
+fi
