@@ -5,12 +5,15 @@
 
 #include "nsCOMPtr.h"
 #include "nsCOMArray.h"
-#include "nsIContent.h"
 #include "nsIMenuBuilder.h"
 #include "nsIXULContextMenuBuilder.h"
-#include "nsIDOMDocumentFragment.h"
 #include "nsCycleCollectionParticipant.h"
 
+class nsIAtom;
+class nsIContent;
+class nsIDocument;
+class nsIDOMHTMLElement;
+
 class nsXULContextMenuBuilder : public nsIMenuBuilder,
                                 public nsIXULContextMenuBuilder
 {
@@ -35,7 +38,7 @@ protected:
   nsCOMPtr<nsIAtom>             mGeneratedItemIdAttr;
 
   nsCOMPtr<nsIContent>          mCurrentNode;
-  PRInt32                       mCurrentGeneratedItemId;
+  int32_t                       mCurrentGeneratedItemId;
 
   nsCOMArray<nsIDOMHTMLElement> mElements;
 };
