@@ -7,12 +7,14 @@
 #ifndef mozilla_dom_indexeddb_idbrequest_h__
 #define mozilla_dom_indexeddb_idbrequest_h__
 
+#include "mozilla/Attributes.h"
 #include "mozilla/dom/indexedDB/IndexedDatabase.h"
 
 #include "nsIIDBRequest.h"
 #include "nsIIDBOpenDBRequest.h"
 #include "nsDOMEventTargetHelper.h"
 #include "mozilla/dom/indexedDB/IDBWrapperCache.h"
+#include "mozilla/dom/DOMError.h"
 
 class nsIScriptContext;
 class nsPIDOMWindow;
@@ -36,11 +38,10 @@ public:
   static
   already_AddRefed<IDBRequest> Create(nsISupports* aSource,
                                       IDBWrapperCache* aOwnerCache,
-                                      IDBTransaction* aTransaction,
-                                      JSContext* aCallingCx);
+                                      IDBTransaction* aTransaction);
 
   // nsIDOMEventTarget
-  virtual nsresult PreHandleEvent(nsEventChainPreVisitor& aVisitor);
+  virtual nsresult PreHandleEvent(nsEventChainPreVisitor& aVisitor) MOZ_OVERRIDE;
 
   nsISupports* Source()
   {
@@ -64,6 +65,8 @@ public:
   }
 #endif
 
+  DOMError* GetError(ErrorResult& aRv);
+
   JSContext* GetJSContext();
 
   void
@@ -80,15 +83,24 @@ public:
     return mActorParent;
   }
 
-  void CaptureCaller(JSContext* aCx);
+  void CaptureCaller();
 
   void FillScriptErrorEvent(nsScriptErrorEvent* aEvent) const;
 
-  bool IsPending() const
+  bool
+  IsPending() const
   {
     return !mHaveResultOrErrorCode;
   }
 
+#ifdef MOZ_ENABLE_PROFILER_SPS
+  uint64_t
+  GetSerialNumber() const
+  {
+    return mSerialNumber;
+  }
+#endif
+
 protected:
   IDBRequest();
   ~IDBRequest();
@@ -96,17 +108,16 @@ protected:
   nsCOMPtr<nsISupports> mSource;
   nsRefPtr<IDBTransaction> mTransaction;
 
-  jsval mResultVal;
-
-  nsCOMPtr<nsIDOMDOMError> mError;
-
+  JS::Heap<JS::Value> mResultVal;
+  nsRefPtr<mozilla::dom::DOMError> mError;
   IndexedDBRequestParentBase* mActorParent;
-
-  nsresult mErrorCode;
-  bool mHaveResultOrErrorCode;
-
   nsString mFilename;
+#ifdef MOZ_ENABLE_PROFILER_SPS
+  uint64_t mSerialNumber;
+#endif
+  nsresult mErrorCode;
   uint32_t mLineNo;
+  bool mHaveResultOrErrorCode;
 };
 
 class IDBOpenDBRequest : public IDBRequest,
@@ -122,13 +133,17 @@ public:
   already_AddRefed<IDBOpenDBRequest>
   Create(IDBFactory* aFactory,
          nsPIDOMWindow* aOwner,
-         JSObject* aScriptOwner,
-         JSContext* aCallingCx);
+         JS::Handle<JSObject*> aScriptOwner);
 
   void SetTransaction(IDBTransaction* aTransaction);
 
   // nsIDOMEventTarget
-  virtual nsresult PostHandleEvent(nsEventChainPostVisitor& aVisitor);
+  virtual nsresult PostHandleEvent(nsEventChainPostVisitor& aVisitor) MOZ_OVERRIDE;
+
+  DOMError* GetError(ErrorResult& aRv)
+  {
+    return IDBRequest::GetError(aRv);
+  }
 
   IDBFactory*
   Factory() const
