@@ -33,7 +33,6 @@
 #include "nsIDOMRange.h"
 #include "nsIEditor.h"
 #include "nsIEditorIMESupport.h"
-#include "nsIEnumerator.h"
 #include "nsINameSpaceManager.h"
 #include "nsINode.h"
 #include "nsISelection.h"
@@ -141,25 +140,15 @@ nsHTMLEditor::SetInlineProperty(nsIAtom *aProperty,
 
   bool cancel, handled;
   nsTextRulesInfo ruleInfo(EditAction::setTextProperty);
+  // Protect the edit rules object from dying
+  nsCOMPtr<nsIEditRules> kungFuDeathGrip(mRules);
   nsresult res = mRules->WillDoAction(selection, &ruleInfo, &cancel, &handled);
   NS_ENSURE_SUCCESS(res, res);
   if (!cancel && !handled) {
-    // get selection range enumerator
-    nsCOMPtr<nsIEnumerator> enumerator;
-    res = selection->GetEnumerator(getter_AddRefs(enumerator));
-    NS_ENSURE_SUCCESS(res, res);
-    NS_ENSURE_TRUE(enumerator, NS_ERROR_FAILURE);
-
     // loop thru the ranges in the selection
-    nsCOMPtr<nsISupports> currentItem;
-    for (enumerator->First();
-         static_cast<nsresult>(NS_ENUMERATOR_FALSE) == enumerator->IsDone();
-         enumerator->Next()) {
-      res = enumerator->CurrentItem(getter_AddRefs(currentItem));
-      NS_ENSURE_SUCCESS(res, res);
-      NS_ENSURE_TRUE(currentItem, NS_ERROR_FAILURE);
-
-      nsCOMPtr<nsIDOMRange> range(do_QueryInterface(currentItem));
+    uint32_t rangeCount = selection->GetRangeCount();
+    for (uint32_t rangeIdx = 0; rangeIdx < rangeCount; ++rangeIdx) {
+      nsRefPtr<nsRange> range = selection->GetRangeAt(rangeIdx);
 
       // adjust range to include any ancestors whose children are entirely
       // selected
@@ -312,8 +301,7 @@ nsHTMLEditor::IsSimpleModifiableNode(nsIContent* aContent,
   // No luck so far.  Now we check for a <span> with a single style=""
   // attribute that sets only the style we're looking for, if this type of
   // style supports it
-  if (!mHTMLCSSUtils->IsCSSEditableProperty(element, aProperty,
-                                            aAttribute, aValue) ||
+  if (!mHTMLCSSUtils->IsCSSEditableProperty(element, aProperty, aAttribute) ||
       !element->IsHTML(nsGkAtoms::span) || element->GetAttrCount() != 1 ||
       !element->HasAttr(kNameSpaceID_None, nsGkAtoms::style)) {
     return false;
@@ -361,8 +349,7 @@ nsHTMLEditor::SetInlinePropertyOnTextNode( nsIDOMCharacterData *aTextNode,
   
   // don't need to do anything if property already set on node
   bool bHasProp;
-  if (mHTMLCSSUtils->IsCSSEditableProperty(node, aProperty,
-                                           aAttribute, aValue)) {
+  if (mHTMLCSSUtils->IsCSSEditableProperty(node, aProperty, aAttribute)) {
     // the HTML styles defined by aProperty/aAttribute has a CSS equivalence
     // in this implementation for node; let's check if it carries those css styles
     nsAutoString value(*aValue);
@@ -471,8 +458,7 @@ nsHTMLEditor::SetInlinePropertyOnNodeImpl(nsIContent* aNode,
   }
 
   // don't need to do anything if property already set on node
-  if (mHTMLCSSUtils->IsCSSEditableProperty(aNode, aProperty,
-                                           aAttribute, aValue)) {
+  if (mHTMLCSSUtils->IsCSSEditableProperty(aNode, aProperty, aAttribute)) {
     if (mHTMLCSSUtils->IsCSSEquivalentToHTMLInlineStyleSet(
           aNode, aProperty, aAttribute, *aValue, nsHTMLCSSUtils::eComputed)) {
       return NS_OK;
@@ -483,8 +469,7 @@ nsHTMLEditor::SetInlinePropertyOnNodeImpl(nsIContent* aNode,
   }
 
   bool useCSS = (IsCSSEnabled() &&
-                 mHTMLCSSUtils->IsCSSEditableProperty(aNode, aProperty,
-                                                      aAttribute, aValue)) ||
+                 mHTMLCSSUtils->IsCSSEditableProperty(aNode, aProperty, aAttribute)) ||
                 // bgcolor is always done using CSS
                 aAttribute->EqualsLiteral("bgcolor");
 
@@ -1119,23 +1104,15 @@ nsHTMLEditor::GetInlinePropertyBase(nsIAtom *aProperty,
   result = GetSelection(getter_AddRefs(selection));
   NS_ENSURE_SUCCESS(result, result);
   NS_ENSURE_TRUE(selection, NS_ERROR_NULL_POINTER);
-  nsCOMPtr<nsISelectionPrivate> selPriv(do_QueryInterface(selection));
+  Selection* sel = static_cast<Selection*>(selection.get());
 
   bool isCollapsed = selection->Collapsed();
   nsCOMPtr<nsIDOMNode> collapsedNode;
-  nsCOMPtr<nsIEnumerator> enumerator;
-  result = selPriv->GetEnumerator(getter_AddRefs(enumerator));
-  NS_ENSURE_SUCCESS(result, result);
-  NS_ENSURE_TRUE(enumerator, NS_ERROR_NULL_POINTER);
-
-  enumerator->First(); 
-  nsCOMPtr<nsISupports> currentItem;
-  result = enumerator->CurrentItem(getter_AddRefs(currentItem));
+  nsRefPtr<nsRange> range = sel->GetRangeAt(0);
   // XXX: should be a while loop, to get each separate range
   // XXX: ERROR_HANDLING can currentItem be null?
-  if (NS_SUCCEEDED(result) && currentItem) {
+  if (range) {
     bool firstNodeInRange = true; // for each range, set a flag 
-    nsCOMPtr<nsIDOMRange> range(do_QueryInterface(currentItem));
 
     if (isCollapsed) {
       range->GetStartContainer(getter_AddRefs(collapsedNode));
@@ -1157,11 +1134,7 @@ nsHTMLEditor::GetInlinePropertyBase(nsIAtom *aProperty,
         return NS_OK;
       }
 
-      // Bug 747889: we don't support CSS for fontSize values
-      if ((aProperty != nsEditProperty::font ||
-           !aAttribute->EqualsLiteral("size")) &&
-          mHTMLCSSUtils->IsCSSEditableProperty(collapsedNode, aProperty,
-                                               aAttribute)) {
+      if (mHTMLCSSUtils->IsCSSEditableProperty(collapsedNode, aProperty, aAttribute)) {
         mHTMLCSSUtils->IsCSSEquivalentToHTMLInlineStyleSet(
           collapsedNode, aProperty, aAttribute, isSet, tOutString,
           nsHTMLCSSUtils::eComputed);
@@ -1245,11 +1218,7 @@ nsHTMLEditor::GetInlinePropertyBase(nsIAtom *aProperty,
 
       bool isSet = false;
       if (first) {
-        if (mHTMLCSSUtils->IsCSSEditableProperty(node, aProperty,
-                                                 aAttribute) &&
-            // Bug 747889: we don't support CSS for fontSize values
-            (aProperty != nsEditProperty::font ||
-             !aAttribute->EqualsLiteral("size"))) {
+        if (mHTMLCSSUtils->IsCSSEditableProperty(node, aProperty, aAttribute)){
           // the HTML styles defined by aProperty/aAttribute has a CSS
           // equivalence in this implementation for node; let's check if it
           // carries those css styles
@@ -1268,11 +1237,7 @@ nsHTMLEditor::GetInlinePropertyBase(nsIAtom *aProperty,
           *outValue = firstValue;
         }
       } else {
-        if (mHTMLCSSUtils->IsCSSEditableProperty(node, aProperty,
-                                                 aAttribute) &&
-            // Bug 747889: we don't support CSS for fontSize values
-            (aProperty != nsEditProperty::font ||
-             !aAttribute->EqualsLiteral("size"))) {
+        if (mHTMLCSSUtils->IsCSSEditableProperty(node, aProperty, aAttribute)){
           // the HTML styles defined by aProperty/aAttribute has a CSS equivalence
           // in this implementation for node; let's check if it carries those css styles
           if (aValue) {
@@ -1391,26 +1356,16 @@ nsresult nsHTMLEditor::RemoveInlinePropertyImpl(nsIAtom *aProperty, const nsAStr
   
   bool cancel, handled;
   nsTextRulesInfo ruleInfo(EditAction::removeTextProperty);
+  // Protect the edit rules object from dying
+  nsCOMPtr<nsIEditRules> kungFuDeathGrip(mRules);
   res = mRules->WillDoAction(selection, &ruleInfo, &cancel, &handled);
   NS_ENSURE_SUCCESS(res, res);
   if (!cancel && !handled)
   {
-    // get selection range enumerator
-    nsCOMPtr<nsIEnumerator> enumerator;
-    res = selection->GetEnumerator(getter_AddRefs(enumerator));
-    NS_ENSURE_SUCCESS(res, res);
-    NS_ENSURE_TRUE(enumerator, NS_ERROR_FAILURE);
-
     // loop thru the ranges in the selection
-    enumerator->First(); 
-    nsCOMPtr<nsISupports> currentItem;
-    while (static_cast<nsresult>(NS_ENUMERATOR_FALSE) == enumerator->IsDone()) {
-      res = enumerator->CurrentItem(getter_AddRefs(currentItem));
-      NS_ENSURE_SUCCESS(res, res);
-      NS_ENSURE_TRUE(currentItem, NS_ERROR_FAILURE);
-      
-      nsCOMPtr<nsIDOMRange> range( do_QueryInterface(currentItem) );
-
+    uint32_t rangeCount = selection->GetRangeCount();
+    for (uint32_t rangeIdx = 0; rangeIdx < rangeCount; ++rangeIdx) {
+      nsRefPtr<nsRange> range = selection->GetRangeAt(rangeIdx);
       if (aProperty == nsEditProperty::name)
       {
         // promote range if it starts or end in a named anchor and we
@@ -1516,7 +1471,6 @@ nsresult nsHTMLEditor::RemoveInlinePropertyImpl(nsIAtom *aProperty, const nsAStr
         }
         arrayOfNodes.Clear();
       }
-      enumerator->Next();
     }
   }
   if (!cancel)
@@ -1582,24 +1536,13 @@ nsHTMLEditor::RelativeFontChange( int32_t aSizeChange)
   nsAutoSelectionReset selectionResetter(selection, this);
   nsAutoTxnsConserveSelection dontSpazMySelection(this);
 
-  // get selection range enumerator
-  nsCOMPtr<nsIEnumerator> enumerator;
-  nsresult res = selection->GetEnumerator(getter_AddRefs(enumerator));
-  NS_ENSURE_SUCCESS(res, res);
-  NS_ENSURE_TRUE(enumerator, NS_ERROR_FAILURE);
-
   // loop thru the ranges in the selection
-  enumerator->First(); 
-  nsCOMPtr<nsISupports> currentItem;
-  while (static_cast<nsresult>(NS_ENUMERATOR_FALSE) == enumerator->IsDone()) {
-    res = enumerator->CurrentItem(getter_AddRefs(currentItem));
-    NS_ENSURE_SUCCESS(res, res);
-    NS_ENSURE_TRUE(currentItem, NS_ERROR_FAILURE);
-    
-    nsCOMPtr<nsIDOMRange> range( do_QueryInterface(currentItem) );
+  uint32_t rangeCount = selection->GetRangeCount();
+  for (uint32_t rangeIdx = 0; rangeIdx < rangeCount; ++rangeIdx) {
+    nsRefPtr<nsRange> range = selection->GetRangeAt(rangeIdx);
 
     // adjust range to include any ancestors who's children are entirely selected
-    res = PromoteInlineRange(range);
+    nsresult res = PromoteInlineRange(range);
     NS_ENSURE_SUCCESS(res, res);
     
     // check for easy case: both range endpoints in same text node
@@ -1681,10 +1624,9 @@ nsHTMLEditor::RelativeFontChange( int32_t aSizeChange)
         NS_ENSURE_SUCCESS(res, res);
       }
     }
-    enumerator->Next();
   }
   
-  return res;  
+  return NS_OK;  
 }
 
 nsresult
