@@ -3,10 +3,14 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+#include <stddef.h>
+
+#include "mozilla/Module.h"
 #include "mozilla/ModuleUtils.h"
-#include "nsTransactionManagerCID.h"
-#include "nsTransactionStack.h"
+#include "nsID.h"
+#include "nsITransactionManager.h"
 #include "nsTransactionManager.h"
+#include "nsTransactionManagerCID.h"
 
 ////////////////////////////////////////////////////////////////////////
 // Define the contructor function for the objects
@@ -17,13 +21,13 @@ NS_GENERIC_FACTORY_CONSTRUCTOR(nsTransactionManager)
 NS_DEFINE_NAMED_CID(NS_TRANSACTIONMANAGER_CID);
 
 static const mozilla::Module::CIDEntry kTxMgrCIDs[] = {
-    { &kNS_TRANSACTIONMANAGER_CID, false, NULL, nsTransactionManagerConstructor },
-    { NULL }
+    { &kNS_TRANSACTIONMANAGER_CID, false, nullptr, nsTransactionManagerConstructor },
+    { nullptr }
 };
 
 static const mozilla::Module::ContractIDEntry kTxMgrContracts[] = {
     { NS_TRANSACTIONMANAGER_CONTRACTID, &kNS_TRANSACTIONMANAGER_CID },
-    { NULL }
+    { nullptr }
 };
 
 static const mozilla::Module kTxMgrModule = {
