@@ -5,7 +5,8 @@ var obj = g.eval("({x: 1})");
 assertEq(g.Object.isExtensible(obj), true);
 
 var dbg = new Debugger;
-var objw = dbg.addDebuggee(obj);
+var gw = dbg.addDebuggee(g);
+var objw = gw.makeDebuggeeValue(obj);
 assertEq(objw.isExtensible(), true);
 
 assertEq(objw.preventExtensions(), undefined);
