@@ -11,7 +11,6 @@
 #include "nsIFormControlFrame.h"
 #include "nsIDOMEventListener.h"
 #include "nsIAnonymousContentCreator.h"
-#include "nsICapturePicker.h"
 #include "nsCOMPtr.h"
 
 class nsTextControlFrame;
@@ -24,24 +23,23 @@ class nsFileControlFrame : public nsBlockFrame,
 public:
   nsFileControlFrame(nsStyleContext* aContext);
 
-  NS_IMETHOD Init(nsIContent* aContent,
-                  nsIFrame*   aParent,
-                  nsIFrame*   aPrevInFlow);
+  virtual void Init(nsIContent* aContent,
+                    nsIFrame*   aParent,
+                    nsIFrame*   aPrevInFlow) MOZ_OVERRIDE;
 
-  NS_IMETHOD BuildDisplayList(nsDisplayListBuilder*   aBuilder,
-                              const nsRect&           aDirtyRect,
-                              const nsDisplayListSet& aLists);
+  virtual void BuildDisplayList(nsDisplayListBuilder*   aBuilder,
+                                const nsRect&           aDirtyRect,
+                                const nsDisplayListSet& aLists) MOZ_OVERRIDE;
 
   NS_DECL_QUERYFRAME
   NS_DECL_FRAMEARENA_HELPERS
 
   // nsIFormControlFrame
-  virtual nsresult SetFormProperty(nsIAtom* aName, const nsAString& aValue);
-  virtual nsresult GetFormProperty(nsIAtom* aName, nsAString& aValue) const MOZ_OVERRIDE;
-  virtual void SetFocus(bool aOn, bool aRepaint);
+  virtual nsresult SetFormProperty(nsIAtom* aName, const nsAString& aValue) MOZ_OVERRIDE;
+  virtual void SetFocus(bool aOn, bool aRepaint) MOZ_OVERRIDE;
 
   virtual nscoord GetMinWidth(nsRenderingContext *aRenderingContext) MOZ_OVERRIDE;
-  
+
   virtual void DestroyFrom(nsIFrame* aDestructRoot) MOZ_OVERRIDE;
 
 #ifdef DEBUG
@@ -52,9 +50,10 @@ public:
                               nsIAtom*        aAttribute,
                               int32_t         aModType) MOZ_OVERRIDE;
   virtual void ContentStatesChanged(nsEventStates aStates);
-  virtual bool IsLeaf() const;
-
-
+  virtual bool IsLeaf() const
+  {
+    return true;
+  }
 
   // nsIAnonymousContentCreator
   virtual nsresult CreateAnonymousContent(nsTArray<ContentInfo>& aElements) MOZ_OVERRIDE;
@@ -63,27 +62,20 @@ public:
 
 #ifdef ACCESSIBILITY
   virtual mozilla::a11y::AccType AccessibleType() MOZ_OVERRIDE;
-#endif  
+#endif
 
   typedef bool (*AcceptAttrCallback)(const nsAString&, void*);
 
 protected:
-  
-  struct CaptureCallbackData {
-    nsICapturePicker* picker;
-    uint32_t mode;
-  };
-  
-  uint32_t GetCaptureMode(const CaptureCallbackData& aData);
-  
+
   class MouseListener;
   friend class MouseListener;
   class MouseListener : public nsIDOMEventListener {
   public:
     NS_DECL_ISUPPORTS
-    
+
     MouseListener(nsFileControlFrame* aFrame)
-     : mFrame(aFrame) 
+     : mFrame(aFrame)
     {}
     virtual ~MouseListener() {}
 
@@ -104,7 +96,7 @@ protected:
       : mFrame(aFrame)
     {}
 
-    NS_IMETHOD Run() {
+    NS_IMETHOD Run() MOZ_OVERRIDE {
       nsFileControlFrame* frame = static_cast<nsFileControlFrame*>(mFrame.GetFrame());
       NS_ENSURE_STATE(frame);
 
@@ -116,21 +108,10 @@ protected:
     nsWeakFrame mFrame;
   };
 
-  class CaptureMouseListener: public MouseListener {
+  class DnDListener: public MouseListener {
   public:
-    CaptureMouseListener(nsFileControlFrame* aFrame) 
+    DnDListener(nsFileControlFrame* aFrame)
       : MouseListener(aFrame)
-      , mMode(0) 
-    {}
-
-    NS_DECL_NSIDOMEVENTLISTENER
-    uint32_t mMode;
-  };
-  
-  class BrowseMouseListener: public MouseListener {
-  public:
-    BrowseMouseListener(nsFileControlFrame* aFrame) 
-      : MouseListener(aFrame) 
     {}
 
     NS_DECL_NSIDOMEVENTLISTENER
@@ -144,8 +125,6 @@ protected:
       ~(nsIFrame::eReplaced | nsIFrame::eReplacedContainsBlock));
   }
 
-  virtual int GetSkipSides() const MOZ_OVERRIDE;
-
   /**
    * The text box input.
    * @see nsFileControlFrame::CreateAnonymousContent
@@ -158,38 +137,21 @@ protected:
   nsCOMPtr<nsIContent> mBrowse;
 
   /**
-   * The capture button input.
-   * @see nsFileControlFrame::CreateAnonymousContent
+   * Drag and drop mouse listener.
+   * This makes sure we don't get used after destruction.
    */
-  nsCOMPtr<nsIContent> mCapture;
-
-  /**
-   * Our mouse listener.  This makes sure we don't get used after destruction.
-   */
-  nsRefPtr<BrowseMouseListener> mMouseListener;
-  nsRefPtr<CaptureMouseListener> mCaptureMouseListener;
+  nsRefPtr<DnDListener> mMouseListener;
 
 protected:
-  /**
-   * @return the text control frame, or null if not found
-   */
-  nsTextControlFrame* GetTextControlFrame();
-
-  /**
-   * Copy an attribute from file content to text and button content.
-   * @param aNameSpaceID namespace of attr
-   * @param aAttribute attribute atom
-   * @param aWhichControls which controls to apply to (SYNC_TEXT or SYNC_FILE)
-   */
-  void SyncAttr(int32_t aNameSpaceID, nsIAtom* aAttribute,
-                int32_t aWhichControls);
-
   /**
    * Sync the disabled state of the content with anonymous children.
    */
   void SyncDisabledState();
+
+  /**
+   * Updates the displayed value by using aValue.
+   */
+  void UpdateDisplayedValue(const nsAString& aValue, bool aNotify);
 };
 
-#endif
-
-
+#endif // nsFileControlFrame_h___
