@@ -4,12 +4,16 @@
 
 // Original author: ekr@rtfm.com
 
+#include "logging.h"
 #include "MediaPipeline.h"
 
+#ifndef USE_FAKE_MEDIA_STREAMS
+#include "MediaStreamGraphImpl.h"
+#endif
+
 #include <math.h>
 
 #include "nspr.h"
-#include <prlog.h>
 #include "srtp.h"
 
 #ifdef MOZILLA_INTERNAL_API
@@ -17,9 +21,12 @@
 #include "Layers.h"
 #include "ImageTypes.h"
 #include "ImageContainer.h"
+#include "VideoUtils.h"
+#ifdef MOZ_WIDGET_GONK
+#include "GonkIOSurfaceImage.h"
+#endif
 #endif
 
-#include "logging.h"
 #include "nsError.h"
 #include "AudioSegment.h"
 #include "MediaSegment.h"
@@ -28,7 +35,6 @@
 #include "transportlayer.h"
 #include "transportlayerdtls.h"
 #include "transportlayerice.h"
-
 #include "runnable_utils.h"
 
 using namespace mozilla;
@@ -37,57 +43,70 @@ using namespace mozilla;
 // Dial up pipeline logging in debug mode
 #define MP_LOG_INFO PR_LOG_WARN
 #else
-#define MP_LOG_INFO PR_LOG_INFO
+#define MP_LOG_INFO PR_LOG_DEBUG
 #endif
 
-
 // Logging context
-MOZ_MTLOG_MODULE("mediapipeline");
+MOZ_MTLOG_MODULE("mediapipeline")
 
 namespace mozilla {
 
 static char kDTLSExporterLabel[] = "EXTRACTOR-dtls_srtp";
 
+MediaPipeline::~MediaPipeline() {
+  MOZ_ASSERT(!stream_);  // Check that we have shut down already.
+  MOZ_MTLOG(PR_LOG_DEBUG, "Destroying MediaPipeline: " << description_);
+}
+
 nsresult MediaPipeline::Init() {
   ASSERT_ON_THREAD(main_thread_);
 
-  // TODO(ekr@rtfm.com): is there a way to make this async?
-  nsresult ret;
   RUN_ON_THREAD(sts_thread_,
-		WrapRunnableRet(this, &MediaPipeline::Init_s, &ret),
-		NS_DISPATCH_SYNC);
-  return ret;
+                WrapRunnable(
+                    nsRefPtr<MediaPipeline>(this),
+                    &MediaPipeline::Init_s),
+                NS_DISPATCH_NORMAL);
+
+  return NS_OK;
 }
 
 nsresult MediaPipeline::Init_s() {
   ASSERT_ON_THREAD(sts_thread_);
   conduit_->AttachTransport(transport_);
 
-  MOZ_ASSERT(rtp_transport_);
-
   nsresult res;
-
+  MOZ_ASSERT(rtp_transport_);
   // Look to see if the transport is ready
   rtp_transport_->SignalStateChange.connect(this,
                                             &MediaPipeline::StateChange);
 
   if (rtp_transport_->state() == TransportLayer::TS_OPEN) {
-    res = TransportReady(rtp_transport_);
+    res = TransportReady_s(rtp_transport_);
     if (NS_FAILED(res)) {
-      MOZ_MTLOG(PR_LOG_ERROR, "Error calling TransportReady()");
+      MOZ_MTLOG(PR_LOG_ERROR, "Error calling TransportReady(); res="
+                << static_cast<uint32_t>(res) << " in " << __FUNCTION__);
       return res;
     }
+  } else if (rtp_transport_->state() == TransportLayer::TS_ERROR) {
+    MOZ_MTLOG(PR_LOG_ERROR, "RTP transport is already in error state");
+    TransportFailed_s(rtp_transport_);
+    return NS_ERROR_FAILURE;
   } else {
     if (!muxed_) {
       rtcp_transport_->SignalStateChange.connect(this,
                                                  &MediaPipeline::StateChange);
 
       if (rtcp_transport_->state() == TransportLayer::TS_OPEN) {
-        res = TransportReady(rtcp_transport_);
+        res = TransportReady_s(rtcp_transport_);
         if (NS_FAILED(res)) {
-          MOZ_MTLOG(PR_LOG_ERROR, "Error calling TransportReady()");
+          MOZ_MTLOG(PR_LOG_ERROR, "Error calling TransportReady(); res="
+                    << static_cast<uint32_t>(res) << " in " << __FUNCTION__);
           return res;
         }
+      } else if (rtcp_transport_->state() == TransportLayer::TS_ERROR) {
+        MOZ_MTLOG(PR_LOG_ERROR, "RTCP transport is already in error state");
+        TransportFailed_s(rtcp_transport_);
+        return NS_ERROR_FAILURE;
       }
     }
   }
@@ -95,10 +114,12 @@ nsresult MediaPipeline::Init_s() {
 }
 
 
-// Disconnect us from the transport so that we can cleanly destruct
-// the pipeline on the main thread.
-void MediaPipeline::DetachTransport_s() {
+// Disconnect us from the transport so that we can cleanly destruct the
+// pipeline on the main thread.  ShutdownMedia_m() must have already been
+// called
+void MediaPipeline::ShutdownTransport_s() {
   ASSERT_ON_THREAD(sts_thread_);
+  MOZ_ASSERT(!stream_); // verifies that ShutdownMedia_m() has run
 
   disconnect_all();
   transport_->Detach();
@@ -106,42 +127,23 @@ void MediaPipeline::DetachTransport_s() {
   rtcp_transport_ = NULL;
 }
 
-void MediaPipeline::DetachTransport() {
-  RUN_ON_THREAD(sts_thread_,
-                WrapRunnable(this, &MediaPipeline::DetachTransport_s),
-                NS_DISPATCH_SYNC);
-}
-
 void MediaPipeline::StateChange(TransportFlow *flow, TransportLayer::State state) {
   if (state == TransportLayer::TS_OPEN) {
     MOZ_MTLOG(MP_LOG_INFO, "Flow is ready");
-    TransportReady(flow);
+    TransportReady_s(flow);
   } else if (state == TransportLayer::TS_CLOSED ||
              state == TransportLayer::TS_ERROR) {
-    TransportFailed(flow);
+    TransportFailed_s(flow);
   }
 }
 
-nsresult MediaPipeline::TransportReady(TransportFlow *flow) {
-  nsresult rv;
-  nsresult res;
-
-  rv = RUN_ON_THREAD(sts_thread_,
-    WrapRunnableRet(this, &MediaPipeline::TransportReadyInt, flow, &res),
-    NS_DISPATCH_SYNC);
-
-  // res is invalid unless the dispatch succeeded
-  if (NS_FAILED(rv))
-    return rv;
-
-  return res;
-}
-
-nsresult MediaPipeline::TransportReadyInt(TransportFlow *flow) {
+nsresult MediaPipeline::TransportReady_s(TransportFlow *flow) {
   MOZ_ASSERT(!description_.empty());
   bool rtcp = !(flow == rtp_transport_);
   State *state = rtcp ? &rtcp_state_ : &rtp_state_;
 
+  // TODO(ekr@rtfm.com): implement some kind of notification on
+  // failure. bug 852665.
   if (*state != MP_CONNECTING) {
     MOZ_MTLOG(PR_LOG_ERROR, "Transport ready for flow in wrong state:" <<
 	      description_ << ": " << (rtcp ? "rtcp" : "rtp"));
@@ -163,6 +165,7 @@ nsresult MediaPipeline::TransportReadyInt(TransportFlow *flow) {
   res = dtls->GetSrtpCipher(&cipher_suite);
   if (NS_FAILED(res)) {
     MOZ_MTLOG(PR_LOG_ERROR, "Failed to negotiate DTLS-SRTP. This is an error");
+    *state = MP_CLOSED;
     return res;
   }
 
@@ -265,7 +268,8 @@ nsresult MediaPipeline::TransportReadyInt(TransportFlow *flow) {
   return NS_OK;
 }
 
-nsresult MediaPipeline::TransportFailed(TransportFlow *flow) {
+nsresult MediaPipeline::TransportFailed_s(TransportFlow *flow) {
+  ASSERT_ON_THREAD(sts_thread_);
   bool rtcp = !(flow == rtp_transport_);
 
   State *state = rtcp ? &rtcp_state_ : &rtp_state_;
@@ -365,6 +369,16 @@ void MediaPipeline::RtpPacketReceived(TransportLayer *layer,
     return;
   }
 
+  if (rtp_state_ != MP_OPEN) {
+    MOZ_MTLOG(PR_LOG_DEBUG, "Discarding incoming packet; pipeline not open");
+    return;
+  }
+
+  if (rtp_transport_->state() != TransportLayer::TS_OPEN) {
+    MOZ_MTLOG(PR_LOG_ERROR, "Discarding incoming packet; transport not open");
+    return;
+  }
+
   MOZ_ASSERT(rtp_recv_srtp_);  // This should never happen
 
   if (direction_ == TRANSMIT) {
@@ -403,6 +417,16 @@ void MediaPipeline::RtcpPacketReceived(TransportLayer *layer,
     return;
   }
 
+  if (rtcp_state_ != MP_OPEN) {
+    MOZ_MTLOG(PR_LOG_DEBUG, "Discarding incoming packet; pipeline not open");
+    return;
+  }
+
+  if (rtcp_transport_->state() != TransportLayer::TS_OPEN) {
+    MOZ_MTLOG(PR_LOG_ERROR, "Discarding incoming packet; transport not open");
+    return;
+  }
+
   if (direction_ == RECEIVE) {
     // Discard any RTCP that is being transmitted to us
     // This will be unnecessary when we have SSRC filtering.
@@ -476,11 +500,17 @@ void MediaPipeline::PacketReceived(TransportLayer *layer,
 }
 
 nsresult MediaPipelineTransmit::Init() {
+  char track_id_string[11];
   ASSERT_ON_THREAD(main_thread_);
 
+  // We can replace this when we are allowed to do streams or std::to_string
+  PR_snprintf(track_id_string, sizeof(track_id_string), "%d", track_id_);
+
   description_ = pc_ + "| ";
   description_ += conduit_->type() == MediaSessionConduit::AUDIO ?
-      "Transmit audio" : "Transmit video";
+      "Transmit audio[" : "Transmit video[";
+  description_ += track_id_string;
+  description_ += "]";
 
   // TODO(ekr@rtfm.com): Check for errors
   MOZ_MTLOG(PR_LOG_DEBUG, "Attaching pipeline to stream "
@@ -494,9 +524,9 @@ nsresult MediaPipelineTransmit::Init() {
   return MediaPipeline::Init();
 }
 
-nsresult MediaPipelineTransmit::TransportReady(TransportFlow *flow) {
+nsresult MediaPipelineTransmit::TransportReady_s(TransportFlow *flow) {
   // Call base ready function.
-  MediaPipeline::TransportReady(flow);
+  MediaPipeline::TransportReady_s(flow);
 
   if (flow == rtp_transport_) {
     // TODO(ekr@rtfm.com): Move onto MSG thread.
@@ -667,7 +697,7 @@ void MediaPipelineTransmit::PipelineListener::ProcessAudioChunk(
   nsAutoArrayPtr<int16_t> samples(new int16_t[chunk.mDuration]);
 
   if (chunk.mBuffer) {
-    switch(chunk.mBufferFormat) {
+    switch (chunk.mBufferFormat) {
       case AUDIO_FORMAT_FLOAT32:
         MOZ_MTLOG(PR_LOG_ERROR, "Can't process audio except in 16-bit PCM yet");
         MOZ_ASSERT(PR_FALSE);
@@ -675,8 +705,7 @@ void MediaPipelineTransmit::PipelineListener::ProcessAudioChunk(
         break;
       case AUDIO_FORMAT_S16:
         {
-          const short* buf = static_cast<const short *>(chunk.mBuffer->Data()) +
-            chunk.mOffset;
+          const short* buf = static_cast<const short *>(chunk.mChannelData[0]);
           ConvertAudioSamplesWithScale(buf, samples, chunk.mDuration, chunk.mVolume);
         }
         break;
@@ -755,72 +784,213 @@ void MediaPipelineTransmit::PipelineListener::ProcessAudioChunk(
 }
 
 #ifdef MOZILLA_INTERNAL_API
+static void FillBlackYCbCr420PixelData(uint8_t* aBuffer, const gfxIntSize& aSize)
+{
+  // Fill Y plane
+}
+
 void MediaPipelineTransmit::PipelineListener::ProcessVideoChunk(
     VideoSessionConduit* conduit,
     TrackRate rate,
     VideoChunk& chunk) {
-  // We now need to send the video frame to the other side
   layers::Image *img = chunk.mFrame.GetImage();
+  gfxIntSize size = img ? img->GetSize() : chunk.mFrame.GetIntrinsicSize();
+  if ((size.width & 1) != 0 || (size.height & 1) != 0) {
+    MOZ_ASSERT(false, "Can't handle odd-sized images");
+    return;
+  }
+
+  if (chunk.mFrame.GetForceBlack()) {
+    uint32_t yPlaneLen = size.width*size.height;
+    uint32_t cbcrPlaneLen = yPlaneLen/2;
+    uint32_t length = yPlaneLen + cbcrPlaneLen;
+
+    // Send a black image.
+    nsAutoArrayPtr<uint8_t> pixelData;
+    static const fallible_t fallible = fallible_t();
+    pixelData = new (fallible) uint8_t[length];
+    if (pixelData) {
+      memset(pixelData, 0x10, yPlaneLen);
+      // Fill Cb/Cr planes
+      memset(pixelData + yPlaneLen, 0x80, cbcrPlaneLen);
+
+      MOZ_MTLOG(PR_LOG_DEBUG, "Sending a black video frame");
+      conduit->SendVideoFrame(pixelData, length, size.width, size.height,
+                              mozilla::kVideoI420, 0);
+    }
+    return;
+  }
+
+  // We now need to send the video frame to the other side
   if (!img) {
     // segment.AppendFrame() allows null images, which show up here as null
     return;
   }
 
-  ImageFormat format = img->GetFormat();
+  // We get passed duplicate frames every ~10ms even if there's no frame change!
+  int32_t serial = img->GetSerial();
+  if (serial == last_img_) {
+    return;
+  }
+  last_img_ = serial;
 
-  if (format != PLANAR_YCBCR) {
-    MOZ_MTLOG(PR_LOG_ERROR, "Can't process non-YCBCR video");
+  ImageFormat format = img->GetFormat();
+#ifdef MOZ_WIDGET_GONK
+  if (format == GONK_IO_SURFACE) {
+    layers::GonkIOSurfaceImage *nativeImage = static_cast<layers::GonkIOSurfaceImage*>(img);
+    layers::SurfaceDescriptor handle = nativeImage->GetSurfaceDescriptor();
+    layers::SurfaceDescriptorGralloc grallocHandle = handle.get_SurfaceDescriptorGralloc();
+
+    android::sp<android::GraphicBuffer> graphicBuffer = layers::GrallocBufferActor::GetFrom(grallocHandle);
+    void *basePtr;
+    graphicBuffer->lock(android::GraphicBuffer::USAGE_SW_READ_MASK, &basePtr);
+    conduit->SendVideoFrame(static_cast<unsigned char*>(basePtr),
+                            (graphicBuffer->getWidth() * graphicBuffer->getHeight() * 3) / 2,
+                            graphicBuffer->getWidth(),
+                            graphicBuffer->getHeight(),
+                            mozilla::kVideoNV21, 0);
+    graphicBuffer->unlock();
+  } else
+#endif
+  if (format == PLANAR_YCBCR) {
+    // Cast away constness b/c some of the accessors are non-const
+    layers::PlanarYCbCrImage* yuv =
+    const_cast<layers::PlanarYCbCrImage *>(
+          static_cast<const layers::PlanarYCbCrImage *>(img));
+    // Big-time assumption here that this is all contiguous data coming
+    // from getUserMedia or other sources.
+    const layers::PlanarYCbCrImage::Data *data = yuv->GetData();
+
+    uint8_t *y = data->mYChannel;
+#ifdef DEBUG
+    uint8_t *cb = data->mCbChannel;
+    uint8_t *cr = data->mCrChannel;
+#endif
+    uint32_t width = yuv->GetSize().width;
+    uint32_t height = yuv->GetSize().height;
+    uint32_t length = yuv->GetDataSize();
+
+    // SendVideoFrame only supports contiguous YCrCb 4:2:0 buffers
+    // Verify it's contiguous and in the right order
+    MOZ_ASSERT(cb == (y + width*height) &&
+               cr == (cb + width*height/4));
+    // XXX Consider making this a non-debug-only check if we ever implement
+    // any subclasses of PlanarYCbCrImage that allow disjoint buffers such
+    // that y+3(width*height)/2 might go outside the allocation.
+    // GrallocPlanarYCbCrImage can have wider strides, and so in some cases
+    // would encode as garbage.  If we need to encode it we'll either want to
+    // modify SendVideoFrame or copy/move the data in the buffer.
+
+    // OK, pass it on to the conduit
+    MOZ_MTLOG(PR_LOG_DEBUG, "Sending a video frame");
+    // Not much for us to do with an error
+    conduit->SendVideoFrame(y, length, width, height, mozilla::kVideoI420, 0);
+  } else {
+    MOZ_MTLOG(PR_LOG_ERROR, "Unsupported video format");
     MOZ_ASSERT(PR_FALSE);
     return;
   }
-
-  // Cast away constness b/c some of the accessors are non-const
-  layers::PlanarYCbCrImage* yuv =
-    const_cast<layers::PlanarYCbCrImage *>(
-      static_cast<const layers::PlanarYCbCrImage *>(img));
-
-  // Big-time assumption here that this is all contiguous data coming
-  // from getUserMedia or other sources.
-  const layers::PlanarYCbCrImage::Data *data = yuv->GetData();
-
-  uint8_t *y = data->mYChannel;
-#ifdef DEBUG
-  uint8_t *cb = data->mCbChannel;
-  uint8_t *cr = data->mCrChannel;
-#endif
-  uint32_t width = yuv->GetSize().width;
-  uint32_t height = yuv->GetSize().height;
-  uint32_t length = yuv->GetDataSize();
-
-  // SendVideoFrame only supports contiguous YCrCb 4:2:0 buffers
-  // Verify it's contiguous and in the right order
-  MOZ_ASSERT(cb == (y + width*height) &&
-             cr == (cb + width*height/4));
-  // XXX Consider making this a non-debug-only check if we ever implement
-  // any subclasses of PlanarYCbCrImage that allow disjoint buffers such
-  // that y+3(width*height)/2 might go outside the allocation.
-  // GrallocPlanarYCbCrImage can have wider strides, and so in some cases
-  // would encode as garbage.  If we need to encode it we'll either want to
-  // modify SendVideoFrame or copy/move the data in the buffer.
-
-  // OK, pass it on to the conduit
-  MOZ_MTLOG(PR_LOG_DEBUG, "Sending a video frame");
-  // Not much for us to do with an error
-  conduit->SendVideoFrame(y, length, width, height, mozilla::kVideoI420, 0);
 }
 #endif
 
 nsresult MediaPipelineReceiveAudio::Init() {
+  char track_id_string[11];
   ASSERT_ON_THREAD(main_thread_);
   MOZ_MTLOG(PR_LOG_DEBUG, __FUNCTION__);
 
-  description_ = pc_ + "| Receive audio";
+  // We can replace this when we are allowed to do streams or std::to_string
+  PR_snprintf(track_id_string, sizeof(track_id_string), "%d", track_id_);
 
-  stream_->AddListener(listener_);
+  description_ = pc_ + "| Receive audio[";
+  description_ += track_id_string;
+  description_ += "]";
+
+  listener_->AddSelf(new AudioSegment());
 
   return MediaPipelineReceive::Init();
 }
 
+
+// Add a track and listener on the MSG thread using the MSG command queue
+static void AddTrackAndListener(MediaStream* source,
+                                TrackID track_id, TrackRate track_rate,
+                                MediaStreamListener* listener, MediaSegment* segment,
+                                const RefPtr<TrackAddedCallback>& completed) {
+  // This both adds the listener and the track
+#ifdef MOZILLA_INTERNAL_API
+  class Message : public ControlMessage {
+   public:
+    Message(MediaStream* stream, TrackID track, TrackRate rate,
+            MediaSegment* segment, MediaStreamListener* listener,
+            const RefPtr<TrackAddedCallback>& completed)
+      : ControlMessage(stream),
+        track_id_(track),
+        track_rate_(rate),
+        segment_(segment),
+        listener_(listener),
+        completed_(completed) {}
+
+    virtual void Run() MOZ_OVERRIDE {
+      StreamTime current_end = mStream->GetBufferEnd();
+      TrackTicks current_ticks = TimeToTicksRoundUp(track_rate_, current_end);
+
+      mStream->AddListenerImpl(listener_.forget());
+
+      // Add a track 'now' to avoid possible underrun, especially if we add
+      // a track "later".
+
+      if (current_end != 0L) {
+        MOZ_MTLOG(MP_LOG_INFO, "added track @ " << current_end <<
+                  " -> " << MediaTimeToSeconds(current_end));
+      }
+
+      // To avoid assertions, we need to insert a dummy segment that covers up
+      // to the "start" time for the track
+      segment_->AppendNullData(current_ticks);
+      mStream->AsSourceStream()->AddTrack(track_id_, track_rate_,
+                                          current_ticks, segment_);
+      // AdvanceKnownTracksTicksTime(HEAT_DEATH_OF_UNIVERSE) means that in
+      // theory per the API, we can't add more tracks before that
+      // time. However, the impl actually allows it, and it avoids a whole
+      // bunch of locking that would be required (and potential blocking)
+      // if we used smaller values and updated them on each NotifyPull.
+      mStream->AsSourceStream()->AdvanceKnownTracksTime(STREAM_TIME_MAX);
+
+      // We need to know how much has been "inserted" because we're given absolute
+      // times in NotifyPull.
+      completed_->TrackAdded(current_ticks);
+    }
+   private:
+    TrackID track_id_;
+    TrackRate track_rate_;
+    MediaSegment* segment_;
+    nsRefPtr<MediaStreamListener> listener_;
+    const RefPtr<TrackAddedCallback> completed_;
+  };
+
+  MOZ_ASSERT(listener);
+
+  source->GraphImpl()->AppendMessage(new Message(source, track_id, track_rate, segment, listener, completed));
+#else
+  source->AddListener(listener);
+  source->AsSourceStream()->AddTrack(track_id, track_rate, 0, segment);
+#endif
+}
+
+void GenericReceiveListener::AddSelf(MediaSegment* segment) {
+  RefPtr<TrackAddedCallback> callback = new GenericReceiveCallback(this);
+  AddTrackAndListener(source_, track_id_, track_rate_, this, segment, callback);
+}
+
+MediaPipelineReceiveAudio::PipelineListener::PipelineListener(
+    SourceMediaStream * source, TrackID track_id,
+    const RefPtr<MediaSessionConduit>& conduit)
+  : GenericReceiveListener(source, track_id, 16000), // XXX rate assumption
+    conduit_(conduit)
+{
+  MOZ_ASSERT(track_rate_%100 == 0);
+}
+
 void MediaPipelineReceiveAudio::PipelineListener::
 NotifyPull(MediaStreamGraph* graph, StreamTime desired_time) {
   MOZ_ASSERT(source_);
@@ -830,72 +1000,105 @@ NotifyPull(MediaStreamGraph* graph, StreamTime desired_time) {
   }
 
   // This comparison is done in total time to avoid accumulated roundoff errors.
-  while (MillisecondsToMediaTime(played_) < desired_time) {
-    // TODO(ekr@rtfm.com): Is there a way to avoid mallocating here?
-    nsRefPtr<SharedBuffer> samples = SharedBuffer::Create(1000);
+  while (TicksToTimeRoundDown(track_rate_, played_ticks_) < desired_time) {
+    // TODO(ekr@rtfm.com): Is there a way to avoid mallocating here?  Or reduce the size?
+    // Max size given mono is 480*2*1 = 960 (48KHz)
+#define AUDIO_SAMPLE_BUFFER_MAX 1000
+    MOZ_ASSERT((track_rate_/100)*sizeof(uint16_t) <= AUDIO_SAMPLE_BUFFER_MAX);
+
+    nsRefPtr<SharedBuffer> samples = SharedBuffer::Create(AUDIO_SAMPLE_BUFFER_MAX);
+    int16_t *samples_data = static_cast<int16_t *>(samples->Data());
     int samples_length;
 
+    // This fetches 10ms of data
     MediaConduitErrorCode err =
         static_cast<AudioSessionConduit*>(conduit_.get())->GetAudioFrame(
-            static_cast<int16_t *>(samples->Data()),
-            16000,  // Sampling rate fixed at 16 kHz for now
-            0,  // TODO(ekr@rtfm.com): better estimate of capture delay
+            samples_data,
+            track_rate_,
+            0,  // TODO(ekr@rtfm.com): better estimate of "capture" (really playout) delay
             samples_length);
+    MOZ_ASSERT(samples_length < AUDIO_SAMPLE_BUFFER_MAX);
 
-    if (err != kMediaConduitNoError)
-      return;
+    if (err != kMediaConduitNoError) {
+      // Insert silence on conduit/GIPS failure (extremely unlikely)
+      MOZ_MTLOG(PR_LOG_ERROR, "Audio conduit failed (" << err << ") to return data @ " << played_ticks_ <<
+                " (desired " << desired_time << " -> " << MediaTimeToSeconds(desired_time) << ")");
+      MOZ_ASSERT(err == kMediaConduitNoError);
+      samples_length = (track_rate_/100)*sizeof(uint16_t); // if this is not enough we'll loop and provide more
+      memset(samples_data, '\0', samples_length);
+    }
 
     MOZ_MTLOG(PR_LOG_DEBUG, "Audio conduit returned buffer of length " << samples_length);
 
     AudioSegment segment;
-    segment.Init(1);
-    segment.AppendFrames(samples.forget(), samples_length,
-                         0, samples_length, AUDIO_FORMAT_S16);
+    nsAutoTArray<const int16_t*,1> channels;
+    channels.AppendElement(samples_data);
+    segment.AppendFrames(samples.forget(), channels, samples_length);
 
-    source_->AppendToTrack(1,  // TODO(ekr@rtfm.com): Track ID
-                           &segment);
-
-    played_ += 10;
+    // Handle track not actually added yet or removed/finished
+    if (source_->AppendToTrack(track_id_, &segment)) {
+      played_ticks_ += track_rate_/100; // 10ms in TrackTicks
+    } else {
+      MOZ_MTLOG(PR_LOG_ERROR, "AppendToTrack failed");
+      // we can't un-read the data, but that's ok since we don't want to
+      // buffer - but don't i-loop!
+      return;
+    }
   }
 }
 
 nsresult MediaPipelineReceiveVideo::Init() {
+  char track_id_string[11];
   ASSERT_ON_THREAD(main_thread_);
   MOZ_MTLOG(PR_LOG_DEBUG, __FUNCTION__);
 
-  description_ = pc_ + "| Receive video";
+  // We can replace this when we are allowed to do streams or std::to_string
+  PR_snprintf(track_id_string, sizeof(track_id_string), "%d", track_id_);
 
+  description_ = pc_ + "| Receive video[";
+  description_ += track_id_string;
+  description_ += "]";
+
+#ifdef MOZILLA_INTERNAL_API
+  listener_->AddSelf(new VideoSegment());
+#endif
+
+  // Always happens before we can DetachMediaStream()
   static_cast<VideoSessionConduit *>(conduit_.get())->
       AttachRenderer(renderer_);
 
   return MediaPipelineReceive::Init();
 }
 
-MediaPipelineReceiveVideo::PipelineRenderer::PipelineRenderer(
-    MediaPipelineReceiveVideo *pipeline) :
-    pipeline_(pipeline),
-    width_(640), height_(480) {
-
+MediaPipelineReceiveVideo::PipelineListener::PipelineListener(
+  SourceMediaStream* source, TrackID track_id)
+  : GenericReceiveListener(source, track_id, USECS_PER_S),
+    width_(640),
+    height_(480),
+#ifdef MOZILLA_INTERNAL_API
+    image_container_(),
+    image_(),
+#endif
+    monitor_("Video PipelineListener") {
 #ifdef MOZILLA_INTERNAL_API
   image_container_ = layers::LayerManager::CreateImageContainer();
-  SourceMediaStream *source =
-      pipeline_->stream_->AsSourceStream();
-  source->AddTrack(1 /* Track ID */, 30, 0, new VideoSegment());
-  source->AdvanceKnownTracksTime(STREAM_TIME_MAX);
 #endif
 }
 
-void MediaPipelineReceiveVideo::PipelineRenderer::RenderVideoFrame(
+void MediaPipelineReceiveVideo::PipelineListener::RenderVideoFrame(
     const unsigned char* buffer,
     unsigned int buffer_size,
     uint32_t time_stamp,
     int64_t render_time) {
 #ifdef MOZILLA_INTERNAL_API
-  SourceMediaStream *source =
-      pipeline_->stream_->AsSourceStream();
+  ReentrantMonitorAutoEnter enter(monitor_);
 
   // Create a video frame and append it to the track.
+#ifdef MOZ_WIDGET_GONK
+  ImageFormat format = GRALLOC_PLANAR_YCBCR;
+#else
   ImageFormat format = PLANAR_YCBCR;
+#endif
   nsRefPtr<layers::Image> image = image_container_->CreateImage(&format, 1);
 
   layers::PlanarYCbCrImage* videoImage = static_cast<layers::PlanarYCbCrImage*>(image.get());
@@ -918,12 +1121,34 @@ void MediaPipelineReceiveVideo::PipelineRenderer::RenderVideoFrame(
 
   videoImage->SetData(data);
 
-  VideoSegment segment;
-  char buf[32];
-  PR_snprintf(buf, 32, "%p", source);
+  image_ = image.forget();
+#endif
+}
 
-  segment.AppendFrame(image.forget(), 1, gfxIntSize(width_, height_));
-  source->AppendToTrack(1, &(segment));
+void MediaPipelineReceiveVideo::PipelineListener::
+NotifyPull(MediaStreamGraph* graph, StreamTime desired_time) {
+  ReentrantMonitorAutoEnter enter(monitor_);
+
+#ifdef MOZILLA_INTERNAL_API
+  nsRefPtr<layers::Image> image = image_;
+  TrackTicks target = TimeToTicksRoundUp(USECS_PER_S, desired_time);
+  TrackTicks delta = target - played_ticks_;
+
+  // Don't append if we've already provided a frame that supposedly
+  // goes past the current aDesiredTime Doing so means a negative
+  // delta and thus messes up handling of the graph
+  if (delta > 0) {
+    VideoSegment segment;
+    segment.AppendFrame(image ? image.forget() : nullptr, delta,
+                        gfxIntSize(width_, height_));
+    // Handle track not actually added yet or removed/finished
+    if (source_->AppendToTrack(track_id_, &segment)) {
+      played_ticks_ = target;
+    } else {
+      MOZ_MTLOG(PR_LOG_ERROR, "AppendToTrack failed");
+      return;
+    }
+  }
 #endif
 }
 
