@@ -4,15 +4,15 @@
 
 package org.mozilla.gecko.background.announcements;
 
-import org.mozilla.gecko.sync.Logger;
+import org.mozilla.gecko.background.BackgroundService;
+import org.mozilla.gecko.background.common.log.Logger;
 
-import android.app.AlarmManager;
 import android.content.BroadcastReceiver;
 import android.content.Context;
 import android.content.Intent;
 
 /**
- * Start the announcements service when instructed by the {@link AlarmManager}.
+ * Start the announcements service when instructed by the {@link android.app.AlarmManager}.
  */
 public class AnnouncementsStartReceiver extends BroadcastReceiver {
 
@@ -25,7 +25,6 @@ public class AnnouncementsStartReceiver extends BroadcastReceiver {
     }
 
     Logger.debug(LOG_TAG, "AnnouncementsStartReceiver.onReceive().");
-    Intent service = new Intent(context, AnnouncementsService.class);
-    context.startService(service);
+    BackgroundService.runIntentInService(context, intent, AnnouncementsService.class);
   }
 }
