@@ -27,15 +27,12 @@
 */
 
 #include "xpcom-config.h"
-#include NEW_H
 #include "nsCOMPtr.h"
 #include "nsIComponentManager.h"
-#include "nsIEnumerator.h"
 #include "nsIRDFCompositeDataSource.h"
 #include "nsIRDFNode.h"
 #include "nsIRDFObserver.h"
 #include "nsIRDFRemoteDataSource.h"
-#include "nsFixedSizeAllocator.h"
 #include "nsTArray.h"
 #include "nsCOMArray.h"
 #include "nsArrayEnumerator.h"
@@ -93,13 +90,11 @@ protected:
     nsCOMArray<nsIRDFObserver> mObservers;
     nsCOMArray<nsIRDFDataSource> mDataSources;
 
-	bool        mAllowNegativeAssertions;
-	bool        mCoalesceDuplicateArcs;
+    bool        mAllowNegativeAssertions;
+    bool        mCoalesceDuplicateArcs;
     int32_t     mUpdateBatchNest;
 
-    nsFixedSizeAllocator mAllocator;
-
-	virtual ~CompositeDataSourceImpl() {}
+    virtual ~CompositeDataSourceImpl() {}
 
     friend class CompositeEnumeratorImpl;
     friend class CompositeArcsInOutEnumeratorImpl;
@@ -126,15 +121,13 @@ class CompositeEnumeratorImpl : public nsISimpleEnumerator
     virtual nsresult
     HasNegation(nsIRDFDataSource* aDataSource, nsIRDFNode* aNode, bool* aResult) = 0;
 
-    virtual void Destroy() = 0;
-
 protected:
     CompositeEnumeratorImpl(CompositeDataSourceImpl* aCompositeDataSource,
                             bool aAllowNegativeAssertions,
                             bool aCoalesceDuplicateArcs);
 
     virtual ~CompositeEnumeratorImpl();
-    
+
     CompositeDataSourceImpl* mCompositeDataSource;
 
     nsISimpleEnumerator* mCurrent;
@@ -156,19 +149,19 @@ CompositeEnumeratorImpl::CompositeEnumeratorImpl(CompositeDataSourceImpl* aCompo
       mAllowNegativeAssertions(aAllowNegativeAssertions),
       mCoalesceDuplicateArcs(aCoalesceDuplicateArcs)
 {
-	NS_ADDREF(mCompositeDataSource);
+    NS_ADDREF(mCompositeDataSource);
 }
 
 
 CompositeEnumeratorImpl::~CompositeEnumeratorImpl(void)
 {
-	NS_IF_RELEASE(mCurrent);
-	NS_IF_RELEASE(mResult);
-	NS_RELEASE(mCompositeDataSource);
+    NS_IF_RELEASE(mCurrent);
+    NS_IF_RELEASE(mResult);
+    NS_RELEASE(mCompositeDataSource);
 }
 
 NS_IMPL_ADDREF(CompositeEnumeratorImpl)
-NS_IMPL_RELEASE_WITH_DESTROY(CompositeEnumeratorImpl, Destroy())
+NS_IMPL_RELEASE(CompositeEnumeratorImpl)
 NS_IMPL_QUERY_INTERFACE1(CompositeEnumeratorImpl, nsISimpleEnumerator)
 
 NS_IMETHODIMP
@@ -330,21 +323,6 @@ class CompositeArcsInOutEnumeratorImpl : public CompositeEnumeratorImpl
 public:
     enum Type { eArcsIn, eArcsOut };
 
-    static CompositeArcsInOutEnumeratorImpl*
-    Create(nsFixedSizeAllocator& aAllocator,
-           CompositeDataSourceImpl* aCompositeDataSource,
-           nsIRDFNode* aNode,
-           Type aType,
-           bool aAllowNegativeAssertions,
-           bool aCoalesceDuplicateArcs) {
-        void* place = aAllocator.Alloc(sizeof(CompositeArcsInOutEnumeratorImpl));
-        return place
-            ? ::new (place) CompositeArcsInOutEnumeratorImpl(aCompositeDataSource,
-                                                             aNode, aType,
-                                                             aAllowNegativeAssertions,
-                                                             aCoalesceDuplicateArcs)
-            : nullptr; }
-
     virtual ~CompositeArcsInOutEnumeratorImpl();
 
     virtual nsresult
@@ -353,9 +331,6 @@ public:
     virtual nsresult
     HasNegation(nsIRDFDataSource* aDataSource, nsIRDFNode* aNode, bool* aResult);
 
-    virtual void Destroy();
-
-protected:
     CompositeArcsInOutEnumeratorImpl(CompositeDataSourceImpl* aCompositeDataSource,
                                      nsIRDFNode* aNode,
                                      Type aType,
@@ -365,11 +340,6 @@ protected:
 private:
     nsIRDFNode* mNode;
     Type        mType;
-
-    // Hide so that only Create() and Destroy() can be used to
-    // allocate and deallocate from the heap
-    static void* operator new(size_t) CPP_THROW_NEW { return 0; }
-    static void operator delete(void*, size_t) {}
 };
 
 
@@ -416,18 +386,6 @@ CompositeArcsInOutEnumeratorImpl::HasNegation(
     return NS_OK;
 }
 
-void
-CompositeArcsInOutEnumeratorImpl::Destroy()
-{
-    // Keep the datasource alive for the duration of the stack
-    // frame so its allocator stays valid.
-    nsCOMPtr<nsIRDFCompositeDataSource> kungFuDeathGrip = mCompositeDataSource;
-
-    nsFixedSizeAllocator& pool = mCompositeDataSource->mAllocator;
-    this->~CompositeArcsInOutEnumeratorImpl();
-    pool.Free(this, sizeof(*this));
-}
-
 
 //----------------------------------------------------------------------
 //
@@ -437,33 +395,12 @@ CompositeArcsInOutEnumeratorImpl::Destroy()
 class CompositeAssertionEnumeratorImpl : public CompositeEnumeratorImpl
 {
 public:
-    static CompositeAssertionEnumeratorImpl*
-    Create(nsFixedSizeAllocator& aAllocator,
-           CompositeDataSourceImpl* aCompositeDataSource,
-           nsIRDFResource* aSource,
-           nsIRDFResource* aProperty,
-           nsIRDFNode* aTarget,
-           bool aTruthValue,
-           bool aAllowNegativeAssertions,
-           bool aCoalesceDuplicateArcs) {
-        void* place = aAllocator.Alloc(sizeof(CompositeAssertionEnumeratorImpl));
-        return place
-            ? ::new (place) CompositeAssertionEnumeratorImpl(aCompositeDataSource,
-                                                             aSource, aProperty, aTarget,
-                                                             aTruthValue,
-                                                             aAllowNegativeAssertions,
-                                                             aCoalesceDuplicateArcs)
-            : nullptr; }
-
     virtual nsresult
     GetEnumerator(nsIRDFDataSource* aDataSource, nsISimpleEnumerator** aResult);
 
     virtual nsresult
     HasNegation(nsIRDFDataSource* aDataSource, nsIRDFNode* aNode, bool* aResult);
 
-    virtual void Destroy();
-
-protected:
     CompositeAssertionEnumeratorImpl(CompositeDataSourceImpl* aCompositeDataSource,
                                      nsIRDFResource* aSource,
                                      nsIRDFResource* aProperty,
@@ -479,11 +416,6 @@ private:
     nsIRDFResource* mProperty;
     nsIRDFNode*     mTarget;
     bool            mTruthValue;
-
-    // Hide so that only Create() and Destroy() can be used to
-    // allocate and deallocate from the heap
-    static void* operator new(size_t) CPP_THROW_NEW { return 0; }
-    static void operator delete(void*, size_t) {}
 };
 
 
@@ -542,18 +474,6 @@ CompositeAssertionEnumeratorImpl::HasNegation(
     }
 }
 
-void
-CompositeAssertionEnumeratorImpl::Destroy()
-{
-    // Keep the datasource alive for the duration of the stack
-    // frame so its allocator stays valid.
-    nsCOMPtr<nsIRDFCompositeDataSource> kungFuDeathGrip = mCompositeDataSource;
-
-    nsFixedSizeAllocator& pool = mCompositeDataSource->mAllocator;
-    this->~CompositeAssertionEnumeratorImpl();
-    pool.Free(this, sizeof(*this));
-}
-
 ////////////////////////////////////////////////////////////////////////
 
 nsresult
@@ -574,17 +494,6 @@ CompositeDataSourceImpl::CompositeDataSourceImpl(void)
 	  mCoalesceDuplicateArcs(true),
       mUpdateBatchNest(0)
 {
-    static const size_t kBucketSizes[] = {
-        sizeof(CompositeAssertionEnumeratorImpl),
-        sizeof(CompositeArcsInOutEnumeratorImpl) };
-
-    static const int32_t kNumBuckets = sizeof(kBucketSizes) / sizeof(size_t);
-
-    // Per news://news.mozilla.org/39BEC105.5090206%40netscape.com
-    static const int32_t kInitialSize = 256;
-
-    mAllocator.Init("nsCompositeDataSource", kBucketSizes, kNumBuckets, kInitialSize);
-
 #ifdef PR_LOGGING
     if (nsRDFLog == nullptr) 
         nsRDFLog = PR_NewLogModule("RDF");
@@ -596,7 +505,6 @@ CompositeDataSourceImpl::CompositeDataSourceImpl(void)
 // nsISupports interface
 //
 
-NS_IMPL_CYCLE_COLLECTION_CLASS(CompositeDataSourceImpl)
 NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(CompositeDataSourceImpl)
     uint32_t i, count = tmp->mDataSources.Count();
     for (i = count; i > 0; --i) {
@@ -687,11 +595,10 @@ CompositeDataSourceImpl::GetSources(nsIRDFResource* aProperty,
     if (! mAllowNegativeAssertions && ! aTruthValue)
         return(NS_RDF_NO_VALUE);
 
-    *aResult = CompositeAssertionEnumeratorImpl::Create(mAllocator,
-                                                        this, nullptr, aProperty,
-                                                        aTarget, aTruthValue,
-                                                        mAllowNegativeAssertions,
-                                                        mCoalesceDuplicateArcs);
+    *aResult = new CompositeAssertionEnumeratorImpl(this, nullptr, aProperty,
+                                                    aTarget, aTruthValue,
+                                                    mAllowNegativeAssertions,
+                                                    mCoalesceDuplicateArcs);
 
     if (! *aResult)
         return NS_ERROR_OUT_OF_MEMORY;
@@ -794,11 +701,11 @@ CompositeDataSourceImpl::GetTargets(nsIRDFResource* aSource,
         return(NS_RDF_NO_VALUE);
 
     *aResult =
-        CompositeAssertionEnumeratorImpl::Create(mAllocator, this,
-                                                 aSource, aProperty, nullptr,
-                                                 aTruthValue,
-                                                 mAllowNegativeAssertions,
-                                                 mCoalesceDuplicateArcs);
+        new CompositeAssertionEnumeratorImpl(this,
+                                             aSource, aProperty, nullptr,
+                                             aTruthValue,
+                                             mAllowNegativeAssertions,
+                                             mCoalesceDuplicateArcs);
 
     if (! *aResult)
         return NS_ERROR_OUT_OF_MEMORY;
@@ -1125,11 +1032,11 @@ CompositeDataSourceImpl::ArcLabelsIn(nsIRDFNode* aTarget, nsISimpleEnumerator**
     if (! aResult)
         return NS_ERROR_NULL_POINTER;
 
-    nsISimpleEnumerator* result = 
-        CompositeArcsInOutEnumeratorImpl::Create(mAllocator, this, aTarget,
-                                                 CompositeArcsInOutEnumeratorImpl::eArcsIn,
-                                                 mAllowNegativeAssertions,
-                                                 mCoalesceDuplicateArcs);
+    nsISimpleEnumerator* result =
+        new CompositeArcsInOutEnumeratorImpl(this, aTarget,
+                                             CompositeArcsInOutEnumeratorImpl::eArcsIn,
+                                             mAllowNegativeAssertions,
+                                             mCoalesceDuplicateArcs);
 
     if (! result)
         return NS_ERROR_OUT_OF_MEMORY;
@@ -1152,10 +1059,10 @@ CompositeDataSourceImpl::ArcLabelsOut(nsIRDFResource* aSource,
         return NS_ERROR_NULL_POINTER;
 
     nsISimpleEnumerator* result =
-        CompositeArcsInOutEnumeratorImpl::Create(mAllocator, this, aSource,
-                                                 CompositeArcsInOutEnumeratorImpl::eArcsOut,
-                                                 mAllowNegativeAssertions,
-                                                 mCoalesceDuplicateArcs);
+        new CompositeArcsInOutEnumeratorImpl(this, aSource,
+                                             CompositeArcsInOutEnumeratorImpl::eArcsOut,
+                                             mAllowNegativeAssertions,
+                                             mCoalesceDuplicateArcs);
 
     if (! result)
         return NS_ERROR_OUT_OF_MEMORY;
