@@ -33,7 +33,7 @@ static nsresult GetExtensionFromWindowsMimeDatabase(const nsACString& aMimeType,
 
 nsOSHelperAppService::nsOSHelperAppService() : 
   nsExternalHelperAppService()
-  , mAppAssoc(nsnull)
+  , mAppAssoc(nullptr)
 {
   CoInitialize(NULL);
   CoCreateInstance(CLSID_ApplicationAssociationRegistration, NULL, CLSCTX_INPROC,
@@ -44,7 +44,7 @@ nsOSHelperAppService::~nsOSHelperAppService()
 {
   if (mAppAssoc)
     mAppAssoc->Release();
-  mAppAssoc = nsnull;
+  mAppAssoc = nullptr;
   CoUninitialize();
 }
 
@@ -102,7 +102,7 @@ static nsresult GetExtensionFrom4xRegistryInfo(const nsACString& aMimeType,
   // this may be a comma separated list of extensions...just take the 
   // first one for now...
 
-  PRInt32 pos = aFileExtension.FindChar(PRUnichar(','));
+  int32_t pos = aFileExtension.FindChar(PRUnichar(','));
   if (pos > 0) {
     // we have a comma separated list of types...
     // truncate everything after the first comma (including the comma)
@@ -120,7 +120,7 @@ nsresult nsOSHelperAppService::OSProtocolHandlerExists(const char * aProtocolSch
   {
     // Vista: use new application association interface
     if (mAppAssoc) {
-      PRUnichar * pResult = nsnull;
+      PRUnichar * pResult = nullptr;
       NS_ConvertASCIItoUTF16 scheme(aProtocolScheme);
       // We are responsible for freeing returned strings.
       HRESULT hr = mAppAssoc->QueryCurrentDefault(scheme.get(),
@@ -162,7 +162,7 @@ NS_IMETHODIMP nsOSHelperAppService::GetApplicationDescription(const nsACString&
 
   // Vista: use new application association interface
   if (mAppAssoc) {
-    PRUnichar * pResult = nsnull;
+    PRUnichar * pResult = nullptr;
     // We are responsible for freeing returned strings.
     HRESULT hr = mAppAssoc->QueryCurrentDefault(buf.get(),
                                                 AT_URLPROTOCOL, AL_EFFECTIVE,
@@ -276,14 +276,14 @@ static void CleanupHandlerPath(nsString& aPath)
   // 3) C:\Windows\some.dll,-foo -bar
   // 4) C:\Windows\some.cpl,-foo -bar
 
-  PRInt32 lastCommaPos = aPath.RFindChar(',');
+  int32_t lastCommaPos = aPath.RFindChar(',');
   if (lastCommaPos != kNotFound)
     aPath.Truncate(lastCommaPos);
 
   aPath.AppendLiteral(" ");
 
   // case insensitive
-  PRUint32 index = aPath.Find(".exe ", true);
+  uint32_t index = aPath.Find(".exe ", true);
   if (index == kNotFound)
     index = aPath.Find(".dll ", true);
   if (index == kNotFound)
@@ -308,15 +308,15 @@ static void StripRundll32(nsString& aCommandString)
   NS_NAMED_LITERAL_STRING(rundllSegmentShort, "rundll32 ");
 
   // case insensitive
-  PRInt32 strLen = rundllSegment.Length();
-  PRInt32 index = aCommandString.Find(rundllSegment, true);
+  int32_t strLen = rundllSegment.Length();
+  int32_t index = aCommandString.Find(rundllSegment, true);
   if (index == kNotFound) {
     strLen = rundllSegmentShort.Length();
     index = aCommandString.Find(rundllSegmentShort, true);
   }
 
   if (index != kNotFound) {
-    PRUint32 rundllSegmentLength = index + strLen;
+    uint32_t rundllSegmentLength = index + strLen;
     aCommandString.Cut(0, rundllSegmentLength);
   }
 }
@@ -345,7 +345,7 @@ static void StripRundll32(nsString& aCommandString)
   //    Viewer.dll", var var
 
   // Expand environment variables so we have full path strings.
-  PRUint32 bufLength = ::ExpandEnvironmentStringsW(handlerCommand.get(),
+  uint32_t bufLength = ::ExpandEnvironmentStringsW(handlerCommand.get(),
                                                    L"", 0);
   if (bufLength == 0) // Error
     return false;
@@ -388,7 +388,7 @@ nsOSHelperAppService::GetDefaultAppInfo(const nsAString& aAppInfo,
   // If all else fails, use the file type key name, which will be 
   // something like "pngfile" for .pngs, "WMVFile" for .wmvs, etc. 
   aDefaultDescription = aAppInfo;
-  *aDefaultApplication = nsnull;
+  *aDefaultApplication = nullptr;
 
   if (aAppInfo.IsEmpty())
     return NS_ERROR_FAILURE;
@@ -428,8 +428,38 @@ nsOSHelperAppService::GetDefaultAppInfo(const nsAString& aAppInfo,
      
     // OK, the default value here is the description of the type.
     rv = regKey->ReadStringValue(EmptyString(), handlerCommand);
-    if (NS_FAILED(rv))
-      return NS_ERROR_FAILURE;
+    if (NS_FAILED(rv)) {
+
+      // Check if there is a DelegateExecute string
+      nsAutoString delegateExecute;
+      rv = regKey->ReadStringValue(NS_LITERAL_STRING("DelegateExecute"), delegateExecute);
+      NS_ENSURE_SUCCESS(rv, rv);
+
+      // Look for InProcServer32
+      nsAutoString delegateExecuteRegPath;
+      delegateExecuteRegPath.AssignLiteral("CLSID\\");
+      delegateExecuteRegPath.Append(delegateExecute);
+      delegateExecuteRegPath.AppendLiteral("\\InProcServer32");
+      rv = chkKey->Open(nsIWindowsRegKey::ROOT_KEY_CLASSES_ROOT,
+                        delegateExecuteRegPath, 
+                        nsIWindowsRegKey::ACCESS_QUERY_VALUE);
+      if (NS_SUCCEEDED(rv)) {
+        rv = chkKey->ReadStringValue(EmptyString(), handlerCommand);
+      }
+
+      if (NS_FAILED(rv)) {
+        // Look for LocalServer32
+        delegateExecuteRegPath.AssignLiteral("CLSID\\");
+        delegateExecuteRegPath.Append(delegateExecute);
+        delegateExecuteRegPath.AppendLiteral("\\LocalServer32");
+        rv = chkKey->Open(nsIWindowsRegKey::ROOT_KEY_CLASSES_ROOT,
+                          delegateExecuteRegPath, 
+                          nsIWindowsRegKey::ACCESS_QUERY_VALUE);
+        NS_ENSURE_SUCCESS(rv, rv);
+        rv = chkKey->ReadStringValue(EmptyString(), handlerCommand);
+        NS_ENSURE_SUCCESS(rv, rv);
+      }
+    }
   }
 
   if (!CleanupCmdHandlerPath(handlerCommand))
@@ -440,12 +470,12 @@ nsOSHelperAppService::GetDefaultAppInfo(const nsAString& aAppInfo,
   // There are some rare cases this can happen - ["url.dll" -foo]
   // for example won't resolve correctly to the system dir. The 
   // subsequent launch of the helper app will work though.
-  nsCOMPtr<nsILocalFile> lf;
+  nsCOMPtr<nsIFile> lf;
   NS_NewLocalFile(handlerCommand, true, getter_AddRefs(lf));
   if (!lf)
     return NS_ERROR_FILE_NOT_FOUND;
 
-  nsILocalFileWin* lfw = nsnull;
+  nsILocalFileWin* lfw = nullptr;
   CallQueryInterface(lf, &lfw);
 
   if (lfw) {
@@ -461,7 +491,7 @@ nsOSHelperAppService::GetDefaultAppInfo(const nsAString& aAppInfo,
 already_AddRefed<nsMIMEInfoWin> nsOSHelperAppService::GetByExtension(const nsAFlatString& aFileExt, const char *aTypeHint)
 {
   if (aFileExt.IsEmpty())
-    return nsnull;
+    return nullptr;
 
   // windows registry assumes your file extension is going to include the '.'.
   // so make sure it's there...
@@ -475,15 +505,15 @@ already_AddRefed<nsMIMEInfoWin> nsOSHelperAppService::GetByExtension(const nsAFl
   nsCOMPtr<nsIWindowsRegKey> regKey = 
     do_CreateInstance("@mozilla.org/windows-registry-key;1");
   if (!regKey) 
-    return nsnull; 
+    return nullptr; 
 
   nsresult rv = regKey->Open(nsIWindowsRegKey::ROOT_KEY_CLASSES_ROOT,
                              fileExtToUse,
                              nsIWindowsRegKey::ACCESS_QUERY_VALUE);
   if (NS_FAILED(rv))
-    return nsnull; 
+    return nullptr; 
 
-  nsCAutoString typeToUse;
+  nsAutoCString typeToUse;
   if (aTypeHint && *aTypeHint) {
     typeToUse.Assign(aTypeHint);
   }
@@ -491,17 +521,13 @@ already_AddRefed<nsMIMEInfoWin> nsOSHelperAppService::GetByExtension(const nsAFl
     nsAutoString temp;
     if (NS_FAILED(regKey->ReadStringValue(NS_LITERAL_STRING("Content Type"),
                   temp)) || temp.IsEmpty()) {
-      return nsnull; 
+      return nullptr; 
     }
     // Content-Type is always in ASCII
     LossyAppendUTF16toASCII(temp, typeToUse);
   }
 
-  nsMIMEInfoWin* mimeInfo = new nsMIMEInfoWin(typeToUse);
-  if (!mimeInfo)
-    return nsnull; // out of memory
-
-  NS_ADDREF(mimeInfo);
+  nsRefPtr<nsMIMEInfoWin> mimeInfo = new nsMIMEInfoWin(typeToUse);
 
   // don't append the '.'
   mimeInfo->AppendExtension(NS_ConvertUTF16toUTF8(Substring(fileExtToUse, 1)));
@@ -515,7 +541,7 @@ already_AddRefed<nsMIMEInfoWin> nsOSHelperAppService::GetByExtension(const nsAFl
     // Vista: use the new application association COM interfaces
     // for resolving helpers.
     nsString assocType(fileExtToUse);
-    PRUnichar * pResult = nsnull;
+    PRUnichar * pResult = nullptr;
     HRESULT hr = mAppAssoc->QueryCurrentDefault(assocType.get(),
                                                 AT_FILEEXTENSION, AL_EFFECTIVE,
                                                 &pResult);
@@ -539,8 +565,7 @@ already_AddRefed<nsMIMEInfoWin> nsOSHelperAppService::GetByExtension(const nsAFl
     found = false;
 
   if (!found) {
-    NS_IF_RELEASE(mimeInfo); // we failed to really find an entry in the registry
-    return nsnull;
+    return nullptr;
   }
 
   // Get other nsIMIMEInfo fields from registry, if possible.
@@ -549,8 +574,7 @@ already_AddRefed<nsMIMEInfoWin> nsOSHelperAppService::GetByExtension(const nsAFl
   
   if (NS_FAILED(GetDefaultAppInfo(appInfo, defaultDescription,
                                   getter_AddRefs(defaultApplication)))) {
-    NS_IF_RELEASE(mimeInfo);
-    return nsnull;
+    return nullptr;
   }
 
   mimeInfo->SetDefaultDescription(defaultDescription);
@@ -559,7 +583,7 @@ already_AddRefed<nsMIMEInfoWin> nsOSHelperAppService::GetByExtension(const nsAFl
   // Grab the general description
   GetMIMEInfoFromRegistry(appInfo, mimeInfo);
 
-  return mimeInfo;
+  return mimeInfo.forget();
 }
 
 already_AddRefed<nsIMIMEInfo> nsOSHelperAppService::GetMIMEInfoFromOS(const nsACString& aMIMEType, const nsACString& aFileExt, bool *aFound)
@@ -589,10 +613,10 @@ already_AddRefed<nsIMIMEInfo> nsOSHelperAppService::GetMIMEInfoFromOS(const nsAC
     }
   }
   // If we found an extension for the type, do the lookup
-  nsMIMEInfoWin* mi = nsnull;
+  nsRefPtr<nsMIMEInfoWin> mi;
   if (!fileExtension.IsEmpty())
-    mi = GetByExtension(fileExtension, flatType.get()).get();
-  LOG(("Extension lookup on '%s' found: 0x%p\n", fileExtension.get(), mi));
+    mi = GetByExtension(fileExtension, flatType.get());
+  LOG(("Extension lookup on '%s' found: 0x%p\n", fileExtension.get(), mi.get()));
 
   bool hasDefault = false;
   if (mi) {
@@ -616,21 +640,18 @@ already_AddRefed<nsIMIMEInfo> nsOSHelperAppService::GetMIMEInfoFromOS(const nsAC
       GetByExtension(NS_ConvertUTF8toUTF16(aFileExt), flatType.get());
     LOG(("Ext. lookup for '%s' found 0x%p\n", flatExt.get(), miByExt.get()));
     if (!miByExt && mi)
-      return mi;
+      return mi.forget();
     if (miByExt && !mi) {
-      miByExt.swap(mi);
-      return mi;
+      return miByExt.forget();
     }
     if (!miByExt && !mi) {
       *aFound = false;
       mi = new nsMIMEInfoWin(flatType);
-      if (mi) {
-        NS_ADDREF(mi);
-        if (!aFileExt.IsEmpty())
-          mi->AppendExtension(aFileExt);
+      if (!aFileExt.IsEmpty()) {
+        mi->AppendExtension(aFileExt);
       }
       
-      return mi;
+      return mi.forget();
     }
 
     // if we get here, mi has no default app. copy from extension lookup.
@@ -640,7 +661,7 @@ already_AddRefed<nsIMIMEInfo> nsOSHelperAppService::GetMIMEInfoFromOS(const nsAC
 
     mi->SetDefaultDescription(desc);
   }
-  return mi;
+  return mi.forget();
 }
 
 NS_IMETHODIMP
