@@ -10,11 +10,9 @@
 #include "nsViewManager.h"
 #include "nsGfxCIID.h"
 #include "nsView.h"
-#include "nsISupportsArray.h"
 #include "nsCOMPtr.h"
 #include "nsGUIEvent.h"
 #include "nsRegion.h"
-#include "nsHashtable.h"
 #include "nsCOMArray.h"
 #include "nsIPluginWidget.h"
 #include "nsXULPopupManager.h"
@@ -22,9 +20,12 @@
 #include "nsPresContext.h"
 #include "nsEventStateManager.h"
 #include "mozilla/StartupTimeline.h"
-#include "sampler.h"
+#include "GeckoProfiler.h"
 #include "nsRefreshDriver.h"
 #include "mozilla/Preferences.h"
+#include "nsContentUtils.h"
+#include "nsLayoutUtils.h"
+#include "mozilla/layers/Compositor.h"
 
 /**
    XXX TODO XXX
@@ -42,6 +43,8 @@
    we ask for a specific z-order, we don't assume that widget z-ordering actually works.
 */
 
+using namespace mozilla::layers;
+
 #define NSCOORD_NONE      INT32_MIN
 
 #undef DEBUG_MOUSE_LOCATION
@@ -126,16 +129,14 @@ nsViewManager::Init(nsDeviceContext* aContext)
 
 nsView*
 nsViewManager::CreateView(const nsRect& aBounds,
-                          const nsView* aParent,
+                          nsView* aParent,
                           nsViewVisibility aVisibilityFlag)
 {
   nsView *v = new nsView(this, aVisibilityFlag);
-  if (v) {
-    v->SetParent(const_cast<nsView*>(aParent));
-    v->SetPosition(aBounds.x, aBounds.y);
-    nsRect dim(0, 0, aBounds.width, aBounds.height);
-    v->SetDimensions(dim, false);
-  }
+  v->SetParent(aParent);
+  v->SetPosition(aBounds.x, aBounds.y);
+  nsRect dim(0, 0, aBounds.width, aBounds.height);
+  v->SetDimensions(dim, false);
   return v;
 }
 
@@ -288,8 +289,7 @@ nsView* nsViewManager::GetDisplayRootFor(nsView* aView)
    aContext may be null, in which case layers should be used for
    rendering.
 */
-void nsViewManager::Refresh(nsView *aView, const nsIntRegion& aRegion,
-                            bool aWillSendDidPaint)
+void nsViewManager::Refresh(nsView *aView, const nsIntRegion& aRegion)
 {
   NS_ASSERTION(aView->GetViewManager() == this, "wrong view manager");
 
@@ -328,14 +328,23 @@ void nsViewManager::Refresh(nsView *aView, const nsIntRegion& aRegion,
                  "Widgets that we paint must all be display roots");
 
     if (mPresShell) {
-#ifdef DEBUG_INVALIDATIONS
-      printf("--COMPOSITE-- %p\n", mPresShell);
+#ifdef MOZ_DUMP_PAINTING
+      if (nsLayoutUtils::InvalidationDebuggingIsEnabled()) {
+        printf("--COMPOSITE-- %p\n", mPresShell);
+      }
 #endif
-      mPresShell->Paint(aView, damageRegion,
-                        nsIPresShell::PAINT_COMPOSITE |
-                        (aWillSendDidPaint ? nsIPresShell::PAINT_WILL_SEND_DID_PAINT : 0));
-#ifdef DEBUG_INVALIDATIONS
-      printf("--ENDCOMPOSITE--\n");
+      uint32_t paintFlags = nsIPresShell::PAINT_COMPOSITE;
+      LayerManager *manager = widget->GetLayerManager();
+      if (!manager->NeedsWidgetInvalidation()) {
+        manager->FlushRendering();
+      } else {
+        mPresShell->Paint(aView, damageRegion,
+                          paintFlags);
+      }
+#ifdef MOZ_DUMP_PAINTING
+      if (nsLayoutUtils::InvalidationDebuggingIsEnabled()) {
+        printf("--ENDCOMPOSITE--\n");
+      }
 #endif
       mozilla::StartupTimeline::RecordOnce(mozilla::StartupTimeline::FIRST_PAINT);
     }
@@ -384,23 +393,25 @@ void nsViewManager::ProcessPendingUpdatesForView(nsView* aView,
             vm->mRootView->IsEffectivelyVisible() &&
             mPresShell && mPresShell->IsVisible()) {
           vm->FlushDelayedResize(true);
-          vm->InvalidateView(vm->mRootView);
         }
       }
 
       NS_ASSERTION(aView->HasWidget(), "Must have a widget!");
 
-      SetPainting(true);
-#ifdef DEBUG_INVALIDATIONS
-      printf("---- PAINT START ----PresShell(%p), nsView(%p), nsIWidget(%p)\n", mPresShell, aView, widget);
+#ifdef MOZ_DUMP_PAINTING
+      if (nsLayoutUtils::InvalidationDebuggingIsEnabled()) {
+        printf("---- PAINT START ----PresShell(%p), nsView(%p), nsIWidget(%p)\n", mPresShell, aView, widget);
+      }
 #endif
       nsAutoScriptBlocker scriptBlocker;
       NS_ASSERTION(aView->HasWidget(), "Must have a widget!");
+      SetPainting(true);
       mPresShell->Paint(aView, nsRegion(),
-                        nsIPresShell::PAINT_LAYERS |
-                        nsIPresShell::PAINT_WILL_SEND_DID_PAINT);
-#ifdef DEBUG_INVALIDATIONS
-      printf("---- PAINT END ----\n");
+                        nsIPresShell::PAINT_LAYERS);
+#ifdef MOZ_DUMP_PAINTING
+      if (nsLayoutUtils::InvalidationDebuggingIsEnabled()) {
+        printf("---- PAINT END ----\n");
+      }
 #endif
       aView->SetForcedRepaint(false);
       SetPainting(false);
@@ -614,11 +625,13 @@ void nsViewManager::InvalidateViews(nsView *aView)
   }
 }
 
-void nsViewManager::WillPaintWindow(nsIWidget* aWidget, bool aWillSendDidPaint)
+void nsViewManager::WillPaintWindow(nsIWidget* aWidget)
 {
   if (aWidget) {
     nsView* view = nsView::GetViewFor(aWidget);
-    if (view && view->ForcedRepaint()) {
+    LayerManager *manager = aWidget->GetLayerManager();
+    if (view &&
+        (view->ForcedRepaint() || !manager->NeedsWidgetInvalidation())) {
       ProcessPendingUpdates();
       // Re-get the view pointer here since the ProcessPendingUpdates might have
       // destroyed it during CallWillPaintOnObservers.
@@ -631,13 +644,12 @@ void nsViewManager::WillPaintWindow(nsIWidget* aWidget, bool aWillSendDidPaint)
 
   nsCOMPtr<nsIPresShell> shell = mPresShell;
   if (shell) {
-    shell->WillPaintWindow(aWillSendDidPaint);
+    shell->WillPaintWindow();
   }
 }
 
-bool nsViewManager::PaintWindow(nsIWidget* aWidget, nsIntRegion aRegion,
-                                uint32_t aFlags)
- {
+bool nsViewManager::PaintWindow(nsIWidget* aWidget, nsIntRegion aRegion)
+{
   if (!aWidget || !mContext)
     return false;
 
@@ -648,7 +660,7 @@ bool nsViewManager::PaintWindow(nsIWidget* aWidget, nsIntRegion aRegion,
   // destroyed it during CallWillPaintOnObservers (bug 378273).
   nsView* view = nsView::GetViewFor(aWidget);
   if (view && !aRegion.IsEmpty()) {
-    Refresh(view, aRegion, (aFlags & nsIWidgetListener::WILL_SEND_DID_PAINT));
+    Refresh(view, aRegion);
   }
 
   return true;
@@ -665,7 +677,7 @@ void nsViewManager::DidPaintWindow()
 void
 nsViewManager::DispatchEvent(nsGUIEvent *aEvent, nsView* aView, nsEventStatus* aStatus)
 {
-  SAMPLE_LABEL("event", "nsViewManager::DispatchEvent");
+  PROFILER_LABEL("event", "nsViewManager::DispatchEvent");
 
   if ((NS_IS_MOUSE_EVENT(aEvent) &&
        // Ignore mouse events that we synthesize.
@@ -840,11 +852,6 @@ nsViewManager::InsertChild(nsView *aParent, nsView *aChild, nsView *aSibling,
       // if the parent view is marked as "floating", make the newly added view float as well.
       if (aParent->GetFloating())
         aChild->SetFloating(true);
-
-      //and mark this area as dirty if the view is visible...
-
-      if (nsViewVisibility_kHide != aChild->GetVisibility())
-        aChild->GetViewManager()->InvalidateView(aChild);
     }
 }
 
@@ -867,7 +874,6 @@ nsViewManager::RemoveChild(nsView *aChild)
   if (nullptr != parent) {
     NS_ASSERTION(aChild->GetViewManager() == this ||
                  parent->GetViewManager() == this, "wrong view manager");
-    aChild->GetViewManager()->InvalidateView(aChild);
     parent->RemoveChild(aChild);
   }
 }
@@ -876,53 +882,7 @@ void
 nsViewManager::MoveViewTo(nsView *aView, nscoord aX, nscoord aY)
 {
   NS_ASSERTION(aView->GetViewManager() == this, "wrong view manager");
-  nsPoint oldPt = aView->GetPosition();
-  nsRect oldBounds = aView->GetBoundsInParentUnits();
   aView->SetPosition(aX, aY);
-
-  // only do damage control if the view is visible
-
-  if ((aX != oldPt.x) || (aY != oldPt.y)) {
-    if (aView->GetVisibility() != nsViewVisibility_kHide) {
-      nsView* parentView = aView->GetParent();
-      if (parentView) {
-        nsViewManager* parentVM = parentView->GetViewManager();
-        parentVM->InvalidateView(parentView, oldBounds);
-        parentVM->InvalidateView(parentView, aView->GetBoundsInParentUnits());
-      }
-    }
-  }
-}
-
-void nsViewManager::InvalidateHorizontalBandDifference(nsView *aView, const nsRect& aRect, const nsRect& aCutOut,
-  nscoord aY1, nscoord aY2, bool aInCutOut) {
-  nscoord height = aY2 - aY1;
-  if (aRect.x < aCutOut.x) {
-    nsRect r(aRect.x, aY1, aCutOut.x - aRect.x, height);
-    InvalidateView(aView, r);
-  }
-  if (!aInCutOut && aCutOut.x < aCutOut.XMost()) {
-    nsRect r(aCutOut.x, aY1, aCutOut.width, height);
-    InvalidateView(aView, r);
-  }
-  if (aCutOut.XMost() < aRect.XMost()) {
-    nsRect r(aCutOut.XMost(), aY1, aRect.XMost() - aCutOut.XMost(), height);
-    InvalidateView(aView, r);
-  }
-}
-
-void nsViewManager::InvalidateRectDifference(nsView *aView, const nsRect& aRect, const nsRect& aCutOut) {
-  NS_ASSERTION(aView->GetViewManager() == this,
-               "InvalidateRectDifference called on view we don't own");
-  if (aRect.y < aCutOut.y) {
-    InvalidateHorizontalBandDifference(aView, aRect, aCutOut, aRect.y, aCutOut.y, false);
-  }
-  if (aCutOut.y < aCutOut.YMost()) {
-    InvalidateHorizontalBandDifference(aView, aRect, aCutOut, aCutOut.y, aCutOut.YMost(), true);
-  }
-  if (aCutOut.YMost() < aRect.YMost()) {
-    InvalidateHorizontalBandDifference(aView, aRect, aCutOut, aCutOut.YMost(), aRect.YMost(), false);
-  }
 }
 
 void
@@ -932,28 +892,7 @@ nsViewManager::ResizeView(nsView *aView, const nsRect &aRect, bool aRepaintExpos
 
   nsRect oldDimensions = aView->GetDimensions();
   if (!oldDimensions.IsEqualEdges(aRect)) {
-    // resize the view.
-    // Prevent Invalidation of hidden views 
-    if (aView->GetVisibility() == nsViewVisibility_kHide) {
-      aView->SetDimensions(aRect, false);
-    } else {
-      nsView* parentView = aView->GetParent();
-      if (!parentView) {
-        parentView = aView;
-      }
-      nsRect oldBounds = aView->GetBoundsInParentUnits();
-      aView->SetDimensions(aRect, true);
-      nsViewManager* parentVM = parentView->GetViewManager();
-      if (!aRepaintExposedAreaOnly) {
-        // Invalidate the union of the old and new size
-        InvalidateView(aView, aRect);
-        parentVM->InvalidateView(parentView, oldBounds);
-      } else {
-        InvalidateRectDifference(aView, aRect, oldDimensions);
-        nsRect newBounds = aView->GetBoundsInParentUnits();
-        parentVM->InvalidateRectDifference(parentView, oldBounds, newBounds);
-      } 
-    }
+    aView->SetDimensions(aRect, true);
   }
 
   // Note that if layout resizes the view and the view has a custom clip
@@ -978,21 +917,6 @@ nsViewManager::SetViewVisibility(nsView *aView, nsViewVisibility aVisible)
 
   if (aVisible != aView->GetVisibility()) {
     aView->SetVisibility(aVisible);
-
-    if (IsViewInserted(aView)) {
-      if (!aView->HasWidget()) {
-        if (nsViewVisibility_kHide == aVisible) {
-          nsView* parentView = aView->GetParent();
-          if (parentView) {
-            parentView->GetViewManager()->
-              InvalidateView(parentView, aView->GetBoundsInParentUnits());
-          }
-        }
-        else {
-          InvalidateView(aView);
-        }
-      }
-    }
   }
 }
 
@@ -1025,27 +949,11 @@ nsViewManager::SetViewZIndex(nsView *aView, bool aAutoZIndex, int32_t aZIndex, b
     return;
   }
 
-  bool oldTopMost = aView->IsTopMost();
-  bool oldIsAuto = aView->GetZIndexIsAuto();
-
   if (aAutoZIndex) {
     aZIndex = 0;
   }
 
-  int32_t oldidx = aView->GetZIndex();
   aView->SetZIndex(aAutoZIndex, aZIndex, aTopMost);
-
-  if (oldidx != aZIndex || oldTopMost != aTopMost ||
-      oldIsAuto != aAutoZIndex) {
-    InvalidateView(aView);
-  }
-}
-
-void
-nsViewManager::GetDeviceContext(nsDeviceContext *&aContext)
-{
-  aContext = mContext;
-  NS_IF_ADDREF(aContext);
 }
 
 nsViewManager*
@@ -1116,7 +1024,7 @@ nsViewManager::ProcessPendingUpdates()
 
   // Flush things like reflows by calling WillPaint on observer presShells.
   if (mPresShell) {
-    CallWillPaintOnObservers(true);
+    CallWillPaintOnObservers();
   }
   ProcessPendingUpdatesForView(mRootView, true);
 }
@@ -1136,7 +1044,7 @@ nsViewManager::UpdateWidgetGeometry()
 }
 
 void
-nsViewManager::CallWillPaintOnObservers(bool aWillSendDidPaint)
+nsViewManager::CallWillPaintOnObservers()
 {
   NS_PRECONDITION(IsRootVM(), "Must be root VM for this to be called!");
 
@@ -1148,7 +1056,7 @@ nsViewManager::CallWillPaintOnObservers(bool aWillSendDidPaint)
       if (vm->mRootView && vm->mRootView->IsEffectivelyVisible()) {
         nsCOMPtr<nsIPresShell> shell = vm->GetPresShell();
         if (shell) {
-          shell->WillPaint(aWillSendDidPaint);
+          shell->WillPaint();
         }
       }
     }
