@@ -29,12 +29,15 @@
 #include "nsDirectoryServiceDefs.h"
 #include "nsITimer.h"
 #include "nsThreadUtils.h"
+#include "mozilla/Preferences.h"
 
 #include "WinUtils.h"
 #include "mozilla/LazyIdleThread.h"
+#include <algorithm>
 
 
 using namespace mozilla;
+using namespace mozilla::widget;
 
 #define DEFAULT_THREAD_TIMEOUT_MS 30000
 
@@ -193,7 +196,7 @@ STDMETHODIMP nsDataObj::CStream::Read(void* pvBuffer,
   // Bytes left for Windows to read out of our buffer
   ULONG bytesLeft = mChannelData.Length() - mStreamRead;
   // Let Windows know what we will hand back, usually this is the entire buffer
-  *nBytesRead = NS_MIN(bytesLeft, nBytesToRead);
+  *nBytesRead = std::min(bytesLeft, nBytesToRead);
   // Copy the buffer data over
   memcpy(pvBuffer, ((char*)mChannelData.Elements() + mStreamRead), *nBytesRead);
   // Update our bytes read tracking
@@ -941,7 +944,7 @@ CreateFilenameFromTextA(nsString & aText, const char * aExtension,
   // way ensures that even in MBCS environments there will be a valid MBCS filename of
   // the correct length.
   int maxUsableFilenameLen = aFilenameLen - strlen(aExtension) - 1; // space for ext + null byte
-  int currLen, textLen = (int) NS_MIN(aText.Length(), aFilenameLen);
+  int currLen, textLen = (int) std::min(aText.Length(), aFilenameLen);
   char defaultChar = '_';
   do {
     currLen = WideCharToMultiByte(CP_ACP, 
@@ -1096,6 +1099,7 @@ nsDataObj :: GetFileDescriptorInternetShortcutW ( FORMATETC& aFE, STGMEDIUM& aST
 HRESULT
 nsDataObj :: GetFileContentsInternetShortcut ( FORMATETC& aFE, STGMEDIUM& aSTG )
 {
+  static const char * kShellIconPref = "browser.shell.shortcutFavicons";
   nsAutoString url;
   if ( NS_FAILED(ExtractShortcutURL(url)) )
     return E_OUTOFMEMORY;
@@ -1108,22 +1112,35 @@ nsDataObj :: GetFileContentsInternetShortcut ( FORMATETC& aFE, STGMEDIUM& aSTG )
   nsCOMPtr<nsIURI> aUri;
   NS_NewURI(getter_AddRefs(aUri), url);
 
-  nsAutoString aUriHash;
-
-  mozilla::widget::FaviconHelper::ObtainCachedIconFile(aUri, aUriHash, mIOThread, true);
-
-  nsresult rv = mozilla::widget::FaviconHelper::GetOutputIconPath(aUri, icoFile, true);
-  NS_ENSURE_SUCCESS(rv, E_FAIL);
+  const char *shortcutFormatStr;
+  int totalLen;
   nsCString path;
-  rv = icoFile->GetNativePath(path);
-  NS_ENSURE_SUCCESS(rv, E_FAIL);
+  if (!Preferences::GetBool(kShellIconPref, true) ||
+      WinUtils::GetWindowsVersion() < WinUtils::VISTA_VERSION) {
+    shortcutFormatStr = "[InternetShortcut]\r\nURL=%s\r\n";
+    const int formatLen = strlen(shortcutFormatStr) - 2;  // don't include %s
+    totalLen = formatLen + asciiUrl.Length();  // don't include null character
+  } else {
+    nsCOMPtr<nsIFile> icoFile;
+    nsCOMPtr<nsIURI> aUri;
+    NS_NewURI(getter_AddRefs(aUri), url);
 
-  static const char* shortcutFormatStr = "[InternetShortcut]\r\nURL=%s\r\n"
-                                         "IDList=\r\nHotKey=0\r\nIconFile=%s\r\n"
-                                         "IconIndex=0\r\n";
-  static const int formatLen = strlen(shortcutFormatStr) - 2*2; // don't include %s (2 times) in the len
-  const int totalLen = formatLen + asciiUrl.Length() 
-                       + path.Length(); // we don't want a null character on the end
+    nsAutoString aUriHash;
+
+    mozilla::widget::FaviconHelper::ObtainCachedIconFile(aUri, aUriHash, mIOThread, true);
+
+    nsresult rv = mozilla::widget::FaviconHelper::GetOutputIconPath(aUri, icoFile, true);
+    NS_ENSURE_SUCCESS(rv, E_FAIL);
+    rv = icoFile->GetNativePath(path);
+    NS_ENSURE_SUCCESS(rv, E_FAIL);
+
+    shortcutFormatStr = "[InternetShortcut]\r\nURL=%s\r\n"
+                        "IDList=\r\nHotKey=0\r\nIconFile=%s\r\n"
+                        "IconIndex=0\r\n";
+    const int formatLen = strlen(shortcutFormatStr) - 2 * 2; // no %s twice
+    totalLen = formatLen + asciiUrl.Length() +
+               path.Length(); // we don't want a null character on the end
+  }
 
   // create a global memory area and build up the file contents w/in it
   HGLOBAL hGlobalMemory = ::GlobalAlloc(GMEM_SHARE, totalLen);
@@ -1140,8 +1157,13 @@ nsDataObj :: GetFileContentsInternetShortcut ( FORMATETC& aFE, STGMEDIUM& aSTG )
   // terminate strings which reach the maximum size of the buffer. Since we know that the 
   // formatted length here is totalLen, this call to _snprintf will format the string into 
   // the buffer without appending the null character.
-  _snprintf( contents, totalLen, shortcutFormatStr, asciiUrl.get(), path.get() );
-    
+
+  if (!Preferences::GetBool(kShellIconPref, true)) {
+    _snprintf(contents, totalLen, shortcutFormatStr, asciiUrl.get());
+  } else {
+    _snprintf(contents, totalLen, shortcutFormatStr, asciiUrl.get(), path.get());
+  }
+
   ::GlobalUnlock(hGlobalMemory);
   aSTG.hGlobal = hGlobalMemory;
   aSTG.tymed = TYMED_HGLOBAL;
