@@ -34,9 +34,12 @@ const kProxyManual = ["network.proxy.ftp",
 const kExistingWindow = Components.interfaces.nsIBrowserDOMWindow.OPEN_CURRENTWINDOW;
 const kNewWindow = Components.interfaces.nsIBrowserDOMWindow.OPEN_NEWWINDOW;
 const kNewTab = Components.interfaces.nsIBrowserDOMWindow.OPEN_NEWTAB;
+const kExistingTab = Components.interfaces.nsIBrowserDOMWindow.OPEN_SWITCHTAB;
+const kNewPrivate = 5;
 var TAB_DROP_TYPE = "application/x-moz-tabbrowser-tab";
 var gShowBiDi = false;
 var gUtilityBundle = null;
+var gPrivate = null;
 
 function toggleOfflineStatus()
 {
@@ -243,7 +246,7 @@ function goPreferences(paneID)
     lastPrefWindow.focus();
   else
     openDialog("chrome://communicator/content/pref/preferences.xul",
-               "PrefWindow", "chrome,titlebar,dialog=no,resizable",
+               "PrefWindow", "non-private,chrome,titlebar,dialog=no,resizable",
                paneID);
 }
 
@@ -572,7 +575,7 @@ function goClickThrobber(urlPref, aEvent)
 
 function getTopWin()
 {
-  return Services.wm.getMostRecentWindow("navigator:browser");
+  return top.gPrivate || Services.wm.getMostRecentWindow("navigator:browser");
 }
 
 function isRestricted( url )
@@ -968,6 +971,7 @@ function openAsExternal(aURL)
 /**
  * openNewTabWith: opens a new tab with the given URL.
  * openNewWindowWith: opens a new window with the given URL.
+ * openNewPrivateWith: opens a private window with the given URL.
  *
  * @param aURL
  *        The URL to open (as a string).
@@ -991,6 +995,14 @@ function openAsExternal(aURL)
  *        If aDocument is null, then this will be used as the referrer.
  *        There will be no security check.
  */
+function openNewPrivateWith(aURL, aDoc, aPostData, aAllowThirdPartyFixup,
+                            aReferrer)
+{
+  return openNewTabWindowOrExistingWith(kNewPrivate, aURL, aDoc, false,
+                                        aPostData, aAllowThirdPartyFixup,
+                                        aReferrer);
+}
+
 function openNewWindowWith(aURL, aDoc, aPostData, aAllowThirdPartyFixup,
                            aReferrer)
 {
@@ -1035,7 +1047,7 @@ function openNewTabWindowOrExistingWith(aType, aURL, aDoc, aLoadInBackground,
 
   var browserWin;
   // if we're not opening a new window, try and find existing window
-  if (aType != kNewWindow)
+  if (aType != kNewWindow && aType != kNewPrivate)
     browserWin = getTopWin();
 
   // Where appropriate we want to pass the charset of the
@@ -1049,10 +1061,13 @@ function openNewTabWindowOrExistingWith(aType, aURL, aDoc, aLoadInBackground,
 
   // We want to open in a new window or no existing window can be found.
   if (!browserWin) {
+    var features = "private,chrome,all,dialog=no";
+    if (aType != kNewPrivate)
+      features = "non-" + features;
     var charsetArg = null;
     if (originCharset)
       charsetArg = "charset=" + originCharset;
-    return window.openDialog(getBrowserURL(), "_blank", "chrome,all,dialog=no",
+    return window.openDialog(getBrowserURL(), "_blank", features,
                              aURL, charsetArg, referrerURI, aPostData,
                              aAllowThirdPartyFixup);
   }
@@ -1092,12 +1107,13 @@ function BrowserOnCommand(event)
 
   const ot = event.originalTarget;
   const ownerDoc = ot.ownerDocument;
+  const docURI = ownerDoc.documentURI;
+  const buttonID = ot.getAttribute("anonid");
 
   // If the event came from an ssl error page, it is probably either the "Add
   // Exception" or "Get Me Out Of Here" button
-  if (/^about:neterror\?e=nssBadCert/.test(ownerDoc.documentURI) ||
-      /^about:certerror\?/.test(ownerDoc.documentURI)) {
-    if (ot.getAttribute('anonid') == 'exceptionDialogButton') {
+  if (docURI.startsWith("about:certerror?")) {
+    if (buttonID == "exceptionDialogButton") {
       var params = { exceptionAdded : false };
 
       switch (GetIntPref("browser.ssl_override_behavior", 2)) {
@@ -1115,12 +1131,70 @@ function BrowserOnCommand(event)
       if (params.exceptionAdded)
         ownerDoc.location.reload();
     }
-    else if (ot.getAttribute('anonid') == 'getMeOutOfHereButton') {
+    else if (buttonID == "getMeOutOfHereButton") {
       // Redirect them to a known-functioning page, default start page
-      content.location = GetLocalizedStringPref("browser.startup.homepage",
-                                                "about:blank");
+      getMeOutOfHere();
     }
   }
+  else if (docURI.startsWith("about:blocked")) {
+    // The event came from a button on a malware/phishing block page
+    // First check whether it's malware or phishing, so that we can
+    // use the right strings/links
+    let isMalware = /e=malwareBlocked/.test(docURI);
+
+    switch (buttonID) {
+      case "getMeOutOfHereButton":
+        getMeOutOfHere();
+        break;
+
+      case "reportButton":
+        // This is the "Why is this site blocked" button.  For malware,
+        // we can fetch a site-specific report, for phishing, we redirect
+        // to the generic page describing phishing protection.
+
+        if (isMalware) {
+          // Get the stop badware "why is this blocked" report url,
+          // append the current url, and go there.
+          try {
+            let reportURL = Services.urlFormatter.formatURLPref("browser.safebrowsing.malware.reportURL");
+            reportURL += ownerDoc.location.href;
+            loadURI(reportURL);
+          } catch (e) {
+            Components.utils.reportError("Couldn't get malware report URL: " + e);
+          }
+        }
+        else { // It's a phishing site, not malware
+          try {
+            loadURI(Services.urlFormatter.formatURLPref("browser.safebrowsing.warning.infoURL"));
+          } catch (e) {
+            Components.utils.reportError("Couldn't get phishing info URL: " + e);
+          }
+        }
+        break;
+
+      case "ignoreWarningButton":
+        getBrowser().getNotificationBox().ignoreSafeBrowsingWarning(isMalware);
+        break;
+    }
+  }
+}
+
+/**
+ * Re-direct the browser to a known-safe page.  This function is
+ * used when, for example, the user browses to a known malware page
+ * and is presented with about:blocked.  The "Get me out of here!"
+ * button should take the user to the default start page so that even
+ * when their own homepage is infected, we can get them somewhere safe.
+ */
+function getMeOutOfHere() {
+  // Get the start page from the *default* pref branch, not the user's
+  var prefs = Services.prefs.getDefaultBranch(null);
+  var url = "about:blank";
+  try {
+    url = prefs.getComplexValue("browser.startup.homepage",
+                                Components.interfaces.nsIPrefLocalizedString).data;
+  } catch(e) {}
+  loadURI(url);
 }
 
 function popupNotificationMenuShowing(event)
