@@ -3,6 +3,8 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+Components.utils.import("resource://gre/modules/Services.jsm");
+
 function GetNewMessages(selectedFolders, server)
 {
   if (!selectedFolders.length)
@@ -369,20 +371,30 @@ saveAsUrlListener.prototype = {
   }
 };
 
-function SaveAsTemplate(uri)
+function SaveAsTemplate(aUris)
 {
-  if (uri)
+  // For backwards compatibility check if the argument is a string and,
+  // if so, convert to an array.
+  if (typeof aUris == "string")
+    aUris = [aUris];
+
+  var num = aUris.length;
+  if (!num)
+    return;
+
+  for (let i = 0; i < num; i++)
   {
+    let uri = aUris[i];
     var hdr = messenger.msgHdrFromURI(uri);
     var identity = GetIdentityForHeader(hdr, Components.interfaces.nsIMsgCompType.Template);
     var templates = MailUtils.getFolderForURI(identity.stationeryFolder, false);
     if (!templates.parent)
     {
       templates.setFlag(Components.interfaces.nsMsgFolderFlags.Templates);
-      let isImap = templates.server.type == "imap";
+      let isAsync = templates.server.protocolInfo.foldersCreatedAsync;
       templates.createStorageIfMissing(new saveAsUrlListener(uri, identity));
-      if (isImap)
-        return;
+      if (isAsync)
+        continue;
     }
     messenger.saveAs(uri, false, identity, null);
   }
@@ -448,9 +460,7 @@ function confirmToProceed(commandName)
   // default to ask user if the pref is not set
   var dontAskAgain = false;
   try {
-    var pref = Components.classes["@mozilla.org/preferences-service;1"]
-                        .getService(Components.interfaces.nsIPrefBranch);
-    dontAskAgain = pref.getBoolPref(kDontAskAgainPref);
+    dontAskAgain = Services.prefs.getBoolPref(kDontAskAgainPref);
   } catch (ex) {}
 
   if (!dontAskAgain)
@@ -466,7 +476,7 @@ function confirmToProceed(commandName)
                    checkbox);
     try {
       if (checkbox.value)
-        pref.setBoolPref(kDontAskAgainPref, true);
+        Services.prefs.setBoolPref(kDontAskAgainPref, true);
     } catch (ex) {}
 
     if (choice != 0)
