@@ -153,7 +153,7 @@ function HandleColumnClick(columnID)
   var dbview = GetDBView();
   var simpleColumns = false;
   try {
-    simpleColumns = !pref.getBoolPref("mailnews.thread_pane_column_unthreads");
+    simpleColumns = !Services.prefs.getBoolPref("mailnews.thread_pane_column_unthreads");
   }
   catch (ex) {
   }
@@ -456,7 +456,7 @@ function ThreadPaneOnLoad()
     return;
 
   tree.addEventListener("mousedown",TreeOnMouseDown,true);
-  var delay = pref.getIntPref("mailnews.threadpane_select_delay");
+  var delay = Services.prefs.getIntPref("mailnews.threadpane_select_delay");
   document.getElementById("threadTree")._selectDelay = delay;
 }
 
